from django.core.exceptions import ValidationError
from django.db import transaction, IntegrityError
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework.permissions import IsAuthenticated
from rest_framework.status import HTTP_201_CREATED, HTTP_404_NOT_FOUND, HTTP_403_FORBIDDEN, HTTP_400_BAD_REQUEST
from rest_framework.views import APIView
from rest_framework import generics, authentication
from rest_framework.response import Response
from rest_framework import status
from django.contrib.auth.password_validation import validate_password
from rest_framework.authtoken.models import Token
from axes.decorators import axes_dispatch
from cryptography.hazmat.primitives import serialization

from ..products.serializers import ProductSerializer
from ..encryption.encryption import assymetric_encrypt, decrypt, decrypt_request_body, symmetric_encrypt
from ..orders.models import Order, OrderItem
from ..orders.serializers import OrderSerializer
from ..products.models import Product
from ..access_control.models import CustomUser
from ..access_control.serializers import UserRegistrationSerializer, UserLoginSerializer
from django.contrib.auth import authenticate
from django_backend.settings import PRIVATE_KEY

from ..reviews.models import ProductReview
from ..reviews.serializers import ProductReviewSerializer


class ProductListEncrypted(APIView):
    authentication_classes = [authentication.TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        if product_name_substring := request.data.get("product_name_substring", ''):
            # Filter products by name substring (case-insensitive)
            products = Product.objects.filter(name__icontains=product_name_substring)
        else:
            products = Product.objects.all()
        serializer = ProductSerializer(products, many=True)
        try:
            public_key = serialization.load_pem_public_key(request.data.get("public_key", "").encode('utf-8'))
        except ValueError:
            return Response({'detail': 'Public key is not correct.'}, status=status.HTTP_400_BAD_REQUEST)
        encrypted_data = assymetric_encrypt(data=serializer.data, public_key=public_key)
        return Response(encrypted_data)


class ProductListDecrypt(APIView):
    authentication_classes = ()

    def post(self, request):
        encrypted_data = request.data.get('encrypted_data')
        encrypted_key = request.data.get('encrypted_key')
        decrypted_data = decrypt(encrypted_data, encrypted_key)
        if decrypted_data.get('error'):
            return Response(decrypted_data, status=422)
        return Response(decrypted_data)


class ProvidePublicKeyView(APIView):
    authentication_classes = ()

    def get(self, request):
        public_key = PRIVATE_KEY.public_key().public_bytes(encoding=serialization.Encoding.PEM,
                                                           format=serialization.PublicFormat.SubjectPublicKeyInfo)
        return Response({'public_key': public_key}, status=HTTP_201_CREATED)


@method_decorator(csrf_exempt, name='dispatch')
class UserRegistrationView(generics.CreateAPIView):
    queryset = CustomUser.objects.all()
    serializer_class = UserRegistrationSerializer
    authentication_classes = ()

    def create(self, request, *args, **kwargs):
        decrypted_data = decrypt_request_body(request=request)
        if not decrypted_data.get('data'):
            return decrypted_data

        serializer = self.get_serializer(data=decrypted_data.get('data'))
        if serializer.is_valid():
            try:
                validate_password(decrypted_data.get('data').get("password"), serializer)
            except Exception as e:
                return Response(e.__dict__, status=status.HTTP_400_BAD_REQUEST)
            user = serializer.save()
            user.set_password(decrypted_data.get('data').get("password"))
            user.save()
            return Response({"status": "Registration successful."}, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


@method_decorator(axes_dispatch, name='dispatch')
class UserLoginView(generics.CreateAPIView):
    serializer_class = UserLoginSerializer
    authentication_classes = ()

    @csrf_exempt
    def create(self, request, *args, **kwargs):
        decrypted_data = decrypt_request_body(request=request)
        if not decrypted_data.get('data'):
            return decrypted_data

        email = decrypted_data.get('data').get('email')
        password = decrypted_data.get('data').get('password')
        user = authenticate(username=email, password=password, request=request)
        if user is not None:
            if user.is_active:
                token, created = Token.objects.get_or_create(user=user)
                encrypted_response = symmetric_encrypt(data={'token': token.key,
                                                             'user_information': UserRegistrationSerializer(user).data},
                                                       symmetric_key=decrypted_data.get('key'))
                return Response({"status": "Login successful.", 'data': encrypted_response}, status=status.HTTP_200_OK)
            else:
                return Response({'detail': 'User is not active.'}, status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({'detail': 'Invalid credentials.'}, status=status.HTTP_401_UNAUTHORIZED)


class UserLogoutView(APIView):
    authentication_classes = ()

    def post(self, request):
        # Get the token from the request data
        token_key = request.data.get('token')

        try:
            # Retrieve the token and the associated user
            token = Token.objects.get(key=token_key)
            user = token.user

            # Delete the token to log the user out
            token.delete()

            # Log the user out (optional if using session-based authentication)
            request.auth = None

            return Response({'detail': 'Logged out successfully.'}, status=status.HTTP_200_OK)

        except Token.DoesNotExist:
            return Response({'detail': 'Invalid token.'}, status=status.HTTP_400_BAD_REQUEST)


class CreateOrderView(APIView):
    authentication_classes = [authentication.TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        decrypted_data = decrypt_request_body(request=request)
        if not decrypted_data.get('data'):
            return decrypted_data

        order_data = decrypted_data.get('data').get('order_data', [])
        if not order_data:
            return JsonResponse({'detail': 'Order cannot be empty.'}, status=HTTP_400_BAD_REQUEST)

        with transaction.atomic():
            try:
                product_quantity_changes = check_products_availability(order_data=order_data)
                update_products_quantities(product_quantities=product_quantity_changes)
                order = create_order(order_data=order_data, user=request.user)
                serializer = OrderSerializer(order)
                encrypted_data = symmetric_encrypt(data=serializer.data,
                                                   symmetric_key=decrypted_data.get('symmetric_key', None))
                return Response({'data': encrypted_data}, status=status.HTTP_201_CREATED)

            except Exception as e:
                return Response({'detail': str(e)}, status=status.HTTP_400_BAD_REQUEST)


class GetOrdersView(APIView):
    authentication_classes = [authentication.TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        try:
            public_key = serialization.load_pem_public_key(request.data.get("public_key", "").encode('utf-8'))
        except ValueError:
            return Response({'detail': 'Public key is not correct.'}, status=status.HTTP_400_BAD_REQUEST)
        user_orders = OrderSerializer(Order.objects.filter(user=request.user), many=True)
        encrypted_data = assymetric_encrypt(data=user_orders.data, public_key=public_key)
        return Response(encrypted_data, status=status.HTTP_200_OK)


class GetOrderView(APIView):
    authentication_classes = [authentication.TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request, order_id):
        try:
            public_key = serialization.load_pem_public_key(request.data.get("public_key", "").encode('utf-8'))
        except ValueError:
            return Response({'detail': 'Public key is not correct.'}, status=status.HTTP_400_BAD_REQUEST)
        try:
            # Retrieve the order by its ID, filtering by the user
            order = Order.objects.get(id=order_id, user=request.user)
            order_serializer = OrderSerializer(order)
            encrypted_data = assymetric_encrypt(data=order_serializer.data, public_key=public_key)
            return Response(encrypted_data, status=status.HTTP_200_OK)
        except Order.DoesNotExist:
            return Response({'detail': 'Order not found.'},
                            status=status.HTTP_404_NOT_FOUND)


class CreateProductReviewView(APIView):
    authentication_classes = [authentication.TokenAuthentication]
    permission_classes = [IsAuthenticated]

    def post(self, request):
        decrypted_data = decrypt_request_body(request=request)
        if not decrypted_data.get('data'):
            return decrypted_data

        review_data = decrypted_data.get('data').get('review_data', {})
        try:
            product = Product.objects.get(pk=review_data.get("product_id", -1))
        except Product.DoesNotExist:
            return JsonResponse({'detail': f'Product with id {review_data.get("product_id", "None")} not found.'},
                                status=HTTP_404_NOT_FOUND)

        # Check if the user has an order for this product
        if not OrderItem.objects.filter(order__user=request.user, product=product).exists():
            return JsonResponse({'detail': 'You can only leave a review for products you have ordered.'},
                                status=HTTP_403_FORBIDDEN)

        try:
            # Create the review
            review = ProductReview(
                user=request.user,
                product=product,
                rating=review_data.get('rating', -1),
                review=review_data.get('review')
            )
            review.full_clean()
            review.save()
            serializer = ProductReviewSerializer(instance=review)
            encrypted_data = symmetric_encrypt(data=serializer.data, symmetric_key=decrypted_data.get('key'))
            return Response({'detail': 'Review created successfully.', 'data': encrypted_data},
                                status=HTTP_201_CREATED)
        except IntegrityError:
            return JsonResponse({'detail': 'Review for this product has been already created by this user.'},
                                status=HTTP_403_FORBIDDEN)
        except ValidationError as e:
            return JsonResponse({'detail': 'Validation Error', **e.message_dict}, status=HTTP_400_BAD_REQUEST)


def lockout(request, credentials, *args, **kwargs):
    return JsonResponse({"detail": "Locked out due to too many login failures", "cooldown": "30 seconds"},
                        status=429)


def check_products_availability(order_data):
    product_quantity_changes = {}

    for item in order_data:
        product_id = item['product_id']
        quantity = item['quantity']

        try:
            product = Product.objects.get(id=product_id)
        except Product.DoesNotExist as e:
            raise Exception(f'Product with ID {product_id} does not exist.')

        if product.stock < quantity:
            raise Exception(f'Not enough stock for product "{product.name}".')

        product_quantity_changes[product_id] = quantity
    return product_quantity_changes


def update_products_quantities(product_quantities):
    for product_id, quantity in product_quantities.items():
        product = Product.objects.get(id=product_id)
        product.stock -= quantity
        product.save()


def create_order(order_data, user):
    order = Order.objects.create(user=user)
    for item in order_data:
        product = Product.objects.get(id=item['product_id'])
        OrderItem.objects.create(product=product, quantity=item['quantity'], order=order)
    return order
