import {createContext, ReactNode, useContext, useState} from "react";
import {ShoppingCart} from "../components/ShoppingCart.tsx";
import {useSessionStorage} from "../hooks/useSessionStorage.tsx";

type ShoppingCartProviderProps = {
    children: ReactNode
}

type ShoppingCartContext = {
    openCart: () => void,
    closeCart: () => void,
    getItemQuantity: (id: number) => number,
    increaseCartQuantity: (id: number) => void,
    decreaseCartQuantity: (id: number) => void,
    cartQuantity: number,
    cartItems: CartItem[],
    emptyCart: () => void,
    setDecryptedData: (newData: []) => void,
    jsonData: never[],
    setfilteredData: (newData: []) => void,
    filteredData: never[],
    isDecrypted: boolean,
    setDecrypted: (bol: boolean) => void,
    isFiltered: boolean,
    setFiltered: (bol: boolean) => void
}

type CartItem = {
    id: number,
    quantity: number
}

const ShoppingCartContext = createContext({} as ShoppingCartContext);

export function useShoppingCart() {
    return useContext(ShoppingCartContext);
}

export function ShoppingCartProvider({children}: ShoppingCartProviderProps) {
    const [isOpen, setIsOpen] = useState(false);
    const [cartItems, setCartItems] = useSessionStorage<CartItem[]>("shopping-cart", []);
    const [jsonData, setJsonData] = useState([]);
    const [filteredData, setfilteredJsonData] = useState([]);
    const [isDecrypt, setDecrypt] = useState(false);
    const [isFiltered, setFilter] = useState(false);

    const cartQuantity = cartItems.reduce((quantity, item) => item.quantity + quantity, 0)

    const openCart = () => setIsOpen(true);
    const closeCart = () => setIsOpen(false);

    const isDecrypted = isDecrypt;

    const setDecrypted = (bol: boolean) => setDecrypt(bol);

    const setFiltered = (bol: boolean) => setFilter(bol);


    const setDecryptedData = (newData: []) => setJsonData(newData);

    const setfilteredData = (newData: []) => setfilteredJsonData(newData);

    function emptyCart() {
        setCartItems([])
    }


    function getItemQuantity(id: number) {
        return cartItems.find(item => item.id === id)?.quantity || 0
    }

    function increaseCartQuantity(id: number) {
        setCartItems(currItems => {
            if (currItems.find(item => item.id === id) == null) {
                return [...currItems, {id, quantity: 1}]
            } else {
                return currItems.map(item => {
                    if (item.id === id) {
                        return {...item, quantity: item.quantity + 1}
                    } else {
                        return item
                    }
                })
            }
        })
    }

    function decreaseCartQuantity(id: number) {
        setCartItems(currItems => {
            if (currItems.find(item => item.id === id)?.quantity === 1) {
                return currItems.filter(item => item.id !== id)
            } else {
                return currItems.map(item => {
                    if (item.id === id) {
                        return {...item, quantity: item.quantity - 1}
                    } else {
                        return item
                    }
                })
            }
        })
    }

    return (
        <ShoppingCartContext.Provider
            value={{getItemQuantity, increaseCartQuantity, decreaseCartQuantity, cartItems, emptyCart, cartQuantity, openCart, closeCart, setDecryptedData, jsonData, isDecrypted, setDecrypted, setfilteredData, filteredData, setFiltered, isFiltered}}>
            {children}
            <ShoppingCart isOpen={isOpen}/>
        </ShoppingCartContext.Provider>
    )
}