import io
import json
import os
from datetime import date, datetime, time
from typing import Annotated, List, Optional

from argon2 import PasswordHasher
from argon2.exceptions import VerifyMismatchError
from borb.pdf import (PDF, Document, FixedColumnWidthTable, Page, Paragraph,
                      SingleColumnLayout)
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from fastapi import (FastAPI, File, Form, HTTPException, Request, Response,
                     UploadFile, status)
from fastapi.params import Depends
from jose import JWTError, jwt
from slowapi import Limiter, _rate_limit_exceeded_handler
from slowapi.errors import RateLimitExceeded
from slowapi.util import get_remote_address
from sqlmodel import (Field, Relationship, Session, SQLModel, create_engine,
                      func, select)
from starlette.status import HTTP_200_OK, HTTP_400_BAD_REQUEST

from app.auth import (ALGORITHM, SECRET, OAuth2MailPasswordRequestForm,
                      OAuth2PasswordCookie, create_access_token)


class User(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    first_name: str
    last_name: str
    mail: str
    password: str
    public_key: bytes | None = None
    invoices: List["Invoice"] = Relationship(back_populates="user_")


class Route(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    ttype: str = Field(alias="type", sa_column_kwargs={"name": "type"})
    city: str
    departure_city: str
    arrival_city: str
    departure_date: date
    departure_time: time
    arrival_date: date
    arrival_time: time
    price: float


class Invoice(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    user: int = Field(
        default=None,
        foreign_key="user.id",
    )
    invoice: bytes
    paid: bool
    created_at: datetime
    user_: User = Relationship(back_populates="invoices")


limiter = Limiter(key_func=get_remote_address)
oauth2_scheme = OAuth2PasswordCookie(tokenUrl="login")
app = FastAPI()  # FastAPI instance
app.state.limiter = limiter
app.add_exception_handler(RateLimitExceeded, _rate_limit_exceeded_handler)
db_engine = create_engine(
    "postgresql://postgres:postgres@db:5432"
)  # DB connection instance


def get_session():
    with Session(db_engine) as session:
        yield session


@app.post("/register")
async def register(
    first_name: Annotated[str, Form()],
    last_name: Annotated[str, Form()],
    mail: Annotated[str, Form()],
    password: Annotated[str, Form()],
    session: Annotated[Session, Depends(get_session)],
    public_key: Annotated[UploadFile | None, File()] = None,
):
    first_name = first_name.strip()
    last_name = last_name.strip()
    mail = mail.strip().lower()

    if not first_name or not last_name or not mail or not password:
        raise HTTPException(status_code=400, detail="Invalid registration form")

    if get_user(mail) is not None:
        raise HTTPException(status_code=400, detail="E-mail already in use")

    ph = PasswordHasher()
    password = ph.hash(password)

    user = User(
        first_name=first_name,
        last_name=last_name,
        mail=mail,
        password=password,
    )

    if public_key is not None:
        user.public_key = await public_key.read()

    session.add(user)

    session.commit()

    return Response(status_code=201)


@app.post("/route")
async def route(
    session: Annotated[Session, Depends(get_session)],
    ttype: Annotated[str | None, Form(alias="type")] = "",
    arrival_city: Annotated[str | None, Form()] = "",
    departure_date: Annotated[str | None, Form()] = "",
    arrival_date: Annotated[str | None, Form()] = "",
    price: Annotated[str | None, Form()] = "",
    city: Annotated[str | None, Form()] = "",
):
    stmt = select(Route)

    if ttype := ttype.strip():
        stmt = stmt.where(func.lower(Route.ttype) == func.lower(ttype))
    if arrival_city := arrival_city.strip():
        stmt = stmt.where(func.lower(Route.arrival_city) == func.lower(arrival_city))
    if departure_date := departure_date.strip():
        stmt = stmt.where(
            Route.departure_date == datetime.strptime(departure_date, "%Y-%m-%d").date()
        )
    if arrival_date := arrival_date.strip():
        stmt = stmt.where(
            Route.arrival_date == datetime.strptime(arrival_date, "%Y-%m-%d").date()
        )
    if price := price.strip():
        stmt = stmt.where(Route.price <= price)
    if city := city.strip():
        stmt = stmt.where(func.lower(Route.city) == func.lower(city))

    stmt = stmt.limit(100)

    return session.exec(stmt).all()


@app.post("/login")
@limiter.limit("5/minute")
def login(
    form_data: Annotated[OAuth2MailPasswordRequestForm, Depends()],
    session: Annotated[Session, Depends(get_session)],
    request: Request,
):
    mail = form_data.mail
    password = form_data.password
    user = get_user(mail)

    if user is None:
        raise HTTPException(status_code=400, detail="E-mail or password is not correct")

    ph = PasswordHasher()

    try:
        ph.verify(user.password, password)
    except VerifyMismatchError as _e:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="E-mail or password is not correct",
        )

    if ph.check_needs_rehash(user.password):
        user.password = ph.hash(password)
        session.add(user)
        session.commit()

    access_token = create_access_token(data={"sub": mail})
    resp = Response(status_code=status.HTTP_200_OK)
    resp.set_cookie(
        "access_token", access_token, secure=True, httponly=True, samesite="none"
    )

    return resp


@app.post("/logout")
def logout():
    resp = Response(status_code=status.HTTP_200_OK)
    resp.set_cookie("access_token", "", secure=True, httponly=True, samesite="none")

    return resp


def get_user(mail: str) -> User | None:
    with Session(db_engine) as session:
        stmt = select(User).where(func.lower(User.mail) == func.lower(mail))
        return session.exec(stmt).first()


async def get_current_user(token: Annotated[str, Depends(oauth2_scheme)]):
    exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Could not validate access_token",
    )

    try:
        payload = jwt.decode(token, SECRET, algorithms=[ALGORITHM])
        mail = payload.get("sub")

        if mail is None:
            raise exception
    except JWTError:
        raise exception

    user = get_user(mail)

    if user is None:
        raise exception

    return user


@app.post("/public_key")
async def set_public_key(
    public_key: Annotated[UploadFile, File()],
    current_user: Annotated[User, Depends(get_current_user)],
    session: Annotated[Session, Depends(get_session)],
):
    current_user.public_key = await public_key.read()
    session.add(current_user)
    session.commit()

    return Response(status_code=HTTP_200_OK)


@app.get("/invoices")
async def invoices(
    current_user: Annotated[User, Depends(get_current_user)],
    session: Annotated[Session, Depends(get_session)],
):
    user = session.exec(select(User).where(User.id == current_user.id)).one()
    return [
        {"id": x.id, "paid": x.paid, "created_at": x.created_at} for x in user.invoices
    ]


@app.get("/invoice/{invoice_id}")
async def invoice(
    invoice_id: int,
    current_user: Annotated[User, Depends(get_current_user)],
    session: Annotated[Session, Depends(get_session)],
):
    user = session.exec(select(User).where(User.id == current_user.id)).one()
    invoice = [x for x in user.invoices if x.id == invoice_id]

    if not invoice:
        raise HTTPException(status_code=HTTP_400_BAD_REQUEST)

    return Response(
        encrypt(invoice[0].invoice, current_user.public_key),
        media_type="application/octet-stream",
    )


@app.post("/purchase/{route_id}")
async def purchase(
    route_id: int,
    current_user: Annotated[User, Depends(get_current_user)],
    session: Annotated[Session, Depends(get_session)],
):
    if current_user.public_key is None:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST,
            detail="User hasn't set up his public key",
        )

    route = session.exec(select(Route).where(Route.id == route_id)).one()
    pdf = Document()
    page = Page()
    pdf.add_page(page)
    page_layout = SingleColumnLayout(page)
    table = FixedColumnWidthTable(number_of_rows=7, number_of_columns=2)
    table.add(Paragraph("Name"))
    table.add(Paragraph(f"{current_user.first_name} {current_user.last_name}"))
    table.add(Paragraph("E-mail"))
    table.add(Paragraph(f"{current_user.mail}"))
    table.add(Paragraph("From"))
    table.add(Paragraph(f"{route.city}"))
    table.add(Paragraph(""))
    table.add(Paragraph(f"{route.departure_date} {route.departure_time}"))
    table.add(Paragraph("To"))
    table.add(Paragraph(f"{route.arrival_city}"))
    table.add(Paragraph(""))
    table.add(Paragraph(f"{route.arrival_date} {route.arrival_time}"))
    table.add(Paragraph("Total price"))
    table.add(Paragraph(f"{route.price}€"))
    table.set_padding_on_all_cells(2, 2, 2, 2)
    table.no_borders()
    page_layout.add(table)

    data = io.BytesIO()
    PDF.dumps(data, pdf)
    data.seek(0)
    data = data.read()

    invoice = Invoice(
        user=current_user.id,
        invoice=data,
        paid=False,
        created_at=datetime.now(),
    )

    session.add(invoice)
    session.commit()

    return Response(
        encrypt(data, current_user.public_key),
        media_type="application/octet-stream",
    )


@app.post("/pay/{invoice_id}")
async def pay(
    invoice_id: int,
    current_user: Annotated[User, Depends(get_current_user)],
    session: Annotated[Session, Depends(get_session)],
):
    invoice = session.exec(select(Invoice).where(Invoice.id == invoice_id)).one()

    if invoice is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Invoice not found",
        )

        # Check if the current user is the owner of the invoice
    if invoice.user != current_user.id:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You are not authorized to pay this invoice",
        )

        # Check if the invoice has already been paid
    if invoice.paid:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Invoice has already been paid",
        )

    invoice.paid = True
    session.add(invoice)
    session.commit()

    return Response(status_code=status.HTTP_200_OK)


@app.post("/data")
async def get_data(current_user: Annotated[User, Depends(get_current_user)]):
    if current_user.public_key is None:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST,
            detail="User hasn't set up his public key",
        )

    data = json.dumps(
        {
            "first_name": current_user.first_name,
            "last_name": current_user.last_name,
            "mail": current_user.mail,
        },
        sort_keys=True,
        default=str,
    ).encode("utf-8")

    ret = encrypt(data, current_user.public_key)

    return Response(
        ret,
        media_type="application/octet-stream",
    )


def encrypt(data, public_key):
    peer_public_key = serialization.load_pem_public_key(public_key)

    key = AESGCM.generate_key(bit_length=128)
    encrypted_key = peer_public_key.encrypt(
        key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )

    aesgcm = AESGCM(key)
    nonce = os.urandom(12)
    cipher_text = aesgcm.encrypt(nonce, data, None)

    return encrypted_key + nonce + cipher_text
