// Importing necessary modules
import { Component, OnInit, ViewChild } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Router } from '@angular/router';
import { AuthService } from '../../auth.service';
import { lastValueFrom } from 'rxjs';
import { CustomModalComponent } from 'src/app/custom-modal/custom-modal.component';
import { FormGroup, FormControl } from '@angular/forms';

// Defining the component
@Component({
  selector: 'app-login-page',
  templateUrl: './login-page.component.html',
  styleUrls: ['./login-page.component.css']
})
export class LoginPageComponent implements OnInit {

  // Declaring variables
  public firstName: string;
  public lastName: string;
  public mail: string;
  public password: string;
  public publicKey: File | null = null;
  showLogin = true;
  private commonPasswords: Set<string> = new Set();

  @ViewChild(CustomModalComponent)
  customModal: CustomModalComponent = new CustomModalComponent;

  

  // Constructor
  constructor(private http: HttpClient, private router: Router, private authService: AuthService) {
    this.firstName = '';
    this.lastName = '';
    this.mail = '';
    this.password = '';
    this.publicKey = null;
    this.loadCommonPasswords();
  }

  async loadCommonPasswords() {
    try {
      const response = await lastValueFrom(
        this.http.get('../../assets/common_passwords.txt', { responseType: 'text' })
      );
      this.commonPasswords = new Set(response.split('\n'));
    } catch (error) {
      //console.error('Failed to load common passwords:', error);
      this.customModal.open("Chyba pri načítaní hesiel", "Uistite sa, že máte pripojenie na internet.");
    }
  }

  // Function to check if password is complex enough
  isPasswordComplexEnough(password: string): boolean {
    const minLength = 8;
    const hasUppercase = /[A-Z]/.test(password);
    const hasLowercase = /[a-z]/.test(password);
    const hasNumber = /\d/.test(password);
    const hasSpecialChar = /[!@#\$%\^\&*\)\(+=._-]/.test(password);
    const isLongEnough = password.length >= minLength;
  
    return hasUppercase && hasLowercase && hasNumber && hasSpecialChar && isLongEnough;
  }

  isNameValid(name: string): boolean {
    return name.length >= 3;
  }

  isEmailValid(email: string): boolean {
    const emailRegex = /^[a-zA-Z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$/;
    return emailRegex.test(email);
  }
  
  

  isPasswordCommon(password: string): boolean {
    return this.commonPasswords.has(password);
  }

  loginForm: FormGroup | undefined;

  ngOnInit(): void {
    this.loginForm = new FormGroup({
      mail: new FormControl(''),
      password: new FormControl('')
    });
  }

  // Function to handle registration
  onRegistration() {
    const formData: FormData = new FormData();
    
    if (!this.isPasswordComplexEnough(this.password)) {
      this.customModal.open("Heslo nie je dostatočne zložité", "Uistite sa, že vaše heslo má aspoň 8 znakov a obsahuje aspoň jedno veľké písmeno, jedno malé písmeno, jedno číslo a jeden špeciálny znak.");
      return;
    }

    if (this.isPasswordCommon(this.password)) {
      this.customModal.open("Heslo je príliš bežné", "Uistite sa, že vaše heslo nie je príliš bežné.");
      return;
    }

    if (!this.isNameValid(this.firstName)) {
      this.customModal.open("Meno je príliš krátke", "Uistite sa, že meno má minimálnu dĺžku znakov 3.");
      return;
    }

    if (!this.isNameValid(this.lastName)) {
      this.customModal.open("Priezvisko je príliš krátke", "Uistite sa, že priezvisko má minimálnu dĺžku znakov 3.");
      return;
    }

    if (!this.isEmailValid(this.mail)) {
      this.customModal.open("Email je neplatný", "Uistite sa, že ste zadali platný email.");
      return;
    }

    formData.append('first_name', this.firstName);
    formData.append('last_name', this.lastName);
    formData.append('mail', this.mail);
    formData.append('password', this.password);

    // Sending registration data to server
    this.http.post('api/register', formData, { observe: 'response' }).subscribe(
      (response: any) => {
        if (response.status === 201) {
          this.showLogin = true;
        }
      },
      (error: any) => {
        this.customModal.open("Chyba pri registrácii", "Uistite sa, že ste zadali správne údaje.");
      }
    );
  }

  // Function to handle file selection
  onFileSelected(event: Event): void {
    const input = event.target as HTMLInputElement;
    if (input.files) {
      this.publicKey = input.files[0];
    }
  }

  // Function to handle login
  onLogin() {
    const formData: FormData = new FormData();
    formData.append('mail', this.mail);
    formData.append('password', this.password);

    // Sending login data to server
    this.http.post('api/login', formData, { observe: 'response' }).subscribe(
      (response: any) => {
        if (response.status === 200) {
          this.authService.login();
          this.router.navigate(['/profile']);
        }
      },
      (error: any) => {
        this.customModal.open("Chyba pri prihlásení", "Uistite sa, že ste zadali správne prihlasovacie údaje.");
      }
    );
  }
}
