import { Component, OnInit, ViewChild } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { CustomModalComponent } from 'src/app/custom-modal/custom-modal.component';
import { Router } from '@angular/router';

/**
 * ProfileComponent class that handles the user's profile page.
 */
@Component({
  selector: 'app-profile',
  templateUrl: './profile.component.html',
  styleUrls: ['./profile.component.css']
})
export class ProfileComponent implements OnInit {
  selectedTab: string = 'keys';
  firstName: string = '';
  publicKey: File | null = null;
  public privateKeyContent: string | ArrayBuffer | null | undefined;
  public publicKeyContent: string | ArrayBuffer | null | undefined;

  invoices: any[] = [];

  /**
   * ViewChild decorator that gets a reference to the CustomModalComponent.
   */
  @ViewChild(CustomModalComponent)
  customModal: CustomModalComponent = new CustomModalComponent;

  /**
   * Constructor that injects the HttpClient service.
   * @param http The HttpClient service.
   */
  constructor(private http: HttpClient, private router: Router) {}

  /**
   * OnInit lifecycle hook that initializes the component.
   */
  ngOnInit() {
    this.fetchInvoices();
  }

  fetchInvoices() {
    this.http.get<any[]>('api/invoices').subscribe({
      next: (response) => {
        this.invoices = response;
      },
      error: (error) => {
        //console.error('Error fetching invoices', error);
        this.customModal.open("Chyba","Nepodarilo sa načítať faktúry");
      }
    });
  }

  stringToBuffer(str: string): ArrayBuffer {
    const encoder = new TextEncoder();
    return encoder.encode(str).buffer;
  }

  downloadInvoice(invoiceId: number) {
    this.http.get(`api/invoice/${invoiceId}`, { responseType: 'blob' })
      .subscribe({
        next: async (response) => {
          const encryptedUserData = await response.arrayBuffer();
          const encryptedKeyLength = 256;
          const nonceSizeInBytes = 12;
          const encryptedKey = encryptedUserData.slice(0, encryptedKeyLength);
          const nonce = encryptedUserData.slice(encryptedKeyLength, encryptedKeyLength + nonceSizeInBytes);
          const encryptedData = encryptedUserData.slice(encryptedKeyLength + nonceSizeInBytes);
          if (this.privateKeyContent) {
            const privateKeyContent = this.privateKeyContent.toString();
            const decryptedData = await this.decryptDataPDF(encryptedData, encryptedKey, nonce, privateKeyContent);
            const fileBlob = new Blob([decryptedData], { type: 'application/pdf' });
            this.downloadFile(fileBlob);
          } else {
            this.customModal.open("Chyba","Najprv musíte nahrať súkromný kľúč. Jednoducho si vygenerujte kľúče pomocou tlačidla 'Generovať kľúčový pár'");
          }
        },
          error: (error) => {
            //console.error('Error downloading invoice', error);
            this.customModal.open("Chyba","Z neznámeho dôvodu sa nepodarilo stiahnuť faktúru");
          }
      });
  }

  downloadFile(data: Blob): void {
    const url = window.URL.createObjectURL(data);
    const a = document.createElement('a');
    a.href = url;
    a.download = 'invoice.pdf';
    document.body.appendChild(a);
    a.click();
    window.URL.revokeObjectURL(url);
    a.remove();
  }

  payInvoice(invoice: any) {
    this.router.navigate(['/payment', { invoiceId: invoice.id }]);
  }


  /**
   * Method that selects the active tab.
   * @param tab The tab to select.
   */
  selectTab(tab: string): void {
    this.selectedTab = tab;
  }

  /**
   * Method that handles the selection of a public key file.
   * @param event The event object.
   */
  onPublicKeySelected(event: Event) {
    const input = event.target as HTMLInputElement;
    if (input.files && input.files.length) {
      this.publicKey = input.files[0];
      const formData = new FormData();
      formData.append('public_key', this.publicKey, this.publicKey.name);

      const reader = new FileReader();
      reader.onload = (e) => {
        this.publicKeyContent = reader.result;
      };
      reader.readAsText(this.publicKey);
      
      this.http.post('api/public_key', formData).subscribe(
        response => {
          this.customModal.open("Úspešné nahranie","Verejný kľúč bol úspešne nahraný");
        },
        error => {
          this.customModal.open("Chyba","Verejný kľúč sa nepodarilo nahrať");
        }
      );
    } else {
      this.publicKey = null;
      this.publicKeyContent = '';
    }
  }

  /**
  * Method that handles the selection of a private key file from the user's file system.
  * @param event - The DOM event object that triggered the file selection.
  */
  onPrivateKeyFileSelected(event: Event): void {
    const input = event.target as HTMLInputElement;
    if (!input.files?.length) {
      return;
    }
    const fileReader = new FileReader();
    fileReader.onload = async (e) => {
      const privateKeyText = fileReader.result as string;
      this.privateKeyContent = privateKeyText;
      // START OF DECRYPTION
      this.getEncryptedUserData().subscribe(async (response: ArrayBuffer) => {
        const encryptedKeyLength = 256;
        const encryptedKey = response.slice(0, encryptedKeyLength);
        const nonceSizeInBytes = 12;
        const nonce = response.slice(encryptedKeyLength, encryptedKeyLength + nonceSizeInBytes);
        const encryptedData = response.slice(encryptedKeyLength + nonce.byteLength);
        const decryptedData = await this.decryptData(encryptedData, encryptedKey, nonce, privateKeyText);
        // END OF DECRYPTION

        const decryptedObject = JSON.parse(decryptedData);
        this.firstName = decryptedObject.first_name;
        

      }, (error: any) => {
        //console.error('Error fetching encrypted data:', error);
        this.customModal.open("Chyba","Nepodarilo sa načítať dáta");
      });
    };
    fileReader.readAsText(input.files[0]);
  }

  /**
   * Method that gets the encrypted user data.
   * @returns An Observable of the encrypted user data.
   */
  getEncryptedUserData(): Observable<ArrayBuffer> {
    const body = {};
    const httpOptions = {
      responseType: 'arraybuffer' as 'json',
    };
  
    return this.http.post<ArrayBuffer>('api/data', body, httpOptions);
  }

  /**
   * Method that decrypts the encrypted data.
   * @param encryptedData The encrypted data.
   * @param encryptedSessionKey The encrypted session key.
   * @param nonce The nonce.
   * @param privateKeyText The private key text.
   * @returns A Promise of the decrypted data.
   */
  private async decryptData(
    encryptedData: ArrayBuffer,
    encryptedSessionKey: ArrayBuffer,
    nonce: ArrayBuffer,
    privateKeyText: string
  ): Promise<string> {
    try {
      const privateKey = await crypto.subtle.importKey(
        "pkcs8",
        this.convertPrivateKeyToCryptoKey(privateKeyText),
        {
          name: "RSA-OAEP",
          hash: {name: "SHA-256"},
        },
        false,
        ["decrypt"]
      );
      const sessionKey = await crypto.subtle.decrypt(
        {
          name: "RSA-OAEP",
        },
        privateKey,
        encryptedSessionKey
      );
      const sessionKeyCrypto = await crypto.subtle.importKey(
        "raw",
        sessionKey,
        {
          name: "AES-GCM",
          length: 128
        },
        false,
        ["decrypt"]
      );
      const decryptedData = await crypto.subtle.decrypt(
        {
          name: "AES-GCM",
          iv: nonce,
          tagLength: 128
        },
        sessionKeyCrypto,
        encryptedData
      );

      const dec = new TextDecoder();
      return dec.decode(decryptedData);
    } catch (e) {
      //console.error('Error during decryption:', e);
      this.customModal.open("Chyba","Nepodarilo sa dešifrovať dáta");
      throw e;
    }
  }

  private async decryptDataPDF(
    encryptedData: ArrayBuffer,
    encryptedSessionKey: ArrayBuffer,
    nonce: ArrayBuffer,
    privateKeyText: string
  ): Promise<ArrayBuffer> {
    try {
      const privateKey = await crypto.subtle.importKey(
        "pkcs8",
        this.convertPrivateKeyToCryptoKey(privateKeyText),
        {
          name: "RSA-OAEP",
          hash: {name: "SHA-256"},
        },
        false,
        ["decrypt"]
      );
      const sessionKey = await crypto.subtle.decrypt(
        {
          name: "RSA-OAEP",
        },
        privateKey,
        encryptedSessionKey
      );
      const sessionKeyCrypto = await crypto.subtle.importKey(
        "raw",
        sessionKey,
        {
          name: "AES-GCM",
          length: 128
        },
        false,
        ["decrypt"]
      );
      const decryptedData = await crypto.subtle.decrypt(
        {
          name: "AES-GCM",
          iv: nonce,
          tagLength: 128
        },
        sessionKeyCrypto,
        encryptedData
      );

      return decryptedData;
    } catch (e) {
      //console.error('Error during decryption:', e);
      this.customModal.open("Chyba","Nepodarilo sa dešifrovať dáta");
      throw e;
    }
  }
  
  

  /**
   * Method that converts a private key to a CryptoKey.
   * @param privateKeyPEM The private key in PEM format.
   * @returns The private key as an ArrayBuffer.
   */
  private convertPrivateKeyToCryptoKey(privateKeyPEM: string): ArrayBuffer {
    const pemHeader = '-----BEGIN PRIVATE KEY-----';
    const pemFooter = '-----END PRIVATE KEY-----';
    const pemContents = privateKeyPEM
      .replace(pemHeader, '')
      .replace(pemFooter, '')
      .replace(/\s/g, '');
  
    const binaryDerString = atob(pemContents);
  
    const binaryDer = new Uint8Array(binaryDerString.length);
    for (let i = 0; i < binaryDerString.length; i++) {
      binaryDer[i] = binaryDerString.charCodeAt(i);
    }
    return binaryDer.buffer;
  }


  // _________________________BONUS_________________________

  /**
   * Method that generates a key pair.
   * @returns A Promise of the generated key pair.
   */
  async generateKeyPair() {
    try {
      const keyPair = await window.crypto.subtle.generateKey(
        {
          name: "RSA-OAEP",
          modulusLength: 2048,
          publicExponent: new Uint8Array([1, 0, 1]),
          hash: {name: "SHA-256"},
        },
        true,
        ["encrypt", "decrypt"]
      );
      return keyPair;
    } catch (error) {
      //console.error('Key generation error:', error);
      this.customModal.open("Chyba", "Nepodarilo sa vygenerovať kľúč");
      throw error;
    }
  }

  /**
   * Method that exports the public and private keys to an ArrayBuffer.
   * @param keyPair The key pair to export.
   * @returns An object containing the public and private keys as ArrayBuffers.
   */
  async exportKeys(keyPair: CryptoKeyPair) {
    try {
      const publicKey = await window.crypto.subtle.exportKey(
        "spki", // Public key format
        keyPair.publicKey
      );
      const privateKey = await window.crypto.subtle.exportKey(
        "pkcs8", // Private key format
        keyPair.privateKey
      );
      return { publicKey, privateKey };
    } catch (error) {
      //console.error('Export keys error:', error);
      this.customModal.open("Chyba", "Nepodarilo sa exportovať kľúče");
      throw error;
    }
  }

  /**
   * Method that uploads the public key to the server.
   * @param publicKey The public key to upload.
   */
  async uploadPublicKey(publicKey: ArrayBuffer) {
    const publicKeyData = new Uint8Array(publicKey);

    const binaryPublicKey = String.fromCharCode.apply(null, Array.from(publicKeyData));

    const base64PublicKey = btoa(binaryPublicKey);
    this.publicKeyContent = base64PublicKey;
    const pemHeader = "-----BEGIN PUBLIC KEY-----\n";
    const pemFooter = "\n-----END PUBLIC KEY-----";
    const chunks = base64PublicKey.match(/.{1,64}/g);
    const pemKey = pemHeader + (chunks ? chunks.join('\n') : '') + pemFooter;

    const publicKeyBlob = new Blob([pemKey], { type: 'text/plain' });

    const formData = new FormData();
    formData.append('public_key', publicKeyBlob, 'public_key.pem');
  
    try {
      const response = await this.http.post('api/public_key', formData).toPromise();
      this.customModal.open("Úspešné nahranie", "Verejný kľúč bol úspešne nahraný");
    } catch (error) {
      this.customModal.open("Chyba", "Verejný kľúč sa nepodarilo nahrať");
      throw error;
    }
  }
  

  /**
   * Method that triggers the download of the private key.
   * @param privateKey The private key to download.
   */
  async downloadPrivateKey(privateKey: ArrayBuffer) {
    try {
      const blob = new Blob([privateKey], { type: 'application/octet-stream' });
      const url = window.URL.createObjectURL(blob);
      const anchor = document.createElement('a');
      anchor.href = url;
      anchor.download = 'private_key.pem';
      anchor.click();
      window.URL.revokeObjectURL(url);
    } catch (error) {
      //console.error('Download private key error:', error);
      this.customModal.open("Chyba", "Nepodarilo sa stiahnuť súkromný kľúč");
      throw error;
    }
  }

  /**
   * Method that handles key generation and uploading.
   */
  async handleGenerateKeys() {
    try {
      const keyPair = await this.generateKeyPair();
      if (keyPair) {
        const exportedKeys = await this.exportKeys(keyPair);
        await this.uploadPublicKey(exportedKeys.publicKey);
        this.downloadPrivateKey(exportedKeys.privateKey);

        const privateKeyData = new Uint8Array(exportedKeys.privateKey);
        const base64PrivateKey = btoa(String.fromCharCode(...privateKeyData));
        this.privateKeyContent = base64PrivateKey;
        
        const encryptedUserData = await this.getEncryptedUserData().toPromise();

        if (!encryptedUserData) {
          throw new Error('Encrypted user data is undefined');
        }

        const encryptedKeyLength = 256;
        const nonceSizeInBytes = 12;

        const encryptedKey = encryptedUserData.slice(0, encryptedKeyLength);
        const nonce = encryptedUserData.slice(encryptedKeyLength, encryptedKeyLength + nonceSizeInBytes);
        const encryptedData = encryptedUserData.slice(encryptedKeyLength + nonceSizeInBytes);

        const decryptedData = await this.decryptData(encryptedData, encryptedKey, nonce, this.privateKeyContent);
        const decryptedObject = JSON.parse(decryptedData);
        
        this.firstName = decryptedObject.first_name;
      }
    } catch (error) {
      //console.error('Handle generate keys error:', error);
      this.customModal.open("Chyba", "Nepodarilo sa vygenerovať kľúče");
    }
  }

}
