import json
import base64

from django.middleware.csrf import get_token
from django.core import serializers
from django.core.exceptions import ValidationError
from django.core.validators import validate_email

from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated, AllowAny

from volby_backend.models import EncryptedVote, PoliticalParty, Vote, EncryptedVoteServer
from volby_backend.serializers import PoliticalPartySerializer
from volby_backend.services.key_management import generate_RSA_keys
from volby_backend.services.encryption_service import (
    symmetric_encrypt_user,
    InvalidPublicKeyFormatError,
    symmetric_encrypt,
)
from volby_backend.services.checksum_service import generate_checksum
from volby_backend.services.encryption_service import decrypt_data
from volby_backend.utils.responses import ErrorResponse, SuccessResponse


@api_view(["POST"])
@permission_classes([IsAuthenticated])
def save_public_key(request):
    public_key = request.data.get("public_key")
    if public_key:
        request.user.public_key = public_key
        request.user.save()  # Save the user's profile
        return SuccessResponse("Public key saved successfully")
    return ErrorResponse(status.HTTP_400_BAD_REQUEST, "Bad Request", "")


@api_view(["GET"])
@permission_classes([IsAuthenticated])
def generate_key_pair(request):
    private_key, public_key = generate_RSA_keys()
    if public_key:
        request.user.public_key = public_key
        request.user.save()
        res_body = {
            "private_key": private_key,
        }
        return SuccessResponse(res_body, "Public key saved successfully")
    return ErrorResponse(status.HTTP_400_BAD_REQUEST, "Bad Request", "")


@api_view(["GET"])
@permission_classes([IsAuthenticated])
def user_encrypt(request):
    if not request.user.public_key:
        return ErrorResponse(400, {"error": "Public key is empty."})
    try:
        user_data = serializers.serialize("json", [request.user])

        encrypted_data, encrypted_key = symmetric_encrypt_user(
            user_data, request.user.public_key
        )

        res_body = {
            "encrypted_data": base64.b64encode(encrypted_data).decode(),
            "encrypted_key": base64.b64encode(encrypted_key).decode(),
        }

        checksum = generate_checksum(json.dumps(res_body))

        res = SuccessResponse(res_body, status_code=status.HTTP_200_OK)
        res["Checksum"] = checksum
        res["csrftoken"] = get_token(request)

        return res
    except InvalidPublicKeyFormatError:
        return ErrorResponse(
            status.HTTP_400_BAD_REQUEST, "Bad Request", "Invalid public key format."
        )


@api_view(["POST"])
@permission_classes([IsAuthenticated])
def vote(request):
    try:
        if not request.user.voting_rights:
            return ErrorResponse(
                status.HTTP_400_BAD_REQUEST, "Bad Request", "User already voted"
            )
        # Extract the selected political party UUID from the request data
        selected_party_id = request.data.get("selected_party_id")

        # Validate the selected political party UUID (you may want to add more validation logic)
        if not selected_party_id:
            return ErrorResponse(
                status.HTTP_400_BAD_REQUEST, "Bad Request", "No party selected."
            )

        # Retrieve the political party based on the UUID
        selected_party = PoliticalParty.objects.get(id=selected_party_id)

        vote_data = {selected_party.name}
        encrypted_data, encrypted_key = symmetric_encrypt_user(str(vote_data), request.user.public_key)
        decoded_data = base64.b64encode(encrypted_data).decode()
        decoded_key = base64.b64encode(encrypted_key).decode()
        encrypted_data_server, encrypted_key_server = symmetric_encrypt(str(vote_data))
        # Create a new encrypted vote instance
        encrypted_vote_server = EncryptedVoteServer.objects.create(
            data=encrypted_data_server, key=encrypted_key_server
        )
        encrypted_vote = EncryptedVote.objects.create(
            data=encrypted_data, key=encrypted_key
        )
        request.user.voting_rights = False
        request.user.editable = False
        request.user.save()
        # Return the selected political party UUID
        return SuccessResponse(
            {"encrypted_vote_data": decoded_data, "encrypted_vote_key": decoded_key},
            "Vote successful"
        )

    except PoliticalParty.DoesNotExist:
        return ErrorResponse(
            status.HTTP_400_BAD_REQUEST, "Bad Request", "Invalid party ID"
        )
    except Exception as e:
        return ErrorResponse(
            status.HTTP_500_INTERNAL_SERVER_ERROR, "Internal Server Error", str(e)
        )


@api_view(["GET"])
@permission_classes([AllowAny])
def decrypt_and_save_votes(request):
    encrypted_votes = EncryptedVoteServer.objects.all()
    print(encrypted_votes.count())
    for encrypted_vote in encrypted_votes:
        try:
            encrypted_data_text = base64.b64decode(encrypted_vote.data)
            encrypted_key_text = base64.b64decode(encrypted_vote.key)
            decrypted_data = decrypt_data(encrypted_data_text, encrypted_key_text)
            # Parse the decrypted data (assuming it's a JSON string)
            # Create a new Vote instance

            vote = Vote.objects.create(
                party_name=decrypted_data, timestamp=encrypted_vote.timestamp
            )
            encrypted_vote.delete()
            return SuccessResponse(
                {"message": "Votes decrypted and saved successfully."}
            )
        except Exception as e:
            return ErrorResponse(
                status.HTTP_500_INTERNAL_SERVER_ERROR,
                "Internal Server Error",
                "Error while decrypting and saving votes.",
            )

    return ErrorResponse(status.HTTP_400_BAD_REQUEST, "Bad Request", "No")


@api_view(["PUT"])
@permission_classes([IsAuthenticated])
def update_user_email(request):
    try:
        user = request.user
        new_email = request.data.get("new_email")

        if not new_email:
            return ErrorResponse(status.HTTP_400_BAD_REQUEST, "Bad Request", "New email is required.")

        try:
            validate_email(new_email)
        except ValidationError:
            return ErrorResponse(status.HTTP_400_BAD_REQUEST, "Bad Request", "Invalid email address.")

        user.email = new_email
        user.save()

        return SuccessResponse({"message": "Email updated successfully."}, status.HTTP_200_OK)

    except Exception as e:
        return ErrorResponse(status.HTTP_500_INTERNAL_SERVER_ERROR, str(e))


@api_view(["GET"])
@permission_classes([AllowAny])
def get_all_political_parties(request):
    try:
        political_parties = PoliticalParty.objects.all()
        serializer = PoliticalPartySerializer(political_parties, many=True)

        return SuccessResponse(serializer.data, "Success", status.HTTP_200_OK)

    except Exception as e:
        return ErrorResponse(status.HTTP_500_INTERNAL_SERVER_ERROR, str(e))
