import React, { FC, useState } from 'react';
import { decryptData, decryptKey, importPrivateKey, importSymetricKey } from '../utils/cryptoHelpers';

type FormData = {
  private_key: File | null,
  confirmation_text: string,
}

const DecryptConfirmationForm: FC = () => {
  const [success, setSuccess] = useState<boolean>(false);
  const [error, setError] = useState<string>('');
  const [inputKey, setInputKey] = useState(Math.random().toString());
  const [formData, setFormData] = useState<FormData>({
    private_key: null,
    confirmation_text: '',
  });
  const [decryptedData, setDecryptedData] = useState<string>('');

  const handleChange = (e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) => {
    const { name, value } = e.target;
    let files;

    if (e.target instanceof HTMLInputElement && e.target.files
      && e.target.files.length > 0) {
      files = e.target.files;
    }

    setFormData({
      ...formData,
      [name]: files ? files[0] : value,
    });
  };

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();
    setError('');
    setSuccess(false);
    setDecryptedData('');

    console.log(formData);

    if (!formData.private_key) {
      setError('Prosím vyberte súbor s privátnym kľúčom');
      return;
    }

    if (!formData.confirmation_text) {
      setError('Prosím zadajte text potvrdenia');
      return;
    }

    const privateKey = await formData.private_key.text();
    const confirmationText = formData.confirmation_text.replace(/\n/g, '');
    const confirmation = JSON.parse(confirmationText);

    try {
      const importedPrivateKey = await importPrivateKey(privateKey, true);
      if (!importedPrivateKey) {
        setError('Nesprávne heslo');
        return;
      }
      console.log(confirmation.data.encrypted_vote_key);
      
      const decryptedKey = await decryptKey(importedPrivateKey, confirmation.data.encrypted_vote_key);
      console.log(decryptedKey);
  
      const symmetricKey = await importSymetricKey(decryptedKey);
      const decryptedData = await decryptData(symmetricKey, confirmation.data.encrypted_vote_data);

      setDecryptedData(decryptedData);
      setSuccess(true);
      setFormData({
        private_key: null,
        confirmation_text: '',
      });
      setInputKey(Math.random().toString());
    } catch (err) {
      setError('Nesprávny formát');
    }
  };

  return (
    <div className='p-4 space-y-5 flex flex-col items-center justify-center border border-slate-200 rounded-md'>
      <form className="flex flex-row items-center justify-center space-x-5" onSubmit={handleSubmit}>
        <div className='rounded-md px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900'>
          <label htmlFor="private_key" className="">Privátny kľúč</label>
          <input key={inputKey} onChange={handleChange} required id="private_key" name="private_key" type="file" accept=".pem" className="appearance-none relative block w-full px-3 py-2 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" />
        </div>
        {/* textarea for confirmation text */}
        <div className='rounded-md px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900'>
          <label htmlFor="confirmation_text" className="block text-sm font-medium text-gray-700">
            Dešifrované dáta
          </label>
          <textarea id="confirmation_text" name="confirmation_text" onChange={handleChange} className="" />
        </div>
        <div>
          <button type="submit" className="group relative w-full flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
            Dešifrovať potvrdenie
          </button>
        </div>
      </form>
      {success && <p className="text-green-500">Potvrdenie bolo úspešne dešifrované</p>}
      {error && <p className="text-red-500">{error}</p>}
      {decryptedData && <p className="text-gray-900">{decryptedData}</p>}
    </div>
  );
};

export default DecryptConfirmationForm;