import { FC, useContext, useEffect, useState } from "react";
import { authenticatedFetch } from "../utils/authenticatedFetch";
import { AuthContext } from "../context/AuthContext";
import { useLogout } from "../hooks/useLogout";
import jsPDF from "jspdf";

type Party = {
  id: string;
  name: string;
  description: string;
  attributes: string[];
};

async function fetchParties(): Promise<Party[]> {
  const response = await authenticatedFetch('/user/politicalparties', { method: 'GET' });
  const data = await response.json();
  return data.data;
}

const ElectionForm: FC = () => {
  const { user } = useContext(AuthContext);
  const logout = useLogout();
  const [selectedParty, setSelectedParty] = useState<Party | null>(null);
  const [success, setSuccess] = useState<boolean>(false);
  const [error, setError] = useState<string>('');
  const [parties, setParties] = useState<Party[]>([]);

  useEffect(() => {
    fetchParties().then(setParties);
  }, []);

  const handleChange = (e: React.ChangeEvent<HTMLSelectElement>) => {
    const selectedParty = parties.find((party) => party.name === e.target.value);
    setSelectedParty(selectedParty || null);
  }

  const handleUpdate = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();
    setSuccess(false);
    setError('');

    if (!selectedParty) {
      setError("Nebola vybraná strana");
      return;
    }

    if (!user?.private_key) {
      setError("Najskôr je potrebné vygenerovať/nahrať kľúče.");
      return;
    }

    try {
      const response = await authenticatedFetch('/user/vote', {
        method: 'POST',
        body: JSON.stringify({ "selected_party_id": selectedParty.id }),
      });
      const data = await response.json();
      if (!response.ok) {
        throw new Error(data.message);
      }
      
      const doc = new jsPDF();
      const lines = doc.splitTextToSize(JSON.stringify(data), 180);
      doc.text(lines, 10, 10);
      const pdfBlob = doc.output('blob');
      const fileURL = URL.createObjectURL(pdfBlob);
      const tempLink = document.createElement('a');
      tempLink.href = fileURL;
      tempLink.download = 'confirmation.pdf';
      document.body.appendChild(tempLink);
      tempLink.click();
      document.body.removeChild(tempLink);
      URL.revokeObjectURL(fileURL);
      
      // wait 2 seconds and logout
      setSuccess(true);
      await new Promise((resolve) => setTimeout(resolve, 2000));
      user.voted = true;
      logout(user.voted);
    } catch (err) {
      setError("Nepodarilo sa odoslať hlas");
    }
  };

  return (
    <form onSubmit={handleUpdate} className="flex flex-col items-center justify-center gap-4">
      <div className="flex flex-row gap-4">
        <select defaultValue={''} className="border rounded-full px-5 py-2" name="party" id="party" onChange={handleChange}>
          <option value={''} disabled>Vyberte stranu</option>
          {parties.map((party) => (
            <option key={party.id} value={party.name}>{party.name}</option>
          ))}
        </select>
        <button type="submit" className="group relative flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-full text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500 transition-all duration-200">
          Odoslať hlas
        </button>
      </div>
      {error && <p className="text-red-500">{error}</p>}
      {success && <p className="text-green-500">Váš hlas bol zaznamenaný ďakujeme</p>}
    </form>
  );
}

export default ElectionForm;