import { useContext, useRef, useState } from "react";
import { exportPrivateCryptoKey, exportPublicCryptoKey, generateRSAKeys } from "../utils/cryptoHelpers";
import { authenticatedFetch } from "../utils/authenticatedFetch";
import { AuthContext } from "../context/AuthContext";

const GenerateKeysButton = () => {
  const { user } = useContext(AuthContext);
  const [success, setSuccess] = useState<JSX.Element | null>(null);
  const [error, setError] = useState<string>('');
  // const [passphrase, setPassphrase] = useState<string>('');

  const controllerRef = useRef<AbortController | null>(null);

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();

    // Cancel the previous fetch request
    if (controllerRef.current) {
      controllerRef.current.abort();
    }

    // Create a new AbortController
    const newController = new AbortController();
    controllerRef.current = newController;

    try {
      // generate keys
      const { privateKey, publicKey } = await generateRSAKeys();

      // export public key
      const exportedPublicKey = await exportPublicCryptoKey(publicKey);
      
      const response = await authenticatedFetch('/user/load-public',
        {
          method: 'POST',
          body: JSON.stringify({
            public_key: exportedPublicKey,
          }),
        }
      );
      
      if (response.ok) {
        // encrypt private key with user's password
        // const encryptedPrivateKey = await encryptPrivateKey(privateKey, passphrase);
        // save encrypted private key to session storage
        // sessionStorage.setItem('privateKey', JSON.stringify(encryptedPrivateKey));

        // save private key to user from auth context
        if (user) {
          user.private_key = privateKey;
        }        

        // expose private key for download as .pem file
        const pemPrivateKey = await exportPrivateCryptoKey(privateKey);

        // display download link for private key in success message
        setSuccess(
          <>
            <p className="text-green-500">Nový pár kľúčov bol úspešne vygenerovaný</p>
            <a
              href={'data:text/plain;charset=utf-8,' + encodeURIComponent(pemPrivateKey)}
              download="private_key.pem"
              className="font-medium text-indigo-600 hover:text-indigo-500"
            >
              Stiahnuť privátny kľúč
            </a>
          </>
        );
        setError('');
      } else {
        const { message } = await response.json();
        setError(message);
      }
    } catch (error) {
      if (error instanceof Error) {
        if (error.name === 'AbortError') {
          console.log('Previous request was cancelled');
        } else {
          setError(error.message);
        }
      } else {
        setError(String(error));
      }
    }
  };

  return (
    <div className='p-4 space-y-5 flex flex-col items-start justify-center border border-slate-200 rounded-md'>
      <form className="flex flex-row items-center justify-center space-x-5" onSubmit={handleSubmit}>
        {/* <div>
          <label htmlFor="passphrase" className="sr-only"></label>
          <input value={passphrase} onChange={(e) => setPassphrase(e.target.value)} required autoComplete="off" id="passphrase" name="passphrase" type="password" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Heslo" />
        </div> */}
        <div>
          <button type="submit" className="group relative w-full flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
            Vygenerovať nový pár kľúčov
          </button>
        </div>
      </form>
      {success && success}
      {error && <p className="text-red-500">{error}</p>}
    </div>
  )
}

export default GenerateKeysButton;
