import React, { useState, useContext } from 'react';
import { AuthContext } from '../context/AuthContext';
import customFetch from '../utils/customFetch';
import { useNavigate } from 'react-router-dom';

type FormData = {
  email: string;
  password: string;
}

const validateForm = (formData: FormData) => {
  const errors: string[] = [];

  if (!formData.email) {
    errors.push('Email is required');
  }

  if (formData.email && !/\S+@\S+\.\S+/.test(formData.email)) {
    errors.push('Email address is invalid');
  }

  if (!formData.password) {
    errors.push('Password is required');
  }

  // check if password contains at least 8 characters, 1 uppercase letter, 1 lowercase letter, 1 number and 1 special character.
  if (formData.password && !/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^\da-zA-Z]).{8,}$/.test(formData.password)) {
    errors.push('Password must contain at least 8 characters, 1 uppercase letter, 1 lowercase letter, 1 number and 1 special character.');
  }

  return errors;
};

const Login: React.FC = () => {
  const { login } = useContext(AuthContext);
  const navigate = useNavigate();
  const [errors, setErrors] = useState<string[]>([]);
  const [formData, setFormData] = useState<FormData>({
    email: '',
    password: '',
  });

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFormData({ ...formData, [e.target.name]: e.target.value });
  }

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();

    const errors = validateForm(formData);

    if (errors.length) {
      setErrors(errors);
      return;
    } else {
      setErrors([]);
    }

    const response = await customFetch(`/auth/login`, {
      method: 'POST',
      body: JSON.stringify(formData),
    })

    if (response.ok) {
      setFormData({ email: '', password: '' });
      const data = await response.json();
      login(data.data.access_token);
      navigate('/');
    } else if (response.status === 401) {
      setErrors(["Nesprávne meno alebo heslo."]);
    } else {
      setErrors(["Nastala chyba pri prihlasovaní."]);
    }
  };

  return (
    <section className='min-h-screen flex flex-col justify-center items-center"'>
      <div className="bg-white p-8 rounded-lg shadow-md w-full max-w-sm mx-auto">
        <form className="space-y-6" onSubmit={handleSubmit}>
          <h2 className="text-center text-3xl font-extrabold text-gray-900">
            Prihláste sa<br />do vášho účtu
          </h2>
          {errors.map((error, idx) => (
            <div key={idx} className="text-red-500">
              <p>{error}</p>
            </div>
          ))}
          <div>
            <label htmlFor="email" className="sr-only">Email</label>
            <input value={formData.email} onChange={handleChange} required id="email" name="email" type="email" autoComplete="email" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Email" />
          </div>
          <div>
            <label htmlFor="password" className="sr-only">Heslo</label>
            <input value={formData.password} onChange={handleChange} required id="password" name="password" type="password" autoComplete="current-password" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Heslo" />
          </div>
          <div>
            <button type="submit" className="group relative w-full flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
              Prihlásiť sa
            </button>
          </div>
          {/* link to register */}
          <div className="text-center">
            <a href="/auth/register" className="font-medium text-indigo-600 hover:text-indigo-500">
              Nemáte účet? Zaregistrujte sa
            </a>
          </div>
        </form>
      </div>
    </section>
  );
};

export default Login;