import { FC, useState } from 'react';
import customFetch from '../utils/customFetch';

type FormData = {
  id_card_number: string;
  birth_number: string;
  email: string;
  password: string;
  password2: string;
}

const validateForm = (formData: FormData) => {
  const errors: string[] = [];

  if (!formData.id_card_number) {
    errors.push('Číslo OP je povinné');
  }

  if (formData.id_card_number && !/^[A-Z0-9]+$/.test(formData.id_card_number)) {
    errors.push('Číslo OP je neplatné');
  }

  if (!formData.birth_number) {
    errors.push('Rodné číslo je povinné');
  }

  // check if birth number is in format XXXXXX/XXX or XXXXXX/XXXX or XXXXXXXXX or XXXXXXXXXX
  if (formData.birth_number && !/^[0-9]{6}\/[0-9]{3,4}$|^[0-9]{9,10}$/.test(formData.birth_number)) {
    errors.push('Rodné číslo je neplatné');
  }

  if (!formData.email) {
    errors.push('Email je povinný');
  }

  if (formData.email && !/\S+@\S+\.\S+/.test(formData.email)) {
    errors.push('Email je neplatný');
  }

  if (!formData.password) {
    errors.push('Heslo je povinné');
  }

  // check if password contains at least 8 characters, 1 uppercase letter, 1 lowercase letter, 1 number and 1 special character.
  if (formData.password && !/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^\da-zA-Z]).{8,}$/.test(formData.password)) {
    errors.push('Heslo musí obsahovať aspoň 8 znakov, 1 veľké písmeno, 1 malé písmeno, 1 číslo a 1 špeciálny znak.');
  }

  if (!formData.password2) {
    errors.push('Potvrdenie hesla je povinné');
  }

  if (formData.password !== formData.password2) {
    errors.push('Heslá sa nezhodujú');
  }

  return errors;
}


const Register: FC = () => {
  const [errors, setErrors] = useState<string[]>([]);
  const [success, setSuccess] = useState<string>('');
  const [formData, setFormData] = useState<FormData>({
    id_card_number: '',
    birth_number: '',
    email: '',
    password: '',
    password2: '',
  });

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setFormData({ ...formData, [e.target.name]: e.target.value });
  }

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();

    const errors = validateForm(formData);

    if (errors.length) {
      setErrors(errors);
      return;
    } else {
      setErrors([]);
    }

    if (formData.birth_number.includes('/')) {
      formData.birth_number = formData.birth_number.replace('/', '');
    }

    const response = await customFetch(`/auth/register`, {
      method: 'POST',
      body: JSON.stringify(formData),
      headers: {
        'Content-Type': 'application/json',
      },
    });

    if (response.status === 201) {
      // redirect to login page
      setSuccess('Registrácia prebehla úspešne. Teraz sa môžete prihlásiť.');
      setFormData({
        id_card_number: '',
        birth_number: '',
        email: '',
        password: '',
        password2: '',
      });
    } else if (response.status === 401) {
      setErrors(["Používateľ nemá oprávnenie voliť online."]);
    } else if (response.status === 403) {
      setErrors(["Používateľ už bol registrovaný."]);
    } else if (response.status === 400) {
      setErrors(["Nesprávne údaje."]);
    } else {
      setErrors(["Nastala chyba pri prihlasovaní."]);
    }
  };

  return (
    <section className='min-h-screen flex flex-col justify-center items-center"'>
      <div className="bg-white p-8 rounded-lg shadow-md w-full max-w-sm mx-auto">
        {success ? (
          <>
            <div className="bg-green-100 border border-green-400 text-green-700 px-4 py-3 rounded relative" role="alert">
              <span className="block sm:inline">{success}</span>
            </div>
            <div className="text-center mt-5">
              <a href="/auth/login" className="font-medium text-indigo-600 hover:text-indigo-500">
                Prihlásiť sa
              </a>
            </div>
          </>
        ) : (
          <form className="space-y-6" onSubmit={handleSubmit}>
            <h2 className="text-center text-3xl font-extrabold text-gray-900">
              Zaregistrujte sa
            </h2>
            <div className="text-red-500">
              {errors.map((error) => (
                <p key={error}>{error}</p>
              ))}
            </div>
            <div>
              <label htmlFor="id_card_number" className="sr-only">Číslo OP</label>
              <input onChange={handleChange} required id="id_card_number" name="id_card_number" type="text" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Číslo OP" />
            </div>
            <div>
              <label htmlFor="birth_number" className="sr-only">Rodné číslo</label>
              <input onChange={handleChange} required id="birth_number" name="birth_number" type="text" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Rodné číslo" />
            </div>
            <div>
              <label htmlFor="email" className="sr-only">Email</label>
              <input value={formData.email} onChange={handleChange} required id="email" name="email" type="email" autoComplete="email" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Email" />
            </div>
            <div>
              <label htmlFor="password" className="sr-only">Heslo</label>
              <input value={formData.password} onChange={handleChange} required id="password" name="password" type="password" autoComplete="current-password" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Heslo" />
            </div>
            <div>
              <label htmlFor="password2" className="sr-only">Zopakujte heslo</label>
              <input value={formData.password2} onChange={handleChange} required id="password2" name="password2" type="password" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" placeholder="Zopakujte heslo" />
            </div>
            <div>
              <button type="submit" className="group relative w-full flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
                Zaregistrovať sa
              </button>
            </div>
            <div className="text-center">
              <a href="/auth/login" className="font-medium text-indigo-600 hover:text-indigo-500">
                Už máte účet? Prihláste sa
              </a>
            </div>
          </form>
        )}
      </div>
    </section>
  );
};

export default Register;