import { useContext, useRef, useState } from "react";
import { authenticatedFetch } from "../utils/authenticatedFetch";
import { AuthContext } from "../context/AuthContext";
import { decryptData, decryptKey, importSymetricKey } from "../utils/cryptoHelpers";

const GenerateKeysButton = () => {
  const { user } = useContext(AuthContext);
  const [success, setSuccess] = useState<boolean>(false);
  const [encryptedData, setEncryptedData] = useState<string>('');
  const [decryptedData, setDecryptedData] = useState<string>('');
  const [error, setError] = useState<string>('');

  const controllerRef = useRef<AbortController | null>(null);

  const handleClick = async () => {
    setError('');
    setSuccess(false);

    if (!user || !user.private_key || user.private_key === null) {
      setError('Najskôr nahrajte súkromný kľúč');
      return;
    }

    // Cancel the previous fetch request
    if (controllerRef.current) {
      controllerRef.current.abort();
    }

    // Create a new AbortController
    const newController = new AbortController();
    controllerRef.current = newController;

    const response = await authenticatedFetch('/user/encrypt', {
      method: 'GET',
      signal: newController.signal,
    });

    if (response.ok) {
      const jsonResponse = await response.json();
      const data = jsonResponse.data;
      setEncryptedData(data.encrypted_data);
      // decrypt encrypted_key with privateKey
      const decryptedKey = await decryptKey(user.private_key, data.encrypted_key);

      const symmetricKey = await importSymetricKey(decryptedKey);

      const decryptedData = await decryptData(symmetricKey, data.encrypted_data);

      setDecryptedData(decryptedData);
      setSuccess(true);
    } else {
      setError('Nastala chyba pri šifrovaní dát');
    }
  };

  return (
    <div className="p-4 space-y-5 flex flex-col items-start justify-center rounded-md border border-slate-200">
      <button
        className='group relative flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500'
        onClick={handleClick}
      >
        Otestovať šifrovanie
      </button>
      {success && <p className=" text-green-500">Šifrovanie bolo úspešne otestované</p>}
      {error && <p className="text-red-500">{error}</p>}
      {encryptedData && (
        <>
          <label htmlFor="encryptedData" className="block text-sm font-medium text-gray-700">
            Zašifrované dáta
          </label>
          <textarea id="encryptedData" value={encryptedData} className="w-[25rem] h-[25rem]" readOnly />
        </>
      )}
      {decryptedData && (
        <>
          <label htmlFor="decryptedData" className="block text-sm font-medium text-gray-700">
            Dešifrované dáta
          </label>
          <textarea id="decryptedData" value={decryptedData} className="w-[25rem] h-[25rem]" readOnly />
        </>
      )}
    </div>
  )
}

export default GenerateKeysButton;
