import React, { useState } from 'react';
import { authenticatedFetch } from '../utils/authenticatedFetch';

const UploadPublicKeyForm = () => {
  const [success, setSuccess] = useState<boolean>(false);
  const [error, setError] = useState<string>('');
  const [publicKeyFile, setPublicKeyFile] = useState<File | null>(null);
  const [inputKey, setInputKey] = useState<string>(Math.random().toString());

  const handleFileChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setPublicKeyFile(e.target.files ? e.target.files[0] : null);
  };

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();
    setError('');
    
    setSuccess(false);

    if (!publicKeyFile) {
      setError('Prosím vyberte súbor s verejným kľúčom');
      return;
    }

    const publicKey = await publicKeyFile.text();

    // Check if the uploaded file is a valid public key
    const publicKeyPattern = /^-----BEGIN PUBLIC KEY-----\n(.|\n)*-----END PUBLIC KEY-----\n?$/;
    if (!publicKeyPattern.test(publicKey)) {
      setError('Nesprávny formát');
      return;
    }

    // Send public key to server
    authenticatedFetch('user/load-public', {
      method: 'POST',
      body: JSON.stringify({ "public_key": publicKey })
    }).then((res) => {
      if (res.status === 200) {
        setPublicKeyFile(null);
        setSuccess(true);
        setInputKey(Math.random().toString());
      } else {
        setError('Pri ukladaní verejného kľúča nastala chyba');
      }
    }).catch((error) => {
      console.error('Error:', error);
      setError('Pri ukladaní verejného kľúča nastala chyba');
    });
  };

  return (
    <div className='p-4 space-y-5 flex flex-col items-center justify-center border border-slate-200 rounded-md'>
      <form className="flex flex-row items-center justify-center space-x-5" onSubmit={handleSubmit}>
        <div>
          <label htmlFor="public_key" className="sr-only">Public Key</label>
          <input key={inputKey} onChange={handleFileChange} required id="public_key" name="public_key" type="file" accept=".pem" className="appearance-none rounded-md relative block w-full px-3 py-2 border border-gray-300 placeholder-gray-500 text-gray-900 focus:outline-none focus:ring-indigo-500 focus:border-indigo-500 focus:z-10 sm:text-sm" />
        </div>
        <div>
          <button type="submit" className="group relative w-full flex justify-center py-2 px-4 border border-transparent text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500">
            Nahrať verejný kľúč
          </button>
        </div>
      </form>
      {success && <p className="text-green-500">Verejný kľúč bol úspešne nahratý</p>}
      {error && <p className="text-red-500">{error}</p>}
    </div>
  );
};

export default UploadPublicKeyForm;