import { createContext, FC, useEffect, useState } from 'react';
import { jwtDecode } from 'jwt-decode';
import { authenticatedFetch } from '../utils/authenticatedFetch';

type User = {
  token_type: string;
  private_key: CryptoKey | null;
  voted: boolean;
}

type AuthContextProps = {
  user: User | null;
  login: (token: string) => void;
  logout: () => void;
}

export const AuthContext = createContext<AuthContextProps>({
  user: null,
  login: () => { },
  logout: () => { },
});

type AuthProviderProps = {
  children: React.ReactNode;
}

export const AuthProvider: FC<AuthProviderProps> = ({ children }) => {
  const [user, setUser] = useState<User | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(true);

  useEffect(() => {
    const verifyToken = async () => {
      const token = localStorage.getItem('jwtToken');

      if (token) {
        try {
          const response = await authenticatedFetch('auth/verify-token', { method: 'GET' });
          const data = await response.json();

          if (data.status === 200) {
            // Token is valid, fetch user data
            const user = jwtDecode<User>(token);
            setUser(user);
          } else {
            setUser(null);
            localStorage.removeItem('jwtToken');
          }
        } catch (error) {
          console.error(error);
          setUser(null);
          localStorage.removeItem('jwtToken');
        }
      } else {
        setUser(null);
      }

      setIsLoading(false);
    };

    verifyToken();
  }, []);

  const login = (token: string) => {
    const user = jwtDecode<User>(token);
    setUser(user);
    localStorage.setItem('jwtToken', token);
  };

  const logout = () => {
    return new Promise((resolve) => {
      localStorage.removeItem('jwtToken');
      localStorage.removeItem('privateKey');
      setUser({ token_type: '', private_key: null, voted: true });
      resolve(null);
    });
  };

  if (isLoading) {
    return <div className='text-black'>Loading...</div>; // Replace this with your actual loading screen
  }

  return (
    <AuthContext.Provider value={{ user, login, logout }}>
      {children}
    </AuthContext.Provider>
  );
};