import { FC, useContext, useEffect, useState } from 'react';
import { AuthContext } from '../context/AuthContext';
import { useNavigate } from 'react-router-dom';
import { authenticatedFetch } from '../utils/authenticatedFetch';

type Party = {
  id: number;
  name: string;
  description: string;
  attributes: string[];
};

async function fetchParties(): Promise<Party[]> {
  const response = await authenticatedFetch('/user/politicalparties', { method: 'GET' });
  const data = await response.json();
  return data.data;
}

const HomePage: FC = () => {
  const { user } = useContext(AuthContext);
  const navigate = useNavigate();
  const [searchTerm, setSearchTerm] = useState<string>('');
  const [parties, setParties] = useState<Party[]>([]);

  useEffect(() => {
    if (!user) {
      navigate('/auth/login');
    }
  }, [user, navigate]);

  useEffect(() => {
    fetchParties().then(setParties);
  }, []);

  const handleSearch = (event: React.ChangeEvent<HTMLInputElement>) => {
    setSearchTerm(event.target.value);
  };

  const filteredParties = parties.filter((party) => {
    const searchIn = [party.name, party.description, ...party.attributes];
    return searchIn.some((field) => field.toLowerCase().includes(searchTerm.toLowerCase()));
  });

  return (
    <section className='flex flex-col items-center justify-start gap-5 mt-24'>
      <h2 className=" font-bold text-2xl text-center">Prehľad strán</h2>
      <input
        type='text'
        placeholder='Hľadať stranu'
        className='border rounded-full px-5 py-2'
        value={searchTerm}
        onChange={handleSearch}
      />
      <div className='flex flex-col gap-4 pb-20'>
        {filteredParties.map((party) => (
          <div key={party.id} className=' max-w-xl flex flex-col gap-4 bg-white rounded-lg shadow-md p-5'>
            <h2 className='text-xl font-bold'>{party.name}</h2>
            <p>{party.description}</p>
            {/* tag cloud with attributes */}
            <div className='flex flex-wrap gap-2'>
              {party.attributes.map((attribute) => (
                <span key={attribute} className='bg-gray-200 rounded-full px-4 py-1'>
                  {attribute}
                </span>
              ))}
            </div>
          </div>
        ))}
      </div>
    </section>
  );
}

export default HomePage;