function ab2str(buf: ArrayBuffer): string {
  return String.fromCharCode.apply(null, Array.from(new Uint8Array(buf)));
}

function ab2str_base64(buf: ArrayBuffer): string {
  let binary = '';
  const bytes = new Uint8Array(buf);
  const len = bytes.byteLength;
  for (let i = 0; i < len; i++) {
    binary += String.fromCharCode(bytes[i]);
  }
  return window.btoa(binary);
}

function str2ab(str: string): ArrayBuffer {
  const buf = new ArrayBuffer(str.length);
  const bufView = new Uint8Array(buf);
  for (let i = 0, strLen = str.length; i < strLen; i++) {
    bufView[i] = str.charCodeAt(i);
  }
  return buf;
}

export function importPrivateKey(pem: string, exportable:boolean = false): Promise<CryptoKey> {
  // fetch the part of the PEM string between header and footer
  const pemHeader = "-----BEGIN PRIVATE KEY-----";
  const pemFooter = "-----END PRIVATE KEY-----";
  let pemContents = pem.substring(
    pemHeader.length,
    pem.length - pemFooter.length,
  );
  
  // check lenghts after removing header and footer
  if (pem.length - pemHeader.length - pemFooter.length !== pemContents.length || pemContents.startsWith("-") || pemContents.endsWith("-")) {
    return Promise.reject("Invalid key format");
  } 

  pemContents = pemContents.trim();
  pemContents = pemContents.replace(/\r?\n|\r/g, "");
  
  // check if pemContents is valid base64
  if (!pemContents.match(/^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)?$/)) {
    return Promise.reject("Invalid key format");
  }

  // base64 decode the string to get the binary data
  const binaryDerString = window.atob(pemContents);
  // convert from a binary string to an ArrayBuffer
  const binaryDer = str2ab(binaryDerString);

  return window.crypto.subtle.importKey(
    "pkcs8",
    binaryDer,
    {
      name: "RSA-OAEP",
      hash: "SHA-256",
    },
    exportable,
    ["decrypt"],
  );
}

export function decryptKey(privateKey: CryptoKey, ciphertext: string): Promise<string> {
  if (!ciphertext.match(/^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)?$/)) {
    return Promise.reject("Invalid format");
  }

  const binary = window.atob(ciphertext)
  const buffer = str2ab(binary);

  return window.crypto.subtle.decrypt(
    {
      name: "RSA-OAEP",
    },
    privateKey,
    buffer,
  ).then((decrypted) => {
    return ab2str_base64(decrypted); 
  });
}

export async function importSymetricKey(key: string): Promise<CryptoKey> {
  const keyBinary = window.atob(key);
  const keyBuffer = str2ab(keyBinary);
  return window.crypto.subtle.importKey(
    "raw",
    keyBuffer,
    "AES-GCM",
    false,
    ["decrypt"],
  );
}

export function decryptData(symmetricKey: CryptoKey, ciphertext: string): Promise<string> {
  if (!ciphertext.match(/^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)?$/)) {
    return Promise.reject("Invalid format");
  }
  
  // base64 decode the string to get the binary data
  const binary = window.atob(ciphertext);
  const buffer = str2ab(binary);

  // Get the nonce
  const nonce = buffer.slice(0, 16);
  const ciphertextBuf = buffer.slice(16);

  return window.crypto.subtle.decrypt(
    {
      name: "AES-GCM",
      iv: nonce,
    },
    symmetricKey,
    ciphertextBuf,
  ).then((decrypted) => {
    // decrypted to utf8
    return ab2str(decrypted);
  })
}

type RSAKeyPair = {
  publicKey: CryptoKey,
  privateKey: CryptoKey,
}

export function generateRSAKeys(): Promise<RSAKeyPair> {
  return window.crypto.subtle.generateKey(
    {
      name: "RSA-OAEP",
      modulusLength: 2048,
      publicExponent: new Uint8Array([1, 0, 1]),
      hash: "SHA-256",
    },
    true,
    ["encrypt", "decrypt"],
  ).then((keyPair) => {
    return {
      publicKey: keyPair.publicKey,
      privateKey: keyPair.privateKey,
    }
  });
}

export async function exportPublicCryptoKey(key: CryptoKey): Promise<string> {
  const exported = await window.crypto.subtle.exportKey("spki", key);
  const exportedAsString = ab2str(exported);
  const exportedAsBase64 = window.btoa(exportedAsString);
  const pemExported = `-----BEGIN PUBLIC KEY-----\n${exportedAsBase64}\n-----END PUBLIC KEY-----`;

  return pemExported;
}

export async function exportPrivateCryptoKey(key: CryptoKey): Promise<string> {
  const exported = await window.crypto.subtle.exportKey("pkcs8", key);
  const exportedAsString = ab2str(exported);
  const exportedAsBase64 = window.btoa(exportedAsString);
  const pemExported = `-----BEGIN PRIVATE KEY-----\n${exportedAsBase64}\n-----END PRIVATE KEY-----`;

  return pemExported;
}

type EncryptedPrivateKey = {
  encryptedKey: ArrayBuffer,
  salt: Uint8Array,
  iv: Uint8Array,
}

export async function encryptPrivateKey(privateKey: CryptoKey, passphrase: string): Promise<EncryptedPrivateKey> {
  // Export the private key
  const exportedKey = await window.crypto.subtle.exportKey(
    "pkcs8",
    privateKey
  );
  
  // Generate a salt for the passphrase
  const salt = window.crypto.getRandomValues(new Uint8Array(16));

  const passphraseKey = await window.crypto.subtle.importKey(
    "raw",
    new TextEncoder().encode(passphrase),
    {name: "PBKDF2"},
    false,
    ["deriveKey"]
  );
  
  // Derive an encryption key from the passphrase
  const encryptionKey = await window.crypto.subtle.deriveKey(
    {
      name: "PBKDF2",
      salt: salt,
      iterations: 100000,
      hash: 'SHA-256'
    },
    passphraseKey,
    { name: "AES-GCM", length: 128},
    false,
    ["encrypt"]
  );

  // Generate an IV for encryption
  const iv = window.crypto.getRandomValues(new Uint8Array(12));

  // Encrypt the private key
  const encryptedKey = await window.crypto.subtle.encrypt(
    {
      name: "AES-GCM",
      iv: iv
    },
    encryptionKey,
    exportedKey
  );

  return {
    encryptedKey: encryptedKey,
    salt: salt,
    iv: iv
  };
}