# app.py
import base64
import json
import os
from calendar import monthrange
from decimal import Decimal
from flask import Flask, request, jsonify, render_template, redirect, url_for, session, \
    send_from_directory, send_file
import psycopg2
from entities.user import User, save_user, create_user_table, \
    get_user_by_email, check_user_password, get_user_by_username, update_user_login_attempts, \
    set_user_lock_datetime_now, unlock_user, update_user_last_login, \
    initialize_database_with_random_users
import time
from datetime import datetime, timedelta, date
from utils.cypher import CryptographyUtils, encryptAES, decryptAES
from utils.password_functions import is_password_valid
from entities.record_type import RecordType, RecordNames, RecordAuthor, get_types_for_select, \
    get_record_name
import entities.record as record
import pytz

SR = pytz.timezone('Europe/Bratislava')
app = Flask(__name__, template_folder='./frontend')
app.secret_key = 'your_secret_key'
# Maximum number of connection attempts
max_connection_attempts = 10

# Delay between connection attempts (in seconds)
connection_attempt_delay = 5

for _ in range(max_connection_attempts):
    try:
        conn = psycopg2.connect(
            dbname="upb-db",
            user="user",
            password="password",
            host="db"  # This is the service name defined in docker-compose.yml
        )
        break  # Successful connection, exit the loop
    except psycopg2.OperationalError as e:
        print(f"Connection attempt failed: {str(e)}")
        time.sleep(connection_attempt_delay)  # Wait before the next attempt

if not conn:
    # If the connection wasn't established after all attempts, you can raise an exception or handle it accordingly.
    raise Exception("Failed to connect to the database")

create_user_table(conn)
record.create_records_table(conn)
initialize_database_with_random_users(conn, 5)
users = ['user1', 'user2', 'user3']
months = [10, 11]
years = [2023]
record.generate_records(conn, users, months, years)

cryptog = CryptographyUtils()
try:
    cryptog.generate_asymmetric_keys()
except:
    cryptog.load_keys("./keys/public_key.pem", "./keys/private_key.pem")


@app.route('/')
def index():
    if 'username' in session:
        return redirect(url_for('home'))
    return render_template('base.html', message="")


@app.route('/add_user', methods=['POST'])
def add_user():
    if 'username' not in session:
        return redirect(url_for('index'))

    data = request.get_json()
    username = data.get('username')
    email = data.get('email')
    password = data.get('password')

    if not username or not email or not password:
        return jsonify({'error': 'Both username and email are required'}), 400  # Bad Request

    new_user = User(id=None, username=username, email=email, password=password)
    save_user(conn, new_user)

    return jsonify({'message': 'User added successfully'}), 201  # Created


# register endpoint from form
@app.route('/register', methods=['POST'])
def register():
    data = request.get_json()
    username = cryptog.decryptMessage(data['username'])
    email = cryptog.decryptMessage(data['email'])
    password = cryptog.decryptMessage(data['password'])

    if not username or not email or not password:
        return jsonify({'error': 'Všetky polia musia byť vyplnené'}), 400  # Bad Request

    if not username.isalnum() or len(username) < 3 or len(username) > 20:
        return jsonify({
            'error': 'Používateľské meno musí obsahovať iba písmená a čísla a musí mať aspoň 3 a najviac 20 znakov'}), 400  # Bad Request

    user = get_user_by_username(conn, username)

    if user:
        return jsonify({'error': 'Používateľské meno alebo email už existuje'}), 400  # Bad Request

    user = get_user_by_email(conn, email)

    if user:
        return jsonify({'error': 'Používateľské meno alebo email už existuje'}), 400  # Bad Request

    if not is_password_valid(password):
        return jsonify({'error': 'Heslo musí obsahovať aspoň 8 až 50 znakov, 1 veľké písmeno, '
                                 '1 malé písmeno, 1 číslo a 1 špeciálny znak'
                        }), 400  # Bad Request

    # check if username has only letters and numbers and is at least 3 characters long and at most 20 characters long

    new_user = User(id=None, username=username, email=email, password=password)
    save_user(conn, new_user)
    session['username'] = username
    key = data['key']
    iv = data['iv']
    session['key'] = cryptog.decryptMessage2(key)
    session['iv'] = cryptog.decryptMessage2(iv)
    return jsonify({'message': 'Používateľ úspešne zaregistrovaný'}), 201  # Created


# login endpoint from form by given username and password
@app.route('/login', methods=['POST'])
def login():
    try:
        data = request.get_json()
        username = cryptog.decryptMessage(data['username'])
        password = cryptog.decryptMessage(data['password'])
        key = cryptog.decryptMessage2(data['key'])
        iv = cryptog.decryptMessage2(data['iv'])

        if not username or not password:
            return jsonify({'error': 'Všetky polia musia byť vyplnené'}), 400  # Bad Request

        user = get_user_by_username(conn, username)

        if not user:
            return jsonify({'error': 'Nesprávne používateľské meno alebo heslo'}), 400  # Bad Request

        if user.lock_datetime is not None:
            if datetime.now() - user.lock_datetime > timedelta(seconds=300):
                unlock_user(conn, user.username)
                user.login_attemps = 0

        if user.login_attemps > 0 and datetime.now() - user.last_login > timedelta(days=1):
            unlock_user(conn, user.username)
            user.login_attemps = 0

        if user.login_attemps >= 5:
            if user.lock_datetime is None:
                set_user_lock_datetime_now(conn, user.username)
            else:
                return jsonify({'error': 'Používateľ je zablokovaný (štandardne 5min.)'}), 400  # Bad Request

            return jsonify(
                {'error': 'Priveľa chybných pokusov o prihlásenie, používateľ bol zablokovaný'}), 400  # Bad Request

        check = check_user_password(conn, username, password)

        if check:
            session['username'] = username
            session['key'] = key
            session['iv'] = iv
            unlock_user(conn, user.username)
            update_user_last_login(conn, user.username)
            return jsonify({'message': 'Prihlásenie prebehlo úspešne'}), 200  # OK
        else:
            update_user_login_attempts(conn, user.username, user.login_attemps + 1)
            if user.login_attemps + 1 >= 5:
                set_user_lock_datetime_now(conn, user.username)
                return jsonify(
                    {'error': 'Priveľa chybných pokusov o prihlásenie, používateľ bol zablokovaný'}), 400  # Bad Request
            return jsonify({'error': 'Nesprávne používateľské meno alebo heslo'}), 400  # Bad Request
    except Exception as e:
        app.logger.info(e)
        return jsonify({'error': 'Nesprávne používateľské meno alebo heslo'}), 400  # Bad Request


@app.route('/logout', methods=['GET'])
def logout():
    if 'username' in session:
        session.clear()
        return redirect(url_for('index'))
    return redirect(url_for('index'))


@app.route('/home', methods=['GET'])
def home():
    if 'username' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])
    # check if user has public key
    if user.public_key is None:
        exist_public_key = False
    else:
        exist_public_key = True

    return render_template('home.html', username=session['username'], exist_public_key=exist_public_key,
                           key=type(session['key']), iv=type(session['iv']))


@app.route('/public-key')
def send_public_key():
    return send_file('./keys/public_key.pem')


@app.route('/terminal')
def terminal():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])
    # check if user has public key
    if user.public_key is None:
        exist_public_key = False
    else:
        exist_public_key = True
    record_data = get_types_for_select()
    return render_template('terminal.html', record_data=record_data, username=session['username'],
                           exist_public_key=exist_public_key)


@app.route('/add-record', methods=['POST'])
def add_record():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])
    # check if user has public key
    if user.public_key is None:
        exist_public_key = False
    else:
        exist_public_key = True

    if 'arrive' in request.form:
        transition = RecordType.PRICHOD.value
        transition_name = get_record_name(transition)

    elif 'departure' in request.form:
        transition = RecordType.ODCHOD.value
        transition_name = get_record_name(transition)
    else:
        return redirect(url_for('terminal'))

    formatted_datetime_str = request.form.get('formatted_datetime')

    # Convert the string to a datetime object
    datetime_obj = datetime.strptime(formatted_datetime_str, '%d. %m. %Y %H:%M:%S')

    new_record = record.Record(user.id, transition, datetime_obj, RecordAuthor.VIRT_TERMINAL.value, transition_name,
                               "Záznam z terminálu")
    try:
        new_record.save_record(conn)
        return render_template('terminal.html', username=session['username'],
                               exist_public_key=exist_public_key, message="Záznam bol uložený")
    except Exception as e:
        return render_template('terminal.html', username=session['username'],
                               exist_public_key=exist_public_key, message=e)


def is_period_in_periods(period, periods):
    year = int(period[:4])
    month = int(period[4:])
    for p in periods:
        if p['year'] == year and p['month'] == month:
            return True
    return False


@app.route('/attend')
def attend():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    return render_template('attend.html')


@app.route('/attend-filter', methods=['POST'])
def attend_filter():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    period = request.form['period']
    periods = record.get_distinct_month_year_combinations_attend(conn, user.id)

    if periods is None or len(periods) == 0:
        pass
    elif not is_period_in_periods(period, periods):
        session['period'] = str(periods[0]['year']) + str(periods[0]['month'])
    else:
        session['period'] = period

    return render_template('attend.html')


@app.route('/attend-data', methods=['GET'])
def attend_data():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])
    periods = record.get_distinct_month_year_combinations_attend(conn, user.id)

    if periods is None or len(periods) == 0:
        month = datetime.now(SR).month
        year = datetime.now(SR).year
    elif 'period' not in session or session['period'] is None or not is_period_in_periods(session['period'], periods):
        session['period'] = str(periods[0]['year']) + str(periods[0]['month'])
        month = int(periods[0]['month'])
        year = int(periods[0]['year'])
    else:
        month = int(session['period'][4:])
        year = int(session['period'][:4])

    period = str(year) + str(month)
    records = record.get_records_by_user_id_month_year_types(conn, user.id, month, year,
                                                             [RecordType.PRICHOD.value, RecordType.ODCHOD.value])

    data = {'username': session['username'],
            'records': records,
            'periods': periods,
            'selected_period': period}

    jsonData = json.dumps(data, cls=DateTimeEncoder)
    encrypredData = encryptAES(session['key'], session['iv'], jsonData)
    return base64.b64encode(encrypredData).decode('utf-8')


def parse_holiday_data(data):
    output = []
    current_start = None
    current_end = None

    for item in data:
        if item['type'] == '3':  # Type '3' indicates the start of a holiday
            if current_start is None or item['datetime'] < current_start['type']:
                current_start = item
        else:  # Type '4' indicates the end of a holiday
            current_end = item

        if current_start and current_end:
            # convert current_start and current_end to yyyy-mm-dd format
            current_start['datetime'] = current_start['datetime'].strftime('%Y-%m-%d')
            current_end['datetime'] = current_end['datetime'].strftime('%Y-%m-%d')

            output.append([current_start['name'],
                           current_start['value'],
                           current_start['datetime'],
                           current_end['datetime'],
                           str(current_start['id']),
                           str(current_end['id'])])
            current_start = None
            current_end = None

    return output


@app.route('/vacation')
def vacation():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    return render_template('vacation.html')


@app.route('/vacation-add', methods=['POST'])
def vacation_add():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    try:
        data = request.get_json()
        start_date = data['start_date']
        end_date = data['end_date']
        name = data['name']
        val = data['description']
        start_date = decryptAES(session['key'], session['iv'], start_date)
        end_date = decryptAES(session['key'], session['iv'], end_date)
        name = decryptAES(session['key'], session['iv'], name)
        val = decryptAES(session['key'], session['iv'], val)
        start_date = datetime.strptime(start_date, '%Y-%m-%d')
        end_date = datetime.strptime(end_date, '%Y-%m-%d')
        end_date = end_date + timedelta(days=1)
    except:
        return jsonify({'error': 'Nesprávny formát dátumu'}), 400
    # add 1 day to end date

    if start_date > end_date:
        return jsonify({'error': "Počiatočný čas je väčší ako koncový"}), 400

    if name == "" or val == "":
        return jsonify({'error': "Nezadaný názov alebo popis"}), 400

    new_record = record.Record(user.id, RecordType.DOVOLENKA_OD.value, start_date, None, name, val)
    new_record.save_record(conn)
    new_record = record.Record(user.id, RecordType.DOVOLENKA_DO.value, end_date, None, name, val)
    new_record.save_record(conn)

    return jsonify({'message': "Success"}), 200


@app.route('/vacation-data', methods=['GET'])
def vacation_data():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])
    # check if user has public key
    if user.public_key is None:
        exist_public_key = False
    else:
        exist_public_key = True

    records = record.get_records_by_user_id_and_types(conn, user.id,
                                                      [RecordType.DOVOLENKA_OD.value, RecordType.DOVOLENKA_DO.value])
    output = parse_holiday_data(records)
    real_holidays = get_minus_day(output.copy())
    data = {'username': session['username'],
            'exist_public_key': exist_public_key,
            'holidays': output,
            'real_holidays': real_holidays}

    jsonData = json.dumps(data, cls=DateTimeEncoder)
    encrypredData = encryptAES(session['key'], session['iv'], jsonData)
    return base64.b64encode(encrypredData).decode('utf-8')


def validate_records_del(rec_to_del, records):
    records_id = []
    for rec in records:
        records_id.append(rec['id'])

    for rec in rec_to_del:
        if rec not in records_id:
            return False
    return True


def get_minus_day(records):
    real_holidays = []
    for rec in records:
        # Create a copy of the date string
        date_copy = datetime.strptime(rec[3], '%Y-%m-%d') - timedelta(days=1)
        date_copy = date_copy.strftime('%d. %m. %Y')
        date_copy2 = datetime.strptime(rec[2], '%Y-%m-%d')
        date_copy2 = date_copy2.strftime('%d. %m. %Y')

        # Append a copy of the record to 'real_holidays'
        real_holidays.append([rec[0], rec[1], date_copy2, date_copy, rec[4], rec[5]])

    return real_holidays


@app.route('/vacation-delete', methods=['POST'])
def vacation_delete():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    records = record.get_records_by_user_id_and_types(conn,
                                                      user.id,
                                                      [RecordType.DOVOLENKA_OD.value,
                                                       RecordType.DOVOLENKA_DO.value])
    try:
        # get data from request as plain text
        data = request.get_data(as_text=True)
        data = decryptAES(session['key'], session['iv'], data)
        data_dict = json.loads(data)  # Parse the JSON data
        request_rec = data_dict.get("rec", [])
    except:
        return jsonify({'error': 'Nesprávny formát dátumu'}), 400
    rec_to_del = []
    for rec in request_rec:
        splitted = rec.split('-')
        rec_to_del.append(int(splitted[0]))
        rec_to_del.append(int(splitted[1]))

    if not validate_records_del(rec_to_del, records):
        return jsonify({'error': 'Nesprávne ID záznamov'}), 400

    try:
        record.delete_records_by_ids(conn, rec_to_del)
        return jsonify({'message': "Success"}), 200
    except:
        return jsonify({'error': 'Nepodarilo sa zmazať záznamy'}), 400


@app.route('/worktime', methods=['GET'])
def worktime():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    return render_template('worktime.html')


# Custom JSON encoder for datetime, date, timedelta, decimal objects
class DateTimeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%d. %m. %Y %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%d. %m. %Y')
        elif isinstance(obj, timedelta):
            return str(obj)
        elif isinstance(obj, Decimal):
            return int(obj)
        return json.JSONEncoder.default(self, obj)


@app.route('/worktime-data', methods=['GET'])
def worktime_data():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    periods = record.get_distinct_month_year_combinations_attend(conn, user.id)
    if periods is None or len(periods) == 0:
        month = datetime.now(SR).month
        year = datetime.now(SR).year
    elif 'period' not in session or session['period'] is None or not is_period_in_periods(session['period'], periods):
        session['period'] = str(periods[0]['year']) + str(periods[0]['month'])
        month = int(periods[0]['month'])
        year = int(periods[0]['year'])
    else:
        month = int(session['period'][4:])
        year = int(session['period'][:4])

    period = str(year) + str(month)
    records = record.get_records_by_user_id_month_year_types(conn, user.id, month, year,
                                                             [RecordType.PRICHOD.value, RecordType.ODCHOD.value])
    worktimes_month = record.get_worktime_for_month_year(conn, user.id, month, year)
    worktime_total, workdays = make_worktime_sum(worktimes_month)
    fond = make_month_fond(month, year)
    delete_data = make_data_for_delete(worktimes_month)
    data = {'records': records,
            'worktimes': worktimes_month,
            'worktime_total': worktime_total,
            'workdays': workdays,
            'fond': fond,
            'delete_data': delete_data,
            'periods': periods,
            'selected_period': period,
            'username': session['username']}
    jsonData = json.dumps(data, cls=DateTimeEncoder)
    encrypredData = encryptAES(session['key'], session['iv'], jsonData)
    return base64.b64encode(encrypredData).decode('utf-8')


def make_worktime_sum(records):
    worktime_sum = 0
    workdays = 0
    for rec in records:
        # if worktime is not 0 than add it to sum and count it
        if rec['worktime'] != 0:
            worktime_sum += rec['worktime']
            workdays += 1

    return worktime_sum, workdays


def make_month_fond(month, year):
    workdays = 0
    for day in range(1, monthrange(year, month)[1] + 1):
        if datetime(year, month, day).weekday() < 5:
            workdays += 1
    return workdays * 8


def make_data_for_delete(records):
    data = []
    for rec in records:
        date = rec['date']
        records = []
        for i in range(0, len(rec['records']), 2):
            start_time = rec['records'][i]['datetime']
            end_time = rec['records'][i + 1]['datetime']
            start_id = rec['records'][i]['id']
            end_id = rec['records'][i + 1]['id']
            records.append(
                {'start_time': start_time, 'end_time': end_time, 'start_id': str(start_id), 'end_id': str(end_id)})
        data.append({'date': date, 'records': records})
    return data


@app.route('/worktime-filter', methods=['POST'])
def worktime_filter():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])
    period = request.form['period']
    periods = record.get_distinct_month_year_combinations_attend(conn, user.id)

    if periods is None or len(periods) == 0:
        pass
    elif not is_period_in_periods(period, periods):
        session['period'] = str(periods[0]['year']) + str(periods[0]['month'])
    else:
        session['period'] = period

    return render_template('worktime.html')


@app.route('/add-attend', methods=['POST'])
def add_attend():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    try:
        data = request.get_json()
        start_date = data['start_date']
        end_date = data['end_date']
        start_time = data['start_time']
        end_time = data['end_time']
        description = data['description']
        start_date = decryptAES(session['key'], session['iv'], start_date)
        end_date = decryptAES(session['key'], session['iv'], end_date)
        start_time = decryptAES(session['key'], session['iv'], start_time)
        end_time = decryptAES(session['key'], session['iv'], end_time)
        description = decryptAES(session['key'], session['iv'], description)
        if description is None:
            description = ''
        start_datetime = datetime.strptime(start_date + ' ' + start_time, '%Y-%m-%d %H:%M')
        end_datetime = datetime.strptime(end_date + ' ' + end_time, '%Y-%m-%d %H:%M')
    except:
        message = 'Nevyplnené alebo nesprávne vyplnené údaje'
        return jsonify({'error': message}), 400

    if start_date is None or end_date is None or start_time is None or end_time is None or start_datetime > end_datetime:
        if start_date is None or end_date is None or start_time is None or end_time is None:
            message = "Musíte vyplniť všetky polia"
        else:
            message = "Príchod musí byť skôr ako odchod"
        return jsonify({'error': message}), 400

    new_start_record = record.Record(user.id, RecordType.PRICHOD.value, start_datetime, RecordAuthor.USER.value,
                                     RecordNames.PRICHOD.value, description)
    new_end_record = record.Record(user.id, RecordType.ODCHOD.value, end_datetime, RecordAuthor.USER.value,
                                   RecordNames.ODCHOD.value, description)
    new_start_record.save_record(conn)
    new_end_record.save_record(conn)

    return jsonify({'success': 'success'}), 200


@app.route('/delete-attend', methods=['POST'])
def delete_attend():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    user = get_user_by_username(conn, session['username'])

    try:
        # get data from request as plain text
        data = request.get_data(as_text=True)
        data = decryptAES(session['key'], session['iv'], data)
        data_dict = json.loads(data)  # Parse the JSON data
        request_rec = data_dict.get("rec", [])
    except:
        return jsonify({'error': 'Nesprávne dáta'}), 400

    if len(request_rec) == 0:
        return jsonify({'error': "Nevybrali ste žiadny záznam na zmazanie"}), 400
    rec_to_del = []
    for rec in request_rec:
        splitted = rec.split('-')
        rec_to_del.append(int(splitted[0]))
        rec_to_del.append(int(splitted[1]))

    records = record.get_records_by_user_id_and_types(conn,
                                                      user.id,
                                                      [RecordType.PRICHOD.value,
                                                       RecordType.ODCHOD.value])
    if not validate_records_del(rec_to_del, records):
        return jsonify({'error': "Záznamy neboli odstránené"}), 400

    try:
        record.delete_records_by_ids(conn, rec_to_del)
    except:
        return jsonify({'error': "Záznamy neboli odstránené"}), 400

    return jsonify({'message': "Záznam bol úspešne vymazaný"}), 200


@app.route('/change_data', methods=['POST'])
def change_data():
    if 'username' not in session or 'key' not in session or 'iv' not in session:
        return redirect(url_for('index'))

    data = request.get_json()
    user_mess = data['message']
    decrypted_message = decryptAES(session['key'], session['iv'], user_mess)
    message = "Hello world"
    key = session['key']
    iv = session['iv']
    encryptedMessage = encryptAES(key, iv, message)
    encrypted_message = base64.b64encode(encryptedMessage).decode('utf-8')
    return jsonify({'message': encrypted_message})


@app.route('/favicon.ico')
def favicon():
    return send_from_directory(os.path.join(app.root_path, 'static'),
                               'images/logoSmaller.ico', mimetype='image/vnd.microsoft.icon')


if __name__ == '__main__':
    is_debug_mode = os.environ.get('FLASK_ENV') == 'development'
    app.run(host='0.0.0.0', port=5001, debug=is_debug_mode)
