from datetime import datetime
import psycopg2
import psycopg2.extras
import random
import string
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import load_pem_public_key
from utils.password_functions import hash_password, check_password
import pytz
SR = pytz.timezone('Europe/Bratislava')

class User:
    def __init__(self, id, username, email, login_attemps=0, lock_datetime=None, password=None, last_login=None,
                 public_key=None):
        self.id = id
        self.username = username
        self.email = email
        self.password = password
        self.login_attemps = login_attemps
        self.lock_datetime = lock_datetime
        self.last_login = last_login
        self.public_key = public_key


def get_user_by_id(conn, user_id):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT id,username,email FROM users WHERE id = %s", (user_id,))
    user_data = cursor.fetchone()
    cursor.close()

    if user_data:
        user = User(id=user_data['id'],
                    username=user_data['username'],
                    email=user_data['email'],
                    login_attemps=user_data['login_attemps'],
                    lock_datetime=user_data['lock_datetime'],
                    last_login=user_data['last_login'])

        if user_data['public_key']:
            user.public_key = load_pem_public_key(user_data['public_key'].encode('utf-8'))
        else:
            user.public_key = None

        return user
    else:
        return None


def get_user_by_email(conn, email):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM users WHERE email = %s", (email,))
    user_data = cursor.fetchone()
    cursor.close()

    if user_data:
        user = User(id=user_data['id'],
                    username=user_data['username'],
                    email=user_data['password'],
                    login_attemps=user_data['login_attemps'],
                    lock_datetime=user_data['lock_datetime'],
                    last_login=user_data['last_login'])

        if user_data['public_key']:
            user.public_key = load_pem_public_key(user_data['public_key'].encode('utf-8'))
        else:
            user.public_key = None

        return user
    else:
        return None


def get_user_by_username(conn, username):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT * FROM users WHERE username = %s", (username,))
    user_data = cursor.fetchone()
    cursor.close()

    if user_data:
        user = User(id=user_data['id'],
                    username=user_data['username'],
                    email=user_data['email'],
                    login_attemps=user_data['login_attemps'],
                    lock_datetime=user_data['lock_datetime'],
                    last_login=user_data['last_login'])

        if user_data['public_key']:
            user.public_key = load_pem_public_key(user_data['public_key'].encode('utf-8'))
        else:
            user.public_key = None

        return user
    else:
        return None


def save_user(conn, user):
    cursor = conn.cursor()
    cursor.execute(
        "INSERT INTO users (username, email, password, login_attemps, last_login, public_key) VALUES (%s, %s, %s, %s, %s, %s)",
        (user.username, user.email, hash_password(user.password), user.login_attemps, datetime.now(SR), user.public_key))
    cursor.close()
    conn.commit()


def initialize_database_with_random_users(conn, num_users=20):
    cursor = conn.cursor()

    for i in range(num_users):
        # Generate random username (8 characters) and email
        username = 'user' + str(i)
        email = f'{username}@user.com'
        password = 'Aaa1234.'

        new_user = User(id=None,
                        username=username,
                        email=email,
                        password=password,
                        login_attemps=0,
                        lock_datetime=None,
                        last_login=datetime.now(SR),
                        public_key=None)
        save_user(conn, new_user)

    cursor.close()
    conn.commit()


def create_user_table(conn):
    cursor = conn.cursor()
    users = "users"
    # Define the CREATE TABLE statement
    create = f"CREATE TABLE IF NOT EXISTS {users} (id serial PRIMARY KEY,username VARCHAR (50) NOT NULL,email VARCHAR (100) NOT NULL,password VARCHAR(255) NOT NULL, login_attemps smallint, lock_datetime timestamp, last_login timestamp, public_key text);"
    cursor.execute(create)
    conn.commit()
    cursor.close()


# return if user with username has correct password
def check_user_password(conn, username, password):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT password FROM users WHERE username = %s", (username,))
    user_data = cursor.fetchone()
    cursor.close()

    if user_data:
        return check_password(password, user_data['password'])
    else:
        return False


def update_user_login_attempts(conn, username, login_attempts):
    cursor = conn.cursor()
    cursor.execute("UPDATE users SET login_attemps = %s WHERE username = %s", (login_attempts, username))
    cursor.close()
    conn.commit()


# get attempts of user with username
def get_user_login_attempts(conn, username):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT login_attemps FROM users WHERE username = %s", username)
    user_data = cursor.fetchone()
    cursor.close()

    if user_data:
        return user_data['login_attemps']
    else:
        return None


# get lock datetime of user with username
def get_user_lock_datetime(conn, username):
    cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)
    cursor.execute("SELECT lock_datetime FROM users WHERE username = %s", (username,))
    user_data = cursor.fetchone()
    cursor.close()

    if user_data and 'lock_datetime' in user_data:
        return user_data['lock_datetime']

    return None


# set lock datetime of user with username
def set_user_lock_datetime(conn, username, lock_datetime=None):
    cursor = conn.cursor()

    # Update the user's lock_datetime in the database
    cursor.execute("UPDATE users SET lock_datetime = %s WHERE username = %s", (lock_datetime, username))

    conn.commit()
    cursor.close()


def set_user_lock_datetime_now(conn, username):
    current_datetime = datetime.now(SR)
    set_user_lock_datetime(conn, username, current_datetime)


def unlock_user(conn, username):
    # Set the lock_datetime to NULL to unlock the user
    set_user_lock_datetime(conn, username, None)
    # Set the login_attemps to 0
    update_user_login_attempts(conn, username, 0)


def update_user_last_login(conn, username):
    cursor = conn.cursor()
    cursor.execute("UPDATE users SET last_login = %s WHERE username = %s", (datetime.now(SR), username))
    cursor.close()
    conn.commit()


def store_public_key_for_user(conn, username, public_key_pem):
    try:
        # Load the PEM-encoded public key
        public_key = load_pem_public_key(public_key_pem)

        # Serialize the public key to a PEM-encoded string
        public_key_str = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo
        ).decode('utf-8')

        cursor = conn.cursor()

        # Check if a row with the given username exists in the database
        cursor.execute("SELECT * FROM users WHERE username = %s", (username,))
        existing_record = cursor.fetchone()

        if existing_record:
            # If the record already exists, update the public key
            cursor.execute("UPDATE users SET public_key = %s WHERE username = %s",
                           (public_key_str, username))
        else:
            # If the record doesn't exist, insert a new one
            cursor.execute("INSERT INTO users (username, public_key) VALUES (%s, %s)",
                           (username, public_key_str))

        # Commit the transaction and close the cursor
        conn.commit()
        cursor.close()

        return True

    except Exception as e:
        return False


def get_public_key_for_user(conn, username):
    try:
        cursor = conn.cursor(cursor_factory=psycopg2.extras.RealDictCursor)

        # Retrieve the public key for the given username
        cursor.execute("SELECT public_key FROM users WHERE username = %s", (username,))
        public_key_str = cursor.fetchone()

        cursor.close()

        if public_key_str:
            return public_key_str['public_key']
        else:
            return False

    except Exception as e:
        return False
