import os
from typing import Annotated

from cryptography.hazmat.primitives import serialization
from fastapi import FastAPI, Depends, HTTPException, Header
from sqlalchemy.orm import Session

from services.user_service import add_public_key_for_user, add_encrypted_text
from database.connect import SessionLocal
from models.models import User, UserRequest, AddPublicKeyRequest, AddEncryptedMessage, ChangePasswordRequest, Candidate
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
import base64
from pydantic import BaseModel
from database.auth import hash_password, verify_password, create_access_token, ACCESS_TOKEN_EXPIRE_MINUTES, \
    validate_password_complexity, get_user_from_token, get_user_email_from_token
from utils.functions import generate_key_pair, encrypt_message, decrypt_message
from datetime import timedelta, datetime
from fastapi.middleware.cors import CORSMiddleware
import json

app = FastAPI()

app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# app = APIRouter(prefix="/api")

# Vytvorenie inštancie Fernet pre šifrovanie a dešifrovanie dát
symetric_key = Fernet.generate_key()
fernet = Fernet(symetric_key)
failed_attempts = {}
MAX_ATTEMPTS = 5
COOLDOWN_PERIOD = timedelta(minutes=15)


class DecryptionRequest(BaseModel):
    private_key_pem: str


def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


# Function to compute the hash of the data
def compute_hash(data: str) -> bytes:
    digest = hashes.Hash(hashes.SHA256(), backend=default_backend())
    digest.update(data.encode())
    return digest.finalize()


# Funkcia na zašifrovanie dát


def encrypt_data(data: str) -> str:
    return fernet.encrypt(data.encode()).decode()


# Funkcia na dešifrovanie dát


def decrypt_data(encrypted_data: str) -> str:
    return fernet.decrypt(encrypted_data.encode()).decode()


@app.get("/api/get_candidates")
async def get_candidates(db: Session = Depends(get_db)):
    candidates = db.query(Candidate).all()
    if not candidates:
        raise HTTPException(status_code=404, detail="No candidates found")
    return candidates

@app.get("/api/get_user_data")
async def get_user_data(user: User = Depends(get_user_from_token)):
    return user.public_key


@DeprecationWarning
@app.post("/api/encrypt/{user_id}")
async def encrypt_user_data(user_id: int, db: Session = Depends(get_db)):
    user = db.query(User).filter(User.id == user_id).first()
    if user is None:
        raise HTTPException(status_code=404, detail="User not found")

    # Zašifrovanie dát
    encrypted_data = encrypt_data(user.text)

    private_key = generate_key_pair()
    public_key = private_key.public_key()
    encyphered_symetric_key = encrypt_message(public_key, symetric_key)

    # Uloženie šifrovaných dát do databázy
    user.text = encrypted_data
    user.encrypted_key = base64.b64encode(encyphered_symetric_key).decode()
    data_hash = compute_hash(encrypted_data)
    user.data_hash = base64.b64encode(data_hash).decode()

    db.commit()
    key_data = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.BestAvailableEncryption(
            str(symetric_key).encode())
    )
    return {
        "message": "Data encrypted successfully",
        "decypher_key": key_data
    }

@DeprecationWarning
@app.get("/api/decrypt/{user_id}")
async def decrypt_user_data(user_id: int, decryption_request: DecryptionRequest, db: Session = Depends(get_db)):
    user = db.query(User).filter(User.id == user_id).first()
    if user is None:
        raise HTTPException(status_code=404, detail="User not found")

    # Check if encrypted text and key exist for the user
    if user.text is None or not user.encrypted_key:
        raise HTTPException(
            status_code=400, detail="No encrypted data or key found for this user")

    # Overenie Integrity
    current_hash = compute_hash(user.text)
    stored_hash = base64.b64decode(user.data_hash.encode())
    if current_hash != stored_hash:
        raise HTTPException(
            status_code=409, detail="Data integrity compromised")

    # Load the provided private key
    private_key = serialization.load_pem_private_key(
        decryption_request.private_key_pem.encode(),
        password=str(symetric_key).encode(),
        backend=default_backend()
    )

    # Decrypt the symmetric key using the private key
    decrypted_symmetric_key = decrypt_message(
        private_key, base64.b64decode(user.encrypted_key))

    # Use the decrypted symmetric key to decrypt the user data
    fernet_dec = Fernet(decrypted_symmetric_key)
    decrypted_data = fernet_dec.decrypt(user.text.encode()).decode()

    return {"decrypted_data": decrypted_data}


with open('password-list.txt', 'r') as f:
    common_passwords = set([line.strip() for line in f.readlines()])


@app.post("/api/register")
async def register_user(user: UserRequest, db: Session = Depends(get_db)):
    if user.password in common_passwords:
        raise HTTPException(
            status_code=400,
            detail="The provided password is too common. Please choose a different password."
        )
    if not validate_password_complexity(user.password):
        raise HTTPException(
            status_code=400,
            detail="Password must be at least 8 characters long, contain at least one uppercase letter, one lowercase letter, one number, and one special character."
        )

    hashed_password = hash_password(user.password)

    db_user = User(email=user.email, hashed_password=hashed_password)
    db.add(db_user)
    db.commit()

    return {"message": "User registered successfully"}


@app.post("/api/change_password")
async def change_password(user: ChangePasswordRequest, db: Session = Depends(get_db)):
    # Fetch the user from the database
    db_user = db.query(User).filter(User.email == user.email).first()
    if not db_user:
        raise HTTPException(status_code=404, detail="User not found")

    # Verify old password
    if not verify_password(db_user.hashed_password, user.old_password):
        raise HTTPException(
            status_code=400, 
            detail="Incorrect old password"
        )

    # Check if new password and confirmation match
    if user.new_password != user.new_password_confirmation:
        raise HTTPException(
            status_code=400,
            detail="New password and confirmation do not match"
        )

    # Check if the new password is too common
    if user.new_password in common_passwords:
        raise HTTPException(
            status_code=400,
            detail="The provided password is too common. Please choose a different password."
        )

    # Validate new password complexity
    if not validate_password_complexity(user.new_password):
        raise HTTPException(
            status_code=400,
            detail="Password must be at least 8 characters long, contain at least one uppercase letter, one lowercase letter, one number, and one special character."
        )

    # Hash the new password and update it in the database
    hashed_password = hash_password(user.new_password)
    db_user.hashed_password = hashed_password
    db.commit()

    return {"message": "Password changed successfully"}


@app.post("/api/add_public_key/")
async def add_public_key(public_key_request: AddPublicKeyRequest, user: User = Depends(get_user_from_token)):
    add_public_key_for_user(user.email, public_key_request.public_key)
    return {"message": "Public key added successfully"}


@app.get("/api/auth/me")
async def auth_me(authorization: str = Header(None, title="Authorization")):
    try:
        token = authorization.split(" ")[1]
        user = get_user_email_from_token(token)

    except Exception as e:
        raise HTTPException(
            status_code=401, detail="Invalid token"
        )
    return {"email": user["sub"]}


@app.post("/api/login")
async def login_user(form_data: UserRequest, db: Session = Depends(get_db)):
    current_time = datetime.utcnow()

    if form_data.email in failed_attempts:
        attempts, last_attempt_time = failed_attempts[form_data.email]
        if attempts >= MAX_ATTEMPTS and (current_time - last_attempt_time) < COOLDOWN_PERIOD:
            raise HTTPException(
                status_code=429, detail="Too many failed attempts. Please try again later."
            )

    user = db.query(User).filter(User.email == form_data.email).first()

    if not user or not verify_password(user.hashed_password, form_data.password):
        if form_data.email in failed_attempts:
            attempts, _ = failed_attempts[form_data.email]
            failed_attempts[form_data.email] = (attempts + 1, current_time)
        else:
            failed_attempts[form_data.email] = (1, current_time)

        raise HTTPException(
            status_code=401, detail="Incorrect email or password"
        )

    if form_data.email in failed_attempts:
        del failed_attempts[form_data.email]

    access_token_expires = timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    access_token = create_access_token(
        data={"sub": user.email}, expires_delta=access_token_expires
    )
    return {"access_token": access_token, "token_type": "bearer"}

@app.post("/api/add_encrypted_message/")
async def add_encrypted_message(data: AddEncryptedMessage, user: User = Depends(get_user_from_token)):
    add_encrypted_text(user.email, data.encrypted_message)
    return {"message": "Encrypted message saved successfully", "encrypted_message": data.encrypted_message}

@app.get("/api/get_encrypted_message/")
async def get_encrypted_message(user: User = Depends(get_user_from_token)):
    return {"message": "Encrypted message fetched successfully", "encrypted_message": user.vote}


@app.get("/api/hello/{name}")
async def say_hello(name: str):
    return {"message": f"Hello {name}"}


@app.get("/api")
async def root():
    return {"message": "Working"}


# _app.include_router(app)
os.system("alembic upgrade head")
