const express = require("express");
const db = require("../db/connect");
const jwt = require("jsonwebtoken");
const rateLimit = require("express-rate-limit");
const { requireProps, consoleLogger } = require("../services/utils");

const searchController = express.Router();
const searchLimiter = rateLimit({
  windowMs: 90 * 1000, // 1.5 min
  max: 15,
  message: "Too many search requests."
})

searchController.post("/searchOffer", searchLimiter, async (req, res) => {
  if (!requireProps(['phrase'], req.body)) {
    return res.status(400).send({error: "Required field is missing!"})
  }
  const searchPhrase = req.body.phrase
  if (searchPhrase.length <= 3) {
    return res.status(400).send({error: "Too short search phrase!"})
  }
  const parts = searchPhrase.split(' ')
  if (parts.length > 1) {
    return res.status(400).send({error: "Too many search parameters!"})
  }

  let results = []

  const query = " \
    SELECT f.id, f.connection_id, f.price, f.departure, f.arrival, c.dept_city_name, c.dept_city_code, c.arr_city_name, c.arr_city_code, c.code, c.duration_min \
    FROM flights f JOIN connections c ON f.connection_id = c.id \
    WHERE f.id IN ( SELECT id FROM ( \
      SELECT f.id, to_char(f.departure, 'DD.MM.YYYY') as col1, LOWER(c.dept_city_name) as col2, LOWER(c.arr_city_name) as col3, \
      LOWER(c.code) as col4 FROM flights f JOIN connections c ON f.connection_id = c.id WHERE departure > now() \
    ) search WHERE col1 LIKE $1 OR remove_accents(col2) LIKE $1 OR remove_accents(col3) LIKE $1 OR col4 LIKE $1 LIMIT 30) \
  "
  for (let i = 0; i < parts.length; i++) {
    const search = parts[i].toLowerCase().normalize('NFD').replace(/[\u0300-\u036f]/g, '')
    const partResults = await db.query(query, [`%${search}%`])
    for (let p = 0; p < partResults.rows.length; p++) {
      if (!results.some(row => row.fl_id === partResults.rows[p].fl_id)) {
        results.push(partResults.rows[p])
      }
    }
  }
  consoleLogger([`\t└ found ${results.length} results total`])
  return res.status(200).send({flights: results})
})

searchController.get("/getConnections", searchLimiter, async (req, res) => {
  try {
    const connections = await db.query('SELECT id, dept_city_name, arr_city_name FROM connections');
    return res.status(200).send(connections.rows);
  } catch(err) {
    return res.status(500).send();
  }
})

searchController.post("/searchFlights", searchLimiter, async (req, res) => {
  const connectionId = req.body.connectionId;
  const dateFrom = req.body.dateFrom;
  const dateTo = req.body.dateTo;
  try {
    const flights = await db.query(
      'SELECT f.id, f.connection_id, f.price, f.departure, f.arrival, c.dept_city_name, c.dept_city_code, c.arr_city_name, c.arr_city_code, c.code, c.duration_min \
        FROM flights f JOIN connections c ON f.connection_id = c.id  \
        WHERE c.id = $1 AND f.departure BETWEEN $2 AND $3 \
        LIMIT 20', [connectionId, dateFrom, dateTo])
    return res.status(200).send(flights.rows);
  } catch(err) {
    return res.status(500).send();
  }
})

module.exports = searchController;
