const express = require("express");
const db = require("../db/connect");
const verifyJWT = require("../middleware/verifyJWT");
const utils = require("../services/utils");
const pdfService = require("../services/pdfService");
const { consoleLogger } = require("../services/utils");
const requestLimiter = require("../middleware/requestLimiter");
const path = require("path");
const fs = require("fs");
const { symmetricEncryptData } = require("../services/utils");

const ticketController = express.Router()

ticketController.get("/api/availableFlights", async (req, res) => {
  const result = await db.query("SELECT f.id, c.id as connection_id, f.price, f.departure, f.arrival, c.dept_city_name, c.dept_city_code, c.arr_city_name, c.arr_city_code, c.code, c.duration_min FROM flights f JOIN connections c ON f.connection_id = c.id WHERE departure > now();");
  res.status(200).send(result.rows);
})


ticketController.post("/api/makeReservation", verifyJWT, async (req, res) => {

  if (!utils.requireProps(['flight_id'], req.body)) {
    return res.status(400).send({error: "Missing required field!"})
  }

  const flights = await db.query('SELECT * FROM flights WHERE id = $1', [req.body.flight_id])
  if (flights.rows.length < 1) {
    return res.status(404).send({error: "Not such flight."})
  }

  const letters = ['A', 'B', 'C', 'D', 'E', 'F']
  const seat = random(1, 37) + letters[random(0, letters.length-1)]

  //const classes = ['economy', 'business', '1st class']
  const class_str = 'economy'
  
  let code = ''
  do {
    code = utils.createRandomString([5, 5, 5])
  } while((await db.query("SELECT * FROM tickets WHERE code = $1", [code])).rows.length > 0)

  const ticket = await db.query('INSERT INTO tickets (flight_id, seat, class, boarding_group, owner, code) VALUES ($1, $2, $3, $4, $5, $6) RETURNING id;',
  [req.body.flight_id, seat, class_str, random(1, 15), res.locals.user.id, code])

  let success = null

  const data = await db.query('SELECT c.code as f_code, c.dept_city_name as d_cit_n, c.arr_city_name as a_cit_n, c.dept_city_code as d_cit_c, c.arr_city_code as a_cit_c, f.departure, f. arrival, u.name, u.surname, t.seat, t.class, t.boarding_group as b_group, t.code FROM tickets t JOIN flights f ON t.flight_id = f.id JOIN users u ON t.owner = u.id JOIN connections c ON f.connection_id = c.id WHERE t.id = $1;',
  [ticket.rows[0].id]);

  try {
    success = await pdfService.createAndSavePdfTicket(code, data.rows[0])
  } catch(err) {
    consoleLogger(['[ERROR]: ' + err])
    return res.status(500).send()
  }

  if (success) {
    let newFlight;
    const userId = res.locals.user.id;
    const symmKey = Buffer.from(res.locals.symmKey, "hex");
    const vector = Buffer.from(res.locals.vector, "hex");
    try {
      newFlight = await db.query('SELECT t.code, t.class, t.seat, f.departure, c.dept_city_code, c.arr_city_code FROM tickets t INNER JOIN flights f ON t.flight_id = f.id INNER JOIN connections c ON f.connection_id = c.id WHERE t.owner = $1 AND t.id = $2', [userId, ticket.rows[0].id])
      const encryptedData = symmetricEncryptData(JSON.stringify(newFlight.rows[0]), symmKey, vector);
      res.status(200).send({data: encryptedData.data.data, tag: encryptedData.tag});
    } catch(err) {
      res.status(500).send();
    }
  } else {
    return res.status(500).send()
  }
})


ticketController.get('/ticket/:ticketId', verifyJWT, requestLimiter, async (req, res) => {
  const ticketId = req.params.ticketId;
  const symmKey = Buffer.from(res.locals.symmKey, "hex");
  const vector = Buffer.from(res.locals.vector, "hex");
  const ticketDir = process.env.MODE === "production" ? path.join("/app", "src", "generated") : path.join(__dirname, "../", "generated");

  const ticketQuery = await db.query('SELECT * FROM tickets WHERE code = $1', [req.params.ticketId]);
  if (ticketQuery.rows.length === 1) {
    if (ticketQuery.rows[0].owner !== res.locals.user.id) {
      return res.status(403).send()
    }
  } else {
    return res.status(500).send()
  }
  try {
    const pdfData = fs.readFileSync(path.join(ticketDir, ticketId + ".pdf"))
    const encryptedData = symmetricEncryptData(pdfData, symmKey, vector);
    return res.status(200).send({data: encryptedData.data.data, tag: encryptedData.tag, id: ticketId});
  } catch(err) {
    return res.status(500).send();
  }
});

ticketController.get('/api/myTickets', verifyJWT, async (req, res) => {
  const userId = res.locals.user.id;
  const symmKey = Buffer.from(res.locals.symmKey, "hex");
  const vector = Buffer.from(res.locals.vector, "hex");
  try {
    const flights = await db.query('SELECT t.code, t.class, t.seat, f.departure, c.dept_city_code, c.arr_city_code FROM tickets t INNER JOIN flights f ON t.flight_id = f.id INNER JOIN connections c ON f.connection_id = c.id WHERE t.owner = $1', [userId])
    const encryptedData = symmetricEncryptData(JSON.stringify(flights.rows), symmKey, vector);
    res.status(200).send({data: encryptedData.data.data, tag: encryptedData.tag});
  } catch(err) {
    res.status(500).send();
  }
});

function random(min, max) {
  return (Math.floor(Math.random() * (max-min)) + min)
}

module.exports = ticketController;
