const express = require("express");
const path = require("path");
const cors = require("cors");
const jsonParser = require("body-parser").json();
const logger = require("./middleware/logger");
const { consoleLogger } = require("./services/utils");
const runtimeService = require("./services/runtimeService");
const flightFactory = require("./db/factories/flightFactory");

const corsOptions = {
  origin: ["http://localhost:5173"],
  methods: "GET,HEAD,PUT,PATCH,POST,DELETE",
};

// Setup env variables
if (!process.env.EXJS_SERVER_PORT) {
  consoleLogger(["loading .env file"]);
  // [!] just for local development
  const dotenv = require("dotenv").config({ path: "../.env.local" });
} else {
  consoleLogger(["tenv variables present"]);
}

// Database
const db = require("./db/connect");

// App setup
const app = express();
app.use(jsonParser);
app.use(cors(corsOptions));

//logger middleware
app.use(logger);

//controllers
const authController = require("./controllers/authController");
app.use("/api", authController);

const cryptoController = require("./controllers/cryptoController");
app.use("/api", cryptoController);

const userController = require("./controllers/userController");
app.use("/api", userController);

const ticketController = require("./controllers/ticketController")
app.use("", ticketController)

const searchController = require("./controllers/searchController")
app.use("/api", searchController)


// Serve app
app.use(express.static(path.join("/webapp", "dist")));
app.get("*", (req, res) => {
  consoleLogger(["serving React build"]);
  if (process.env.MODE === "production") {
    res.sendFile(path.join("/webapp", "dist", "index.html"));
  } else {
    res.status(200).send({ message: "Run React dev when in developer mode" });
  }
});

// Run
async function startServer() {
  await runtimeService.checkAllKeyDependecies();

  setTimeout(() => {
    app.listen(process.env.EXJS_SERVER_PORT, () => {
      db.init();
      consoleLogger([`Server started on port ${process.env.EXJS_SERVER_PORT}`]);
    });
  }, 3000);
  
  setTimeout(() => {
    flightFactory.createFlightsIfNecessary(db)
  }, 5000)
}

startServer();
