
const passwordService = require('../services/passwordService')
const utils = require('../services/utils')

async function registerUser(db, props) {
  
  const emailRegex = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
  if (!emailRegex.test(props.email)) {
    return {ok: false, result: 'Not valid email!'}
  }

  // [DOCS] 8+ length, nums, digits, special chars 
  const passRegex = /^.*(?=.{8,})(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[!#@-_$%&?^()|{}:<>*"]).*$/
  if (!passRegex.test(props.password)) {
    return {ok: false, result: 'Password does not meet criteria!'}
  }

  if (await passwordService.isCommonPassword(props.password)) {
    return {ok: false, result: 'Common password!'}
  }
  if (!await utils.isEmailUnique(db, props.email)) {
    return {ok: false, result: 'Email already used!'}
  }

  if (await passwordService.isPasswordExposed(props.password)) {
    return {ok: false, result: 'Password exposed!'}
  }

  const { salt, hash } = passwordService.hashSaltPassword(props.password)
  const userTag = await utils.generateUserTag(db)

  try {
    db.query('INSERT INTO users (name, surname, email, system_id, hashed_password, password_salt) VALUES ($1, $2, $3, $4, $5, $6)',
    [props.firstname, props.lastname, props.email, userTag, hash, salt])
  } catch(err) {
    return {ok: false, result: 'Unknown internal error.'}
  }

  console.log('[INFO]:\t registered successfully: user', props.email);
  return {ok: true, data: {system_id: userTag}}
}

async function loginUser(db, props) {
  try {
    const user = await db.query('SELECT * FROM users WHERE email = $1', [props.email])
    if (user.rows.length > 1) {
      return {ok: false, result: 'Oops'}
    } else {
      const data = user.rows[0] 
      const success = passwordService.verifySaltyPassword(data.hashed_password, data.password_salt, props.password)
      
      console.log('[INFO]:\t login attempt: user', props.email, 'success:', success);
      return {ok: success, data: data}
    }
  } catch(err) {
    console.log('error');
    return {ok: false, result: 'Unknown internal error.'}
  }
}

module.exports = {
  registerUser,
  loginUser,
}
