const crypto = require("node:crypto");
const fs = require("fs");
const path = require("path");
const forge = require("node-forge");
const { consoleLogger } = require("./utils");

const KEYS_TARGET_PATH = "./src/runtime";
function getKeysPath() {
  return KEYS_TARGET_PATH;
}

async function generateCryptoKey() {
  if (checkForSavedKeyPair()) {
    return { ok: true, result: "key already exists" };
  }

  // DELETE other key, if some exists
  try {
    fs.unlinkSync(path.join(KEYS_TARGET_PATH, "private-key.pem"));
    fs.unlinkSync(path.join(KEYS_TARGET_PATH, "public-key.pem"));
  } catch (err) {}

  const keyPair = forge.pki.rsa.generateKeyPair({ bits: 2048, workers: 2 });
  const pubKeyPem = forge.pki.publicKeyToPem(keyPair.publicKey);
  const privKeyPem = forge.pki.privateKeyToPem(keyPair.privateKey);
  try {
    saveKeyPair(privKeyPem, pubKeyPem);
  } catch (err) {
    return { ok: false, result: "key pair save failed" };
  }
  consoleLogger(["RSA key pair created"]);
  return { ok: true, result: "key created" };
}

async function generateJwtKeys() {
  crypto.generateKeyPair(
    "rsa",
    {
      modulusLength: 2048,
      publicKeyEncoding: {
        type: "pkcs1",
        format: "pem",
      },
      privateKeyEncoding: {
        type: "pkcs8",
        format: "pem",
      },
    },
    (err, publicKey, privateKey) => {
      if (err) {
        console.error("[Error]:\t Generation of jwt key pair failed");
        return;
      }
      if (!fs.existsSync(KEYS_TARGET_PATH)) {
        console.log("[INFO]:\t Created src/runtime dir");
        fs.mkdirSync(KEYS_TARGET_PATH, { recursive: true });
      }
      fs.writeFileSync(
        path.join(KEYS_TARGET_PATH, "jwt.key"),
        privateKey,
        "utf-8"
      );
      fs.writeFileSync(
        path.join(KEYS_TARGET_PATH, "jwt_pub.key"),
        publicKey,
        "utf-8"
      );
      consoleLogger(["JWT key pair created"]);
    }
  );
  return;
}

function checkForSavedKeyPair() {
  consoleLogger(["Looking for saved key pair"]);
  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "private-key.pem"));
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "public-key.pem"));
    return true;
  } catch (err) {
    return false;
  }
}

async function saveKeyPair(privateKey, publicKey) {
  consoleLogger(["Saving key files"]);

  if (!fs.existsSync(KEYS_TARGET_PATH)) {
    console.log("[INFO]:\t Created src/runtime dir");
    fs.mkdirSync(KEYS_TARGET_PATH, { recursive: true });
  }

  fs.writeFileSync(
    path.join(KEYS_TARGET_PATH, "private-key.pem"),
    privateKey,
    "utf-8"
  );
  fs.writeFileSync(
    path.join(KEYS_TARGET_PATH, "public-key.pem"),
    publicKey,
    "utf-8"
  );
}

function generateSymKey() {
  if (checkForSavedSymKey()) {
    return { ok: true, result: "sym.key already exists" };
  }

  // DELETE other key, if some exists
  try {
    fs.unlinkSync(path.join(KEYS_TARGET_PATH, "sym-key.pem"));
    fs.unlinkSync(path.join(KEYS_TARGET_PATH, "sym-key-vector.pem"));
  } catch (err) {}

  const encKey = crypto.randomBytes(32);
  const vector = crypto.randomBytes(16);

  saveSymKey(encKey, vector);

  return { ok: true, result: "sym.key created" };
}

function checkForSavedSymKey() {
  consoleLogger(["Looking for saved sym key"]);
  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "sym-key.pem"));
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "sym-key-vector.pem"));
    return true;
  } catch (err) {
    return false;
  }
}

function saveSymKey(key, vector) {
  consoleLogger(["Saving sym.key files"]);
  fs.writeFileSync(
    path.join(KEYS_TARGET_PATH, "sym-key.pem"),
    key.toString("hex"),
    "utf-8"
  );
  fs.writeFileSync(
    path.join(KEYS_TARGET_PATH, "sym-key-vector.pem"),
    vector.toString("hex"),
    "utf-8"
  );
}

function generateHmacKey() {
  if (checkForSavedHmacKey()) {
    return { ok: true, result: "HMAC key already exists" };
  }

  // DELETE other HMAC key, if some exists
  try {
    fs.unlinkSync(path.join(KEYS_TARGET_PATH, "hmac-key.pem"));
  } catch (err) {}

  const hmacKey = crypto.randomBytes(32);

  saveHmacKey(hmacKey);

  return { ok: true, result: "HMAC key created" };
}

function checkForSavedHmacKey() {
  consoleLogger(["Looking for saved HMAC key"]);
  try {
    fs.readFileSync(path.join(KEYS_TARGET_PATH, "hmac-key.pem"));
    return true;
  } catch (err) {
    return false;
  }
}

function saveHmacKey(key) {
  consoleLogger(["Saving HMAC key file"]);
  fs.writeFileSync(
    path.join(KEYS_TARGET_PATH, "hmac-key.pem"),
    key.toString("hex"),
    "utf-8"
  );
}

function symEnc(message) {
  const { key_hex, vector_hex } = loadSavedSymKey();
  const key = Buffer.from(key_hex, "hex");
  const vector = Buffer.from(vector_hex, "hex");

  const cipher = crypto.createCipheriv("aes-256-cbc", key, vector);
  let encryptedMessage = cipher.update(message, "utf8", "hex");
  encryptedMessage += cipher.final("hex");

  const hmacKey = fs.readFileSync(
    path.join(KEYS_TARGET_PATH, "hmac-key.pem"),
    "utf-8"
  );
  const hmac = crypto.createHmac("sha256", hmacKey);
  hmac.update(encryptedMessage);
  const messageHmac = hmac.digest("hex");

  return { ok: true, result: { encryptedMessage, messageHmac } };
}

function loadSavedSymKey() {
  const key = fs.readFileSync(
    path.join(KEYS_TARGET_PATH, "sym-key.pem"),
    "utf-8"
  );
  const vector = fs.readFileSync(
    path.join(KEYS_TARGET_PATH, "sym-key-vector.pem"),
    "utf-8"
  );
  return { key_hex: key, vector_hex: vector };
}

function encryptSymKeyWithPublicKey() {
  const publicKeyPath = path.join(KEYS_TARGET_PATH, "public-key.pem");
  const publicKey = fs.readFileSync(publicKeyPath, "utf8");

  const symKeyPath = path.join(KEYS_TARGET_PATH, "sym-key.pem");
  const symKey = fs.readFileSync(symKeyPath, "utf8");

  const encryptedSymKey = crypto.publicEncrypt(
    {
      key: publicKey,
      padding: crypto.constants.RSA_PKCS1_OAEP_PADDING,
      oaepHash: "sha256",
    },
    Buffer.from(symKey)
  );

  return { ok: true, result: encryptedSymKey.toString("base64") };
}

function symDec(encryptedMessage, messageHmac) {
  const { key_hex, vector_hex } = loadSavedSymKey();
  const key = Buffer.from(key_hex, "hex");
  const vector = Buffer.from(vector_hex, "hex");

  const decipher = crypto.createDecipheriv("aes-256-cbc", key, vector);
  let decrypted = decipher.update(encryptedMessage, "hex", "utf8");
  decrypted += decipher.final("utf8");

  const hmacKey = fs.readFileSync(
    path.join(KEYS_TARGET_PATH, "hmac-key.pem"),
    "utf-8"
  );
  const hmac = crypto.createHmac("sha256", hmacKey);
  hmac.update(encryptedMessage);
  const generatedHmac = hmac.digest("hex");

  if (generatedHmac !== messageHmac) {
    return { ok: false, result: "Message integrity verification failed!" };
  }

  return { ok: true, result: decrypted };
}

function decryptFile(filePath, encryptedSymKeyBase64) {
  const privateKeyPath = path.join(KEYS_TARGET_PATH, "private-key.pem");
  const privateKey = fs.readFileSync(privateKeyPath, "utf8");

  const vector = fs.readFileSync(
    path.join(KEYS_TARGET_PATH, "sym-key-vector.pem"),
    "utf-8"
  );

  const encryptedSymKey = Buffer.from(encryptedSymKeyBase64, "base64");
  const decryptedSymKey = crypto.privateDecrypt(
    {
      key: privateKey,
      padding: crypto.constants.RSA_PKCS1_OAEP_PADDING,
      oaepHash: "sha256",
    },
    encryptedSymKey
  );

  const encryptedFileBuffer = fs.readFileSync(filePath);

  const decipher = crypto.createDecipheriv(
    "aes-256-cbc",
    decryptedSymKey,
    vector
  );
  let decryptedFileBuffer = decipher.update(encryptedFileBuffer);
  decryptedFileBuffer += decipher.final();

  return { ok: true, result: decryptedFileBuffer.toString("utf8") };
}

module.exports = {
  generateCryptoKey,
  generateSymKey,
  generateHmacKey,
  symEnc,
  symDec,
  encryptSymKeyWithPublicKey,
  decryptFile,
  generateJwtKeys,
  getKeysPath,
};
