import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import { Input } from '@/components/ui/input';
import { buttonVariants } from '@/components/ui/button';
import { useAuthContext } from '@/providers/auth';
import { useState } from 'react';
import { cn, decryptData } from '@/lib/utils';
import {
  Lock,
  AlertTriangle,
} from 'lucide-react';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import * as z from 'zod';
import apiService from '@/lib/apiService';

const KeyInputFormSchema = z.object({
  privateKey: z.any().refine(
    value => {
      if (value) {
        const file = value;
        if (file.size > 1024 * 1024) {
          return 'File size should not exceed 1MB.';
        }
      }
      return true;
    },
    {
      message: 'Nesprávny súbor',
    },
  )
})

type TKeyInputForm = z.infer<typeof KeyInputFormSchema>

const KeyStatus = () => {

  const form = useForm<TKeyInputForm>({
    resolver: zodResolver(KeyInputFormSchema),
    defaultValues: {
      privateKey: '',
    },
  });
  const {
    setError,
    formState: {},
  } = form;
  const { setSymmKey } = useAuthContext();
  const { setVector } = useAuthContext();

  const [isKeyPresent, setKeyPresent] = useState(!!localStorage.getItem('client_private_key'))

  const handleChange = (e: any) => {
    try {
      if (e.target.files.length > 0) {
        var reader = new FileReader();
        reader.onload = () => {
          const result = reader.result as string;
          if (!result.includes('PRIVATE KEY')) {
            setError('privateKey', {message: 'Neplatný klúč!'});
            return;
          }
          localStorage.setItem('client_private_key', result);
        }
        reader.readAsText(e.target.files[0]);
      }
      apiService.get('/api/symmetricKey').then(response => {
        const symmKeyData = decryptData(response.data);
        setSymmKey(symmKeyData.symmKey);
        setVector(symmKeyData.vector)
      })
      .then(() => {
        setKeyPresent(true)
      })
    } catch(err) {
    }
  }

  return (
    <div>
      {isKeyPresent &&
        <div className="w-full">
          <div className="flex justify-center items-center rounded-md dark:bg-green-900 bg-green-50 px-2 py-2 text-xs font-medium dark:text-green-50 text-green-600 ring-1 ring-inset dark:ring-green-100/50 ring-green-500/10">
            <Lock className="mr-2 h-4 w-4" /> Prenos zabezpečený
          </div>
          <div className="text-xs font-thin mt-2">Citlivé údaje sú prenášané šifrované.</div>
        </div>
      }
      {!isKeyPresent &&
        <div className='w-full'>
          <div className="flex justify-center items-center rounded-md dark:bg-red-900 bg-red-50 px-2 py-2 text-xs font-medium dark:text-red-200 text-red-600 ring-1 ring-inset dark:ring-red-100/50 ring-red-500/10">
            <AlertTriangle className="mr-2 h-4 w-4" /> Chýba kľúč
          </div>
          <div className="text-xs font-thin mt-2">Citlivé údaje nie je možné prenášať šifrované.</div>
          <div className='w-full mt-3 inline-flex justify-center'>
            <Form {...form}>
              <form
                className="flex flex-col w-[min(500px,100%)] rounded-lg mb-4"
              >
                <FormField
                  control={form.control}
                  name="privateKey"
                  render={({ fieldState: { error } }) => (
                    <FormItem>
                      <FormLabel>Nahrať kľuč:</FormLabel>
                      <FormControl>
                        <Input
                          onChange={handleChange}
                          className={cn(
                            buttonVariants({ variant: 'ghost' }),
                            'cursor-pointer  file:cursor-pointer file:text-muted-foreground',
                          )}
                          type="file"
                          error={error}
                        />
                      </FormControl>
                      <FormMessage/>
                    </FormItem>
                  )}
                />
              </form>
            </Form>
          </div>
        </div>
      }
    </div>
  )
};

export default KeyStatus;
