import ThemeToggle from './ThemeToggle';
import { cn } from '../lib/utils';
import { ReactNode, useEffect, useRef, useState } from 'react';
import { NavLink, NavLinkProps, Link, LinkProps } from 'react-router-dom';
import {
  AlignJustify,
  Computer,
  KeyRound,
  LogOut,
  X,
  ArrowUpToLine,
  User,
  Ticket,
  ShoppingCart,
} from 'lucide-react';
import { useAuthContext } from '../providers/auth';
import { Button } from './ui/button';
import { useQueryClient } from '@tanstack/react-query';
import KeyStatus from './KeyStatus';

const Logo = ({ display = false }: { display?: boolean }) => {
  const { isLogin } = useAuthContext();

  return (
    <Link
      to={isLogin ? '/home' : '/'}
      className={`${
        !display && 'hidden sm:flex'
      } flex italic font-bold rounded-full cursor-pointer px-2 py-1`}
    >
      UPB Zadanie
      <Computer className="ml-2" />
    </Link>
  );
};

type MenuItemProps = {
  type?: 'active' | 'notActive';
  children: ReactNode;
} & NavLinkProps &
  LinkProps;

const MenuItem = ({
  children,
  className,
  type = 'active',
  ...props
}: MenuItemProps) => {
  if (type === 'notActive') {
    return (
      <Link
        {...props}
        className={cn(
          'hover:bg-slate-200 hover:bg-opacity-60 dark:hover:bg-slate-800 dark:hover:bg-opacity-70 py-2 px-3 flex flex-row items-center justify-center',
          className,
        )}
      >
        {children}
      </Link>
    );
  }

  return (
    <NavLink
      {...props}
      className={({ isActive }) =>
        isActive
          ? cn(
              ` hover:bg-slate-200 hover:bg-opacity-60 dark:text-blue-400 dark:hover:bg-slate-800 dark:hover:bg-opacity-70 underline underline-offset-2 text-slate-900 font-bold py-2 px-3 flex flex-row items-center justify-center`,
              className,
            )
          : cn(
              'hover:bg-slate-200 hover:bg-opacity-60 dark:hover:bg-slate-800 dark:hover:bg-opacity-70 py-2 px-3 flex flex-row items-center justify-center',
              className,
            )
      }
    >
      {children}
    </NavLink>
  );
};

const MenuBar = () => {
  const queryClient = useQueryClient();
  const { isLogin } = useAuthContext();
  const [isOpen, setIsOpen] = useState(false);
  const menuRef = useRef<HTMLDivElement | null>(null);
  const { setToken } = useAuthContext();
  const buttonRef = useRef<HTMLButtonElement | null>(null);

  useEffect(() => {
    function handleClickOutside(event: MouseEvent) {
      if (
        menuRef.current &&
        !menuRef.current.contains(event.target as Node) &&
        buttonRef &&
        !buttonRef.current?.contains(event.target as Node)
      ) {
        setIsOpen(false);
      }
    }
    document.addEventListener('mousedown', handleClickOutside);
    return () => {
      document.removeEventListener('mousedown', handleClickOutside);
    };
  }, []);

  return (
    <>
      <header
        className={`${
          isLogin && 'lg:w-[calc(100%-210px)]'
        }  z-40 px-4 h-11 top-0 flex gap-4 fixed flex-row items-center w-full shadow-lg dark:shadow-slate-800 dark:bg-slate-950 bg-slate-100 bg-opacity-70 dark:bg-opacity-70`}
      >
        <nav
          className={`${
            isLogin && 'lg:justify-end'
          } w-full h-full flex flex-row gap-x-2 items-center justify-between`}
        >
          <Button
            ref={buttonRef}
            className={`${
              !isLogin && 'hidden'
            } lg:hidden cursor-pointer p-2 hover:bg-slate-200`}
            variant="ghost"
            onClick={() => setIsOpen(true)}
          >
            <AlignJustify />
          </Button>
          {!isLogin && <Logo />}
          <div className="flex gap-x-2 sm:gap-x-8">
            {!isLogin && (
              <>
                <MenuItem className="rounded-md" to="/">
                  Úvod
                </MenuItem>
                <MenuItem className="rounded-md" to="/login">
                  Prihlásenie
                </MenuItem>
                <MenuItem className="rounded-md" to="/registration">
                  Registrácia
                </MenuItem>
              </>
            )}
          </div>
          <ThemeToggle />
        </nav>
      </header>
      <div
        ref={menuRef}
        className={`
        ${(!isOpen || !isLogin) && '-translate-x-full'} 
        ${isOpen && isLogin && 'translate-x-0 lg:translate-x-0'} 
        ${!isOpen && isLogin && 'lg:translate-x-0'} 
        fixed z-50 top-0 left-0 transition-all duration-300 ease-in-out dark:bg-slate-950 dark:shadow-slate-700 shadow-slate-400 bg-slate-100 shadow-lg w-[210px] h-screen px-2`}
      >
        <div className="flex justify-end w-full mt-2">
          <Button
            className="lg:invisible cursor-pointer p-1 w-8 h-8 hover:bg-slate-200"
            variant="ghost"
            onClick={() => setIsOpen(false)}
          >
            <X />
          </Button>
        </div>
        <div className="flex justify-center my-6">
          <Logo display={true} />
        </div>
        <nav className="mt-4 overflow-auto divide-y-[1px] px-2">
          <MenuItem to="/buy-ticket" className="justify-start">
            <ShoppingCart className="mr-3 h-4 w-4" /> Kúpiť letenku
          </MenuItem>
          <MenuItem to="/tickets" className="justify-start">
            <Ticket className="mr-3 h-4 w-4" /> Letenky
          </MenuItem>
          <MenuItem to="/home" className="justify-start">
            <User className="mr-3 h-4 w-4" /> Profil
          </MenuItem>
          <MenuItem to="/keys" className="justify-start">
            <KeyRound className="mr-3 h-4 w-4" /> Šifrovacie kľúče
          </MenuItem>
          <MenuItem to="/key-input" className="justify-start">
            <ArrowUpToLine className="mr-3 h-4 w-4" /> Nahrať kľúč
          </MenuItem>
          <MenuItem
            onClick={() => {
              queryClient.clear();
              setToken(null);
            }}
            type="notActive"
            to="/login"
          >
            Odhlásiť sa <LogOut className="ml-2 h-4 w-4" />
          </MenuItem>
        </nav>

        <div className="mt-8 px-2 flex justify-center">
          <KeyStatus />
        </div>
      </div>
    </>
  );
};

export default MenuBar;
