import { type ClassValue, clsx } from 'clsx';
import { twMerge } from 'tailwind-merge';
import forge from 'node-forge';

export function cn(...inputs: ClassValue[]) {
  return twMerge(clsx(inputs));
}

export function symmetricDecryptData(secret: string, tag: string) {
  const symmKey = forge.util.hexToBytes(
    sessionStorage.getItem('symmKey') as string,
  );
  const vector = forge.util.hexToBytes(
    sessionStorage.getItem('vector') as string,
  );
  var decipher = forge.cipher.createDecipher(
    'AES-GCM',
    forge.util.createBuffer(symmKey),
  );
  try {
    decipher.start({
      iv: forge.util.createBuffer(vector),
      tag: forge.util.createBuffer(forge.util.hexToBytes(tag)),
    });
    decipher.update(forge.util.createBuffer(secret));
    var data = forge.util.encode64(decipher.output.getBytes());
    var result = decipher.finish();
    if (result) {
      return data;
    }
  } catch (error) {
    throw new Error('Decryption error');
  }
}

export function downloadPdfFile(base64Data: string, name: string) {
  try {
    const pdfData = 'data:application/pdf;base64,' + base64Data;
    const link = document.createElement('a');
    link.download = name;
    link.href = pdfData;
    link.click();
    if (document.body.contains(link)) {
      document.body.removeChild(link);
    }
  } catch (error) {
    throw new Error('Pdf download failed');
  }
}

export function symmDecryptData(secret: string, tag: string) {
  const symmKey = forge.util.hexToBytes(
    sessionStorage.getItem('symmKey') as string,
  );
  const vector = forge.util.hexToBytes(
    sessionStorage.getItem('vector') as string,
  );
  var decipher = forge.cipher.createDecipher(
    'AES-GCM',
    forge.util.createBuffer(symmKey),
  );
  try {
    decipher.start({
      iv: forge.util.createBuffer(vector),
      tag: forge.util.createBuffer(forge.util.hexToBytes(tag)),
    });
    decipher.update(forge.util.createBuffer(secret));
    var success = decipher.finish();
    if (success) {
      return decipher.output.toString();
    }
  } catch (error) {
    console.log(error);
    throw new Error('Decryption error');
  }
}

export function decryptData(secret: string) {
  const privateKeyPem = localStorage.getItem('client_private_key');
  const privateKey = forge.pki.privateKeyFromPem(privateKeyPem as string);
  try {
    const decryptedData = privateKey.decrypt(secret, 'RSA-OAEP', {
      md: forge.md.sha256.create(),
    });
    const decodedData = forge.util.decodeUtf8(decryptedData);
    return JSON.parse(decodedData);
  } catch (error) {
    console.log(error);
    throw new Error('Decryption error');
  }
}

export function encryptData<T>(secret: T) {
  try {
    const pubKeyPem = sessionStorage.getItem('server_pub_key');
    const pubKey = forge.pki.publicKeyFromPem(pubKeyPem as string);
    const encodedData = forge.util.encodeUtf8(JSON.stringify(secret));
    const encryptedData = pubKey.encrypt(encodedData, 'RSA-OAEP', {
      md: forge.md.sha256.create(),
    });
    return encryptedData;
  } catch (error) {
    console.log(error);
    throw new Error('Encryption error');
  }
}

export function generateRSAKeys() {
  const keyPair = forge.pki.rsa.generateKeyPair({ bits: 2048, workers: -1 });
  const pubKeyPem = forge.pki.publicKeyToPem(keyPair.publicKey);
  const privKeyPem = forge.pki.privateKeyToPem(keyPair.privateKey);
  return {
    pubKeyPem,
    privKeyPem,
  };
}
