import PageHeader from '@/components/PageHeader';
import { Card, CardTitle, CardContent, CardFooter } from '@/components/ui/card';
import apiService from '@/lib/apiService';
import { Loader2, Search, ShoppingCart } from 'lucide-react';
import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from '@/components/ui/select';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query';
import Loader from '@/components/Loader';
import {
  FlightSearchSchema,
  TAvailableFlights,
  TBuyTicket,
  TConnections,
  TFlightSearch,
} from './schema';
import {
  Table,
  TableBody,
  TableCaption,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
} from '@/components/ui/table';
import { useRef, useState } from 'react';
import { DatePicker } from '@/components/ui/date-picker';
import { Button } from '@/components/ui/button';
import CloseModal from '@/components/CloseModal';

const BuyTicketPage = () => {
  const form = useForm<TFlightSearch>({
    resolver: zodResolver(FlightSearchSchema),
    defaultValues: {
      connectionId: '',
      dateFrom: new Date(new Date().setDate(new Date().getDate() + 1)),
      dateTo: new Date(new Date().setDate(new Date().getDate() + 1)),
    },
  });

  const { clearErrors } = form;

  const {
    isPending: loadFlights,
    data: flights,
    mutateAsync: onLoadFlights,
  } = useMutation({
    mutationFn: async (bodyData: TFlightSearch) => {
      const { data } = await apiService.post('/api/searchFlights', {
        connectionId: bodyData.connectionId,
        dateFrom: bodyData.dateFrom,
        dateTo: new Date(
          bodyData.dateTo.setDate(bodyData.dateTo.getDate() + 1),
        ),
      });
      return data as TAvailableFlights[];
    },
    retry: false,
  });

  const { isLoading: loadConnections, data: connections } = useQuery<
    TConnections[]
  >({
    queryFn: async () => {
      const { data } = await apiService.get('/api/getConnections');
      return data;
    },
    queryKey: ['connections'],
    retry: false,
  });

  const onFindConnection = async (data: TFlightSearch) => {
    await onLoadFlights(data);
  };

  return (
    <>
      <PageHeader title="Zakúpenie letenky" />
      <div className="flex w-full justify-center">
        <Card className="w-[min(900px,100%)]">
          <Form {...form}>
            <form
              className="w-full flex flex-col justify-center"
              onSubmit={form.handleSubmit(onFindConnection)}
            >
              <CardTitle className="text-center py-6">
                Zakúpenie letenky
              </CardTitle>
              <CardContent className="grid sm:grid-cols-2 gap-4 w-full">
                <FormField
                  control={form.control}
                  name="connectionId"
                  render={({ field, fieldState: { error } }) => (
                    <FormItem className="w-full">
                      <FormLabel>Výber spojenia</FormLabel>
                      <Select onValueChange={field.onChange}>
                        <FormControl>
                          <SelectTrigger error={error}>
                            <SelectValue placeholder="Výber spojenia" />
                          </SelectTrigger>
                        </FormControl>
                        <SelectContent className="max-h-[150px] overflow-auto">
                          {loadConnections || !connections ? (
                            <Loader />
                          ) : (
                            connections.map((connection, index) => (
                              <SelectItem
                                key={index}
                                value={connection.id.toString()}
                              >
                                {connection.dept_city_name +
                                  ' - ' +
                                  connection.arr_city_name}
                              </SelectItem>
                            ))
                          )}
                        </SelectContent>
                      </Select>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  control={form.control}
                  name="dateFrom"
                  render={({ field, fieldState: { error } }) => (
                    <FormItem className="w-full">
                      <FormLabel>Dátum od</FormLabel>
                      <DatePicker
                        disablePast={true}
                        onValueChange={value => {
                          clearErrors('dateTo');
                          field.onChange(value);
                        }}
                        value={field.value}
                        error={error}
                      />
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  control={form.control}
                  name="dateTo"
                  render={({ field, fieldState: { error } }) => (
                    <FormItem className="w-full">
                      <FormLabel>Dátum do</FormLabel>
                      <DatePicker
                        disablePast={true}
                        onValueChange={value => {
                          clearErrors('dateFrom');
                          field.onChange(value);
                        }}
                        value={field.value}
                        error={error}
                      />
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <div className="flex justify-center items-end">
                  <Button
                    type="submit"
                    variant="default"
                    className="space-x-4 flex px-[12px] w-[140px]"
                  >
                    <p>Hľadať</p>
                    {loadFlights ? (
                      <Loader2 className="w-[24px] h-[24px] animate-spin" />
                    ) : (
                      <Search className="w-[24px] h-[24px]" />
                    )}
                  </Button>
                </div>
              </CardContent>
              <CardFooter className="flex flex-col justify-center">
                <h3 className="text-lg font-bold pb-2">Zoznam letov</h3>
                {flights && <FlightsTable flights={flights} />}
              </CardFooter>
            </form>
          </Form>
        </Card>
      </div>
    </>
  );
};

const FlightsTable = ({ flights }: { flights: TAvailableFlights[] }) => {
  const [open, setOpen] = useState(false);
  const queryClient = useQueryClient();
  const buttonRef = useRef('0');
  const { mutateAsync: buyTicket, isPending } = useMutation({
    mutationFn: async (bodyData: TBuyTicket) => {
      buttonRef.current = bodyData.flight_id;
      const { data } = await apiService.post('/api/makeReservation', bodyData);
      return data;
    },
    retry: false,
    onSuccess: () => {
      setOpen(true);
      queryClient.invalidateQueries({
        queryKey: ['tickets'],
      });
    },
  });

  return (
    <>
      <CloseModal
        title="Letenka bola úspešne zakúpená"
        description="Informácie o zakúpenej letenke môžete nájsť v sekcii Letenky."
        onClose={() => setOpen(false)}
        open={open}
      />
      <Table className="overflow-x-auto w-[850px]">
        <TableHeader>
          <TableRow>
            <TableHead className="w-[140px]">Odchod</TableHead>
            <TableHead className="w-[140px]">Príchod</TableHead>
            <TableHead>Miesto odchodu</TableHead>
            <TableHead>Miesto príchodu</TableHead>
            <TableHead>Trvanie letu</TableHead>
            <TableHead>Cena</TableHead>
            <TableHead>Kúpiť</TableHead>
          </TableRow>
        </TableHeader>
        <TableBody>
          {flights.map((flight, index) => (
            <TableRow key={index}>
              <TableCell className="w-[140px]">
                {new Date(flight.departure).toLocaleString('sk-SK', {
                  timeStyle: 'short',
                  dateStyle: 'short',
                })}
              </TableCell>
              <TableCell className="w-[140px]">
                {new Date(flight.arrival).toLocaleString('sk-SK', {
                  timeStyle: 'short',
                  dateStyle: 'short',
                })}
              </TableCell>
              <TableCell>{flight.dept_city_name}</TableCell>
              <TableCell>{flight.arr_city_name}</TableCell>
              <TableCell>{flight.duration_min + ' min.'}</TableCell>
              <TableCell>{flight.price / 100 + '€'}</TableCell>
              <TableCell className="py-2">
                <div className="flex h-full">
                  <Button
                    type="button"
                    disabled={isPending}
                    onClick={async () =>
                      await buyTicket({ flight_id: flight.id.toString() })
                    }
                  >
                    {isPending && buttonRef.current === flight.id.toString() ? (
                      <Loader2 className="animate-spin" />
                    ) : (
                      <ShoppingCart />
                    )}
                  </Button>
                </div>
              </TableCell>
            </TableRow>
          ))}
        </TableBody>
        {flights.length === 0 && (
          <TableCaption className="text-md pb-2">
            Neboli nájdené žiadne spoje
          </TableCaption>
        )}
      </Table>
    </>
  );
};

export default BuyTicketPage;
