import Loader from '@/components/Loader';
import { TUpdateUser, UpdateUserSchema, User } from './schema';
import PageHeader from '@/components/PageHeader';
import useGetQuery from '@/hooks/useGetQuery';
import {
  Card,
  CardContent,
  CardFooter,
  CardHeader,
  CardTitle,
} from '@/components/ui/card';
import { Avatar, AvatarFallback, AvatarImage } from '@/components/ui/avatar';
import { Check, Loader2, Pencil, X } from 'lucide-react';
import { Button } from '@/components/ui/button';
import { useState } from 'react';
import { useForm } from 'react-hook-form';
import { zodResolver } from '@hookform/resolvers/zod';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import { Input } from '@/components/ui/input';
import usePutQuery from '@/hooks/usePutQuery';
import { useQueryClient } from '@tanstack/react-query';

const HomePage = () => {
  const queryClient = useQueryClient();
  const [edit, setEdit] = useState(false);

  const form = useForm<TUpdateUser>({
    resolver: zodResolver(UpdateUserSchema),
    defaultValues: {
      firstname: '',
      lastname: '',
    },
  });

  const { setValue, getValues } = form;

  const { isLoading, data: user } = useGetQuery<User>({
    url: '/api/loggedInUser',
    queryKey: ['user'],
    decrypt: false,
  });

  const { mutateAsync, isPending } = usePutQuery<any, TUpdateUser>({
    url: '/api/updateUser',
    encrypt: false,
    decrypt: false,
    onSettled: () => setEdit(false),
    onSuccess: () => {
      queryClient.setQueryData(['user'], {
        ...user,
        name: getValues('firstname'),
        surname: getValues('lastname'),
      });
    },
  });

  const onSubmit = async (data: TUpdateUser) => {
    await mutateAsync({
      secret: {},
      data: data,
    });
  };

  return (
    <>
      <PageHeader title="Profil" />
      {isLoading ? (
        <Loader />
      ) : (
        <div className="flex items-center justify-center min-h-[calc(100vh-136.8px)] w-full">
          <Form {...form}>
            <form
              className="w-[min(100%,400px)]"
              onSubmit={form.handleSubmit(onSubmit)}
            >
              <Card>
                <CardHeader>
                  <CardTitle className="text-center">Tvoje údaje</CardTitle>
                </CardHeader>
                <CardContent className="flex flex-col items-center pb-2 space-y-4 w-full">
                  <Avatar className="w-24 h-24">
                    <AvatarImage
                      src="https://github.com/shadcn.png"
                      alt="@shadcn"
                    />
                    <AvatarFallback>CN</AvatarFallback>
                  </Avatar>

                  {edit ? (
                    <>
                      <FormField
                        control={form.control}
                        name="firstname"
                        render={({ field, fieldState: { error } }) => (
                          <FormItem className="w-full">
                            <FormLabel>Meno</FormLabel>
                            <FormControl>
                              <Input
                                placeholder="Meno"
                                {...field}
                                error={error}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                      <FormField
                        control={form.control}
                        name="lastname"
                        render={({ field, fieldState: { error } }) => (
                          <FormItem className="w-full">
                            <FormLabel>Priezvisko</FormLabel>
                            <FormControl>
                              <Input
                                placeholder="Priezvisko"
                                {...field}
                                error={error}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                    </>
                  ) : (
                    <>
                      <div className="flex flex-col h-[72px] py-2 space-y-1 w-full">
                        <p className=" font-bold">Meno</p>
                        <p>{user?.name}</p>
                      </div>
                      <div className="flex flex-col h-[72px] py-2 space-y-1 w-full">
                        <p className=" font-bold">Priezvisko</p>
                        <p>{user?.surname}</p>
                      </div>
                    </>
                  )}
                  <div className="flex flex-col space-y-1 w-full overflow-x-auto">
                    <p className=" font-bold">Email</p>
                    <p>{user?.email}</p>
                  </div>
                </CardContent>
                <CardFooter className="pb-2 justify-end">
                  <Button
                    type="button"
                    className={`w-8 h-8 p-1 ${!edit && 'hidden'}`}
                    variant="ghost"
                    onClick={() => setEdit(false)}
                  >
                    <X className="w-full h-full" />
                  </Button>
                  <Button
                    type="submit"
                    className={`w-8 h-8 p-1 ${!edit && 'hidden'}`}
                    variant="ghost"
                  >
                    {!isPending ? (
                      <Check className="w-full h-full" />
                    ) : (
                      <Loader2 className="w-full h-full animate-spin" />
                    )}
                  </Button>

                  <Button
                    type="button"
                    className={`w-8 h-8 p-1 ${edit && 'hidden'}`}
                    variant="ghost"
                    onClick={() => {
                      setValue('firstname', user?.name ? user.name : '');
                      setValue('lastname', user?.name ? user.surname : '');
                      setEdit(true);
                    }}
                  >
                    <Pencil className="w-full h-full" />
                  </Button>
                </CardFooter>
              </Card>
            </form>
          </Form>
        </div>
      )}
    </>
  );
};

export default HomePage;
