import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import { Button, buttonVariants } from '@/components/ui/button';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import { Input } from '@/components/ui/input';
import { KeyInputFormSchema, TKeyInputForm } from './schema';
import { Loader2 } from 'lucide-react';
import PageHeader from '@/components/PageHeader';
import CloseModal from '@/components/CloseModal';
import { useState } from 'react';
import { cn } from '@/lib/utils';
import usePutQuery from '@/hooks/usePutQuery';
import { TNewPubKey } from '../keys/schema';

const KeyInputPage = () => {
  const [open, setOpen] = useState(false);
  const [privateKey, setPrivateKey] = useState('');
  const [publicKey, setPublicKey] = useState('');
  const form = useForm<TKeyInputForm>({
    resolver: zodResolver(KeyInputFormSchema),
    defaultValues: {
      publicKeyFile: '',
      privateKeyFile: '',
    },
  });
  const {
    setError,
    clearErrors,
    formState: { errors },
  } = form;

  const handlePrivateUpload = (e: any) => {
    try {
      if (e.target.files.length > 0) {
        var reader = new FileReader();
        reader.onload = () => {
          const result = reader.result as string;
          if (!result.includes('PRIVATE KEY')) {
            setError('privateKeyFile', {message: 'Neplatný klúč!'});
            return;
          }
          clearErrors('privateKeyFile')
          setPrivateKey(result)
        }
        reader.readAsText(e.target.files[0])
      }
    } catch(err) {}
  }

  const handlePublicUpload = (e: any) => {
    try {
      if (e.target.files.length > 0) {
        var reader = new FileReader();
        reader.onload = () => {
          const result = reader.result as string;
          if (!result.includes('PUBLIC KEY')) {
            setError('publicKeyFile', {message: 'Neplatný klúč!'});
            return;
          }
          clearErrors('publicKeyFile')
          setPublicKey(result)
        }
        reader.readAsText(e.target.files[0])
      }
    } catch(err) {}
  }

  const { mutateAsync, isPending } = usePutQuery<any, TNewPubKey>({
    url: '/api/updateUserPubKey',
    encrypt: false,
    decrypt: false,
    onError: () => {
    },
    onSuccess: () => {
      localStorage.setItem('client_private_key', privateKey);
    },
  })

  const onSubmit = async () => {
    // FIXME: global form validation doesn't work...
    if ((!!publicKey && publicKey != "") && (!!privateKey && privateKey != "")) {
      await mutateAsync({secret: {}, data: {publicKey: publicKey}})
    }
  };

  return (
    <>
      <PageHeader title="Nahrávanie kľúčov" />
      <CloseModal
        title="Vloženie kľúča prebehla úspešne"
        onClose={() => setOpen(false)}
        open={open}
      />
      <Form {...form}>
        <form
          onSubmit={form.handleSubmit(onSubmit)}
          className="flex flex-col  p-6 space-y-4 border w-[min(500px,100%)] rounded-lg mb-4"
        >
          <h2 className="self-center text-lg font-bold">Nahranie kľúčov</h2>
          <hr />
          <h3 className="self-center text-md font-bold">Súkromný kľúč</h3>
          <FormField
            control={form.control}
            name="privateKeyFile"
            render={({ fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Súbor</FormLabel>
                <FormControl>
                  <Input
                    onChange={handlePrivateUpload}
                    className={cn(
                      buttonVariants({ variant: 'ghost' }),
                      'cursor-pointer  file:cursor-pointer file:text-muted-foreground',
                    )}
                    type="file"
                    error={error}
                  />
                </FormControl>
                <FormMessage/>
              </FormItem>
            )}
          />
          <hr />
          <h3 className="self-center text-md font-bold">Verejný kľúč</h3>
          <FormField
            control={form.control}
            name="publicKeyFile"
            render={({ fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Súbor</FormLabel>
                <FormControl>
                  <Input
                    onChange={handlePublicUpload}
                    className={cn(
                      buttonVariants({ variant: 'ghost' }),
                      'cursor-pointer  file:cursor-pointer file:text-muted-foreground',
                    )}
                    type="file"
                    error={error}
                  />
                </FormControl>
                <FormMessage/>
              </FormItem>
            )}
          />
          <p className="text-sm font-medium text-red-600 dark:text-red-600">
            {errors.root && errors.root.message}
          </p>
          <div className="flex items-center w-full justify-center">
            <Button
              disabled={isPending}
              className="w-[200px]"
              variant="default"
              type="submit"
            >
              <Loader2
                className={`w-4 h-4 mr-2 animate-spin ${
                  !isPending && 'hidden'
                }`}
              />
              Uložiť
            </Button>
          </div>
        </form>
      </Form>
    </>
  );
};

export default KeyInputPage;
