import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import { Button } from '@/components/ui/button';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import { Input } from '@/components/ui/input';
import {
  LoginFormSchema,
  TPubKeyRequest,
  TLoginForm,
  TPubKeyResponse,
} from './schema';
import { useAuthContext } from '@/providers/auth';
import { Loader2 } from 'lucide-react';
import usePostQuery from '@/hooks/usePostQuery';
import { useNavigate } from 'react-router-dom';
import usePutQuery from '@/hooks/usePutQuery';
import { TNewPubKey } from '../keys/schema';
import { decryptData, generateRSAKeys } from '@/lib/utils';
import { useState } from 'react';
import apiService from '@/lib/apiService';

const LoginPage = () => {
  const navigate = useNavigate();
  const { setToken } = useAuthContext();
  const [generatingKey, setGeneratingKey] = useState(false);
  const { setSymmKey } = useAuthContext();
  const { setVector } = useAuthContext();
  
  const form = useForm<TLoginForm>({
    resolver: zodResolver(LoginFormSchema),
    defaultValues: {
      email: '',
      password: '',
    },
  });

  const {
    setError,
    formState: { errors },
  } = form;

  const { mutateAsync, isPending } = usePostQuery<
    TLoginForm,
    TPubKeyRequest,
    TPubKeyResponse
  >({
    url: '/api/login',
    encrypt: false,
    decrypt: false,
    onError: error => {
      if (error.response?.status === 401) {
        setError('email', {
          message: 'Nesprávny email alebo heslo',
        });
      } else if (error.response?.status === 429) {
        setError('root', {
          message: 'Príliš veľa pokusov o prihlásenie. Skuste to neskôr.',
        });
      }
    },
    onSuccess: data => {
      sessionStorage.removeItem('client_pub_key');
      setToken(data.token);
      setGeneratingKey(true);
      generatePrivateKey();
      navigate('/home');
    },
  });

  const onSubmit = async (data: TLoginForm) => {
    await mutateAsync({
      secret: {
        email: data.email,
        password: data.password,
      },
      data: {
        email: data.email,
        password: data.password,
      },
    });
  };

  const mutateKeys = usePutQuery<any, TNewPubKey>({
    url: '/api/updateUserPubKey',
    encrypt: false,
    decrypt: false,
    onError: () => {
    },
    onSuccess: () => {},
  });

  const generatePrivateKey = async () => {
    const {pubKeyPem, privKeyPem } = generateRSAKeys();
    localStorage.setItem('client_private_key', privKeyPem);
    await mutateKeys.mutateAsync({secret: {}, data: {publicKey: pubKeyPem}})
    apiService.get('/api/symmetricKey').then(response => {
      const symmKeyData = decryptData(response.data);
      setSymmKey(symmKeyData.symmKey);
      setVector(symmKeyData.vector)
    })
    .then(() => {
      setGeneratingKey(false)
    })
  }

  return (
    <Form {...form}>
      <form
        onSubmit={form.handleSubmit(onSubmit)}
        className="flex flex-col p-6 space-y-4 border w-[min(500px,100%)] rounded-lg"
      >
        <h1 className="self-center text-xl font-bold">Prihlásenie</h1>
        <FormField
          control={form.control}
          name="email"
          render={({ field, fieldState: { error } }) => (
            <FormItem>
              <FormLabel>Email</FormLabel>
              <FormControl>
                <Input placeholder="Email" {...field} error={error} />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <FormField
          control={form.control}
          name="password"
          render={({ field, fieldState: { error } }) => (
            <FormItem>
              <FormLabel>Heslo</FormLabel>
              <FormControl>
                <Input
                  type="password"
                  placeholder="Heslo"
                  {...field}
                  error={error}
                />
              </FormControl>
              <FormMessage />
            </FormItem>
          )}
        />
        <p className="text-sm font-medium text-red-600 dark:text-red-600">
          {errors.root && errors.root.message}
        </p>
        <div className="flex items-center w-full justify-center">
          <Button
            disabled={isPending}
            className="w-[200px]"
            variant="default"
            type="submit"
          >
            <Loader2
              className={`w-4 h-4 mr-2 animate-spin ${!isPending && 'hidden'}`}
            />
            Prihlásiť
          </Button>
        </div>
      </form>
      <div className="flex items-center w-full justify-center mt-4">
        <Loader2 className={`w-4 h-4 mr-2 animate-spin ${(!isPending && !generatingKey) && 'hidden'}`} />
        {(isPending && !generatingKey ) &&
          <div>Prebieha prihlásenie...</div>
        }
        {(generatingKey) &&
          <div>Vytvára sa s súkromný kľúč...</div>
        }
      </div>
    </Form>
  );
};

export default LoginPage;
