import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import { Button } from '@/components/ui/button';
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form';
import { Input } from '@/components/ui/input';
import {
  RegistrationFormSchema,
  TRegistrationForm,
  TRegistrationFormData,
  TRegistrationFormSecret,
} from './schema';
import {
  Download,
} from 'lucide-react';
import { useNavigate } from 'react-router-dom';
import { Loader2 } from 'lucide-react';
import CloseModal from '@/components/CloseModal';
import { useState } from 'react';
import usePostQuery from '@/hooks/usePostQuery';
import usePutQuery from '@/hooks/usePutQuery';
import { TNewPubKey } from '../keys/schema';
import { decryptData, generateRSAKeys } from '@/lib/utils';
import { useAuthContext } from '@/providers/auth';
import apiService from '@/lib/apiService';

const RegistrationPage = () => {
  const navigate = useNavigate();
  const [open, setOpen] = useState(false);
  const [generatingKeys, setGeneratingKeys] = useState(false);
  const [privateKey, setPrivateKey] = useState('');
  const { setSymmKey } = useAuthContext();
  const { setVector } = useAuthContext();
  const form = useForm<TRegistrationForm>({
    resolver: zodResolver(RegistrationFormSchema),
    defaultValues: {
      email: '',
      firstname: '',
      lastname: '',
      password: '',
      confirmPassword: '',
    },
  });
  const {
    setError,
    formState: { errors },
  } = form;

  const onGenerateKeys = async () => {
    const {pubKeyPem, privKeyPem } = generateRSAKeys();
    localStorage.setItem('client_private_key', privKeyPem);
    setPrivateKey(privKeyPem);
    await mutateKeys.mutateAsync({secret: {}, data: {publicKey: pubKeyPem}});
    apiService.get('/api/symmetricKey').then(response => {
      const symmKeyData = decryptData(response.data);
      setSymmKey(symmKeyData.symmKey);
      setVector(symmKeyData.vector)
    })
    .then(() => {
      setGeneratingKeys(false)
    })
  }

  const handleDownload = () => {
    const blob = new Blob([privateKey], {type: 'text/plain'});
    const link = document.createElement('a');
    link.download = 'private_key.pem';
    link.href = window.URL.createObjectURL(blob);
    link.click();
    if (document.body.contains(link)) {
      document.body.removeChild(link);
    }
  }

  const { setToken } = useAuthContext();

  const mutateKeys = usePutQuery<any, TNewPubKey>({
    url: '/api/updateUserPubKey',
    encrypt: false,
    decrypt: false,
    onError: () => {
    },
    onSuccess: () => {},
  });

  const { mutateAsync, isPending } = usePostQuery<
    TRegistrationFormSecret,
    TRegistrationFormData
  >({
    url: '/api/register',
    encrypt: false,
    decrypt: false,
    onError: error => {
      let message = error.response?.data?.error;
      if (message === 'Email already used!') {
        setError('email', { message: 'Tento email je už obsadený' });
      } else if (message === 'Password exposed!') {
        setError('password', { message: 'Toto heslo bolo odhalené!' });
      } else if (message === 'Common password!') {
        setError('password', { message: 'Toto heslo je často využívané!' });
      } else if (error.response?.status === 429) {
        setError('root', {
          message: 'Príliš veľa pokusov o registráciu. Skuste to neskôr.',
        });
      }
    },
    onSuccess: data => {
      setToken(data.token);
      setGeneratingKeys(true);
      onGenerateKeys();
      setOpen(true);
    },
  });
  
  const onSubmit = async (data: TRegistrationForm) => {
    await mutateAsync({
      secret: {
        email: data.email,
        password: data.password,
      },
      data: {
        email: data.email,
        password: data.password,
        firstname: data.firstname,
        lastname: data.lastname,
      },
    });
  };

  return (
    <Form {...form}>
      <CloseModal
        title="Registracia prebehla úspešne"
        onClose={() => navigate('/home')}
        open={open}
      >
        <div>
          <div className="font-light">Pre prenos citlivých údajov bol vygenerovaný pár kľúčov.</div>
          <div className="mt-2 flex justify-center mt-4 mb-4">
            <Button onClick={handleDownload}><Download className="mr-3 h-4 w-4" />Stiahnuť kľuč</Button>
          </div>
          <div className="font-light">Súkromný kľúč si uchovajte pre prípad použitia aplikácie na inom zariadení (alebo po lokálnom premazaní pamäte).</div>
        </div>
      </CloseModal>
      <form
        onSubmit={form.handleSubmit(onSubmit)}
        className="border w-[min(650px,100%)] rounded-lg p-4"
      >
        <h1 className="text-center text-xl font-bold">Registrácia</h1>
        <div className="grid sm:grid-cols-2 gap-6 py-6">
          <FormField
            control={form.control}
            name="email"
            render={({ field, fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Email</FormLabel>
                <FormControl>
                  <Input placeholder="Email" {...field} error={error} />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="firstname"
            render={({ field, fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Meno</FormLabel>
                <FormControl>
                  <Input placeholder="Meno" {...field} error={error} />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="lastname"
            render={({ field, fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Priezvisko</FormLabel>
                <FormControl>
                  <Input placeholder="Priezvisko" {...field} error={error} />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="password"
            render={({ field, fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Heslo</FormLabel>
                <FormControl>
                  <Input
                    type="password"
                    placeholder="Heslo"
                    {...field}
                    error={error}
                  />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <FormField
            control={form.control}
            name="confirmPassword"
            render={({ field, fieldState: { error } }) => (
              <FormItem>
                <FormLabel>Potvrdenie hesla</FormLabel>
                <FormControl>
                  <Input
                    type="password"
                    placeholder="Potvrdenie hesla"
                    {...field}
                    error={error}
                  />
                </FormControl>
                <FormMessage />
              </FormItem>
            )}
          />
          <p className="text-sm font-medium text-red-600 dark:text-red-600">
            {errors.root && errors.root.message}
          </p>
        </div>
        <div className="flex items-center w-full justify-center">
          <Button
            disabled={isPending}
            className="w-[200px]"
            variant="default"
            type="submit"
          >
            <Loader2
              className={`w-4 h-4 mr-2 animate-spin ${!isPending && 'hidden'}`}
            />
            Registrovať
          </Button>
        </div>
      </form>

      <div className="flex items-center w-full justify-center mt-4">
        <Loader2 className={`w-4 h-4 mr-2 animate-spin ${(!isPending && !generatingKeys) && 'hidden'}`} />
        {(isPending && !generatingKeys ) &&
          <div>Vytvára sa účet...</div>
        }
        {(generatingKeys) &&
          <div>Vytvárajú sa šifrovacie kľúče...</div>
        }
      </div>
    </Form>
  );
};

export default RegistrationPage;
