import PageHeader from '@/components/PageHeader';
import { Button } from '@/components/ui/button';
import {
  Card,
  CardHeader,
  CardTitle,
  CardContent,
  CardFooter,
} from '@/components/ui/card';
import apiService from '@/lib/apiService';
import { symmDecryptData, symmetricDecryptData } from '@/lib/utils';
import { TTicket } from './schema';
import { useQuery } from '@tanstack/react-query';
import { Badge } from '@/components/ui/badge';
import Loader from '@/components/Loader';

const Tickets = () => {
  const { isLoading: loadingTickets, data: tickets } = useQuery<TTicket[]>({
    queryFn: async () => {
      const { data } = await apiService.get('/api/myTickets');
      const decryptedData = symmDecryptData(data.data, data.tag) as string;
      const flights: TTicket[] = JSON.parse(decryptedData);
      return flights;
    },
    retry: false,
    queryKey: ['tickets'],
  });

  const getTicket = (code: string) => {
    apiService.get('/ticket/' + code).then(response => {
      const decryptedFile = symmetricDecryptData(
        response.data.data,
        response.data.tag,
      );

      if (decryptedFile !== undefined) {
        const decodedData = atob(decryptedFile);

        const uint8Array = new Uint8Array(decodedData.length);
        for (let i = 0; i < decodedData.length; ++i) {
          uint8Array[i] = decodedData.charCodeAt(i);
        }
        const blob = new Blob([uint8Array], { type: 'application/pdf' });
        const link = document.createElement('a');
        link.download = 'ticket_'+response.data.id+'.pdf';
        link.href = window.URL.createObjectURL(blob);
        link.click();
        if (document.body.contains(link)) {
          document.body.removeChild(link);
        }
      } else {
        console.error('Decrypted file is undefined.');
      }

    });
  };

  return (
    <>
      <PageHeader title="Prehľad leteniek" />
      {loadingTickets ? (
        <Loader />
      ) : (
        <div className="flex items-center min-h-[calc(100vh-136.8px)] justify-center">
          <div
            className={`grid 
            ${tickets && tickets?.length === 1 && 'sm:grid-cols-2'} 
            ${tickets &&
              tickets?.length === 2 &&
              'sm:grid-cols-2 lg:grid-cols-3'
              }
            ${tickets &&
              tickets?.length > 2 &&
              'sm:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4'
              }
             gap-6`}
          >
            {tickets &&
              tickets.map((ticket, index) => {
                let isActive = new Date(ticket.departure) > new Date();
                return (
                  <>
                    <Card
                      className="w-60 h-60 border-solid justify-center text-center flex flex-col"
                      key={index}
                    >
                      <CardHeader>
                        <CardTitle>
                          {ticket.dept_city_code} - {ticket.arr_city_code}
                        </CardTitle>
                      </CardHeader>
                      <CardContent className="space-y-2">
                        <Badge
                          className={`${isActive
                            ? 'bg-green-700 dark:text-inherit'
                            : 'bg-red-700 dark:text-inherit'
                            }`}
                        >
                          {isActive ? 'Aktívny' : 'Expiroval'}
                        </Badge>
                        <div className="text-sm text-muted-foreground flex flex-col justify-center">
                          {'Odlet: ' +
                            new Date(ticket.departure).toLocaleString('sk-SK', {
                              timeStyle: 'short',
                              dateStyle: 'short',
                            })}
                        </div>
                        <div className="text-sm text-muted-foreground flex flex-col justify-center">
                          Trieda: {ticket.class}, Miesto: {ticket.seat}
                        </div>
                      </CardContent>
                      <CardFooter className="justify-center">
                        <Button onClick={() => getTicket(ticket.code)}>
                          Stiahnuť
                        </Button>
                      </CardFooter>
                    </Card>
                  </>
                );
              })}
          </div>
        </div>
      )}
    </>
  );
};

export default Tickets;
