const db = require("../models");
const User = db.users;
const Invoice = db.invoices;

const fontkit = require('@pdf-lib/fontkit');
const crypto = require('crypto');
const fs = require('fs');

const { PDFDocument } = require('pdf-lib');

exports.getInvoice = async ({ userId, invoiceId }) => {

    const invoice = await Invoice.findOne({ where: { id: invoiceId } });

    // Create a new PDF document
    const pdfDoc = await generatePDFForInvoice(invoice);

    // Serialize the PDF to binary
    const pdfBytes = await pdfDoc.save();

    // Generate a random symmetric encryption key
    // 32 bytes (256 bits) key
    const encryptionKey = crypto.randomBytes(32);

    // Create an initialization vector (IV)
    // 16 bytes IV for AES
    const iv = crypto.randomBytes(16);

    // Encrypt the PDF binary stream
    const pdfEncryptedBytes = encryptPdf(pdfBytes, encryptionKey, iv);

    const user = await User.findOne({ where: { id: userId }, attributes: ['publicKey'] })

    if (user == null) {
        return null
    }

    const { publicKey } = user

    let encryptedEncryptionKey
    try {
        // Encrypt the symmetric encryption key with the user's public key
        encryptedEncryptionKey = crypto.publicEncrypt({
            key: publicKey,
            padding: crypto.constants.RSA_PKCS1_OAEP_PADDING,
            oaepHash: 'sha256'
        }, encryptionKey);
    }
    catch (e) {
        return null;
    }

    // Concat the IV, encrypted encryption key, and the encrypted PDF content
    const combinedEncryptedBytes = Buffer.concat([iv, encryptedEncryptionKey, pdfEncryptedBytes]);

    return combinedEncryptedBytes.toString("base64");
}

function formatDateTime(dateTimeString) {
    const options = {
        hour: 'numeric',
        minute: 'numeric',
        second: 'numeric',
        day: 'numeric',
        month: 'numeric',
        year: 'numeric',
        timeZone: 'Europe/Bratislava'
    };

    return new Date(Date.parse(dateTimeString)).toLocaleDateString('sk-SK', options);
}

async function generatePDFForInvoice(invoice) {
    const pdfDoc = await PDFDocument.create();
    const page = pdfDoc.addPage([400, 600]);

    pdfDoc.registerFontkit(fontkit)
    const fontBytes = fs.readFileSync('./UbuntuFont.ttf');
    const customFont = await pdfDoc.embedFont(fontBytes);

    const pdfInvoiceJson = {
        id_faktury: invoice.id,
        id_stravnika: invoice.userId,
        objednavka: invoice.foodName,
        cena: '' + invoice.price + '€',
        datum_objednavky: formatDateTime(invoice.createdAt),
    }

    page.setFont(customFont);
    page.setFontSize(8);
    page.moveTo(50, 500);
    page.drawText(`Faktúra:\n${JSON.stringify(pdfInvoiceJson, null, 2)}`);

    return pdfDoc;
}

function encryptPdf(pdfBytes, encryptionKey, iv) {
    // Create a cipher with AES-256-GCM algorithm
    const cipher = crypto.createCipheriv('aes-256-gcm', encryptionKey, iv);

    // Encrypt the PDF bytes
    const encryptedPDFBytes = Buffer.concat([cipher.update(pdfBytes), cipher.final()]);

    // Get the GCM authentication tag
    const authenticationTag = cipher.getAuthTag();

    // Combine the encrypted data and authentication tag
    return Buffer.concat([encryptedPDFBytes, authenticationTag]);
}
