import React, { useState, useContext, useEffect } from "react";

import {
  InputLabel,
  OutlinedInput,
  Typography,
  Button,
  Select,
  MenuItem,
  Rating,
  Divider,
  Container,
} from "@mui/material";

import "./css/Feedback.css";
import "./css/general.css";
import { AlertContext, BACKEND_URL } from "../App";
import { SearchableList } from "./SearchableList";
import { hashPassword } from "../components_old/KeyPairs";

export const Feedback = () => {
  const [jedlo, setJedlo] = useState("");
  const [stars, setStars] = useState(0);
  const [textRecenzie, setTextRecenzie] = useState("");
  const [password, setPassword] = useState("");
  const [reviews, setReviews] = useState([]);
  const [menuOffer, setMenuOffer] = useState([]);

  const { setAlert } = useContext(AlertContext);

  const loadReviews = async () => {
    try {
      const response = await fetch(`${BACKEND_URL}/api/reviews`, {
        method: "GET",
        headers: { "Content-Type": "application/json" },
      });

      if (!response.ok) {
        throw new Error("Failed to fetch reviews");
      }

      const data = await response.json();

      // Map the data to match the expected structure
      const mappedData = data.map((item) => ({
        id: item.id,
        name: item.food,
        date: formatDateString(item.createdAt),
        autor: `${item.firstName} ${item.lastName}`,
        stars: item.stars,
        text: item.message,
      }));

      setReviews(mappedData);
    } catch (error) {
      console.error("Error fetching reviews:", error);
    }
  };

  const loadFoodsNames = async () => {
    try {
      const response = await fetch(`${BACKEND_URL}/api/foods`, {
        method: "GET",
        headers: { "Content-Type": "application/json" },
      });

      if (!response.ok) {
        throw new Error("Failed to fetch foods");
      }

      const data = await response.json();

      // Map the data to match the expected structure
      const mappedData = data.map((item) => ({
        name: item.name,
        price: item.price,
        type: item.isSoup ? 0 : 1,
      }));

      setMenuOffer(mappedData);
    } catch (error) {
      console.error("Error fetching foods:", error);
    }
  };

  useEffect(() => {
    loadReviews();
    loadFoodsNames();
  }, []);

  // Function to format ISO date string as "day.month.year"
  const formatDateString = (isoDateString) => {
    const date = new Date(isoDateString);
    return `${date.getDate()}.${date.getMonth() + 1}.${date.getFullYear()}`;
  };

  const postReview = async () => {
    if (!stars) {
      setAlert({
        state: true,
        severity: "error",
        message: "Vyberte najmenej 1 hviezdičku",
      });
      return;
    }

    if (password.trim().length === 0) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nezadali ste heslo",
      });
      return;
    }

    if (jedlo.trim().length === 0) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nevybrali ste hodnotené jedlo",
      });
      return;
    }

    // Parse the JSON string to get the user object
    const user = JSON.parse(localStorage.getItem("user"));

    let hashedPassword = null;

    try {
      hashedPassword = await hashPassword(password);
    } catch (e) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nepodarilo sa zahešovať heslo",
      });
      return;
    }

    // Create the review object
    const review = {
      userId: user.id,
      password: hashedPassword,
      food: jedlo,
      stars: stars,
      message: textRecenzie,
    };

    console.log(review);

    try {
      // Make a POST request to the API endpoint using fetch
      const response = await fetch(`${BACKEND_URL}/api/reviews`, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(review),
      });

      if (!response.ok) {
        // If the response status is not in the range 200-299, treat it as an error
        throw new Error(`HTTP error! Status: ${response.status}`);
      }

      // Parse the JSON response
      const responseData = await response.json();

      // Log the response from the server
      console.log(responseData);

      // Update the state or perform other actions based on the response if needed

      // Show a success alert
      setAlert({
        state: true,
        severity: "success",
        message: "Recenzia úspešne pridaná",
      });

      loadReviews();
    } catch (error) {
      // Show an error alert
      setAlert({
        state: true,
        severity: "error",
        message: "Nastala chyba, pravdepodobne ste zadali nesprávne heslo",
      });
      console.error("Error posting review");
    } finally {
      setJedlo("");
      setStars("");
      setTextRecenzie("");
      setPassword("");
    }
  };

  return (
    <Container maxWidth="lg" className="custom-container feedback-container">
      <div>
        <Typography variant="h3" gutterBottom>
          Recenzie
        </Typography>
        <Divider />
        <Typography variant="h5" gutterBottom className="small-title">
          Vytvorenie recenzie
        </Typography>
        <div className="feedback-inputs-container">
          <div className="feedback-inputs">
            <div className="feedback-one-input-field one-input-field">
              <InputLabel id="demo-simple-select-label">Jedlo:</InputLabel>
              <Select
                value={jedlo}
                displayEmpty
                inputProps={{ "aria-label": "Without label" }}
                onChange={(e) => setJedlo(e.target.value)}
              >
                <MenuItem value="" disabled>
                  <em>Výber jedál</em>
                </MenuItem>
                {menuOffer.map((item) => (
                  <MenuItem value={item.name} key={item.name}>
                    {item.name}
                  </MenuItem>
                ))}
              </Select>
            </div>
            <div className="stars-container one-input-field">
              <Typography component="legend">Počet hviezdičiek:</Typography>
              <Rating
                className="custom-rating"
                name="no-value"
                value={stars}
                onChange={(event, newValue) => {
                  setStars(newValue);
                }}
              />
            </div>
          </div>

          <div className="recenzia-container">
            <InputLabel htmlFor="text-recenzie">Text recenzie:</InputLabel>
            <OutlinedInput
              id="text-recenzie"
              size="small"
              multiline={true}
              minRows={5}
              className="custom-recenzia-input"
              onChange={(e) => setTextRecenzie(e.target.value)}
              value={textRecenzie}
              placeholder="Text recenzie"
            />
          </div>
        </div>

        <div className="inputs-container">
          <div className="one-input-field">
            <InputLabel htmlFor="heslo">Heslo:</InputLabel>
            <OutlinedInput
              id="heslo"
              size="small"
              type="password"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setPassword(e.target.value)}
              value={password}
              placeholder="Heslo"
            />
          </div>
        </div>
        <Button
          className="dark-button"
          variant="outlined"
          size="medium"
          onClick={postReview}
        >
          Uverejniť
        </Button>
      </div>

      <Divider />

      <div>
        <Typography variant="h5" gutterBottom className="small-title">
          Prehliadanie recenzií
        </Typography>
        <SearchableList data={reviews} />
      </div>
    </Container>
  );
};
