import React, { useState, useContext, useEffect } from "react";

import {
  FormControl,
  Radio,
  RadioGroup,
  FormControlLabel,
  Typography,
  Button,
  Divider,
  Container,
  InputLabel,
  OutlinedInput,
} from "@mui/material";

import "./css/general.css";
import "./css/Orders.css";
import { AlertContext, BACKEND_URL } from "../App";
import { hashPassword } from "../components_old/KeyPairs";

export const Orders = () => {
  const [polievka, setPolievka] = useState("");
  const [hlJedlo, setHlJedlo] = useState("");
  const [menuOffer, setMenuOffer] = useState([]);
  const [password, setPassword] = useState("");

  const { setAlert } = useContext(AlertContext);

  const today = new Date();
  today.setDate(today.getDate() + 1);
  const actualDate = `${today.getDate()}.${
    today.getMonth() + 1
  }.${today.getFullYear()}`;

  const loadFoods = async () => {
    try {
      const response = await fetch(`${BACKEND_URL}/api/foods`, {
        method: "GET",
        headers: { "Content-Type": "application/json" },
      });

      if (!response.ok) {
        throw new Error("Failed to fetch foods");
      }

      const data = await response.json();

      // Map the data to match the expected structure
      const mappedData = data.map((item) => ({
        id: item.id,
        name: item.name,
        price: item.price,
        type: item.isSoup ? 0 : 1,
      }));

      setMenuOffer(mappedData);
    } catch (error) {
      // console.error("Error fetching foods:", error);
    }
  };

  const handleOrder = async () => {
    if (polievka.length === 0 && hlJedlo.length === 0) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nevybrali ste žiadne jedlo",
      });
      return;
    }

    if (password.trim().length === 0) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nezadali ste heslo",
      });
      return;
    }

    let hashedPassword = null;

    try {
      hashedPassword = await hashPassword(password);
    } catch (e) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nepodarilo sa zahešovať heslo",
      });
      return;
    }

    try {
      const user = JSON.parse(localStorage.getItem("user"));

      const order = {
        userId: user.id,
        foodIds: {
          soupId: polievka,
          mainId: hlJedlo,
        },
        password: hashedPassword,
      };

      const response = await fetch(`${BACKEND_URL}/api/invoices/create`, {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(order),
      });

      if (!response.ok) {
        throw new Error("Nepodarilo sa vytvoriť objednávku.");
      }

      setAlert({
        state: true,
        severity: "success",
        message: "Objednávka a platba prebehla úspešne",
      });
    } catch (error) {
      console.error("Nepodarilo sa vytvoriť objednávku.");
      setAlert({
        state: true,
        severity: "error",
        message: "Nastala chyba, pravdepodobne ste zadali nesprávne heslo",
      });
    } finally {
      setPolievka("");
      setHlJedlo("");
      setPassword("")
    }
  };

  useEffect(() => {
    // Load foods when the component mounts
    loadFoods();
  }, []);

  return (
    <Container maxWidth="lg" className="custom-container">
      <div>
        <Typography variant="h3" gutterBottom>
          Ponuka dňa {actualDate}
        </Typography>
        <Divider />
        <div className="orders-inputs-container">
          <div className="table-titles-container">
            <Typography variant="h6">Polievka</Typography>
            <Typography variant="h6">Cena</Typography>
          </div>
          <Divider />
          <FormControl className="orders-form-control">
            <RadioGroup
              className="orders-radio-group"
              aria-labelledby="polievka-radio"
              name="polievka-radio"
              value={polievka}
            >
              {menuOffer.map((item) =>
                item.type === 0 ? (
                  <div className="radio-container" key={item.name}>
                    <FormControlLabel
                      value={item.id}
                      control={
                        <Radio
                          onClick={(e) => {
                            if (polievka === e.target.value) {
                              setPolievka("");
                            } else {
                              setPolievka(e.target.value);
                            }
                          }}
                        />
                      }
                      label={item.name}
                    />
                    <Typography variant="p">{item.price}€</Typography>
                  </div>
                ) : null
              )}
            </RadioGroup>
          </FormControl>

          <Divider />
          <Divider />

          <div className="table-titles-container">
            <Typography variant="h6">Hlavné jedlo</Typography>
            <Typography variant="h6">Cena</Typography>
          </div>
          <Divider />
          <FormControl className="orders-form-control">
            <RadioGroup
              className="orders-radio-group"
              aria-labelledby="hl-jedlo-radio"
              name="hl-jedlo-radio"
              value={hlJedlo}
            >
              {menuOffer.map((item) =>
                item.type === 1 ? (
                  <div className="radio-container" key={item.name}>
                    <FormControlLabel
                      value={item.id}
                      control={
                        <Radio
                          onClick={(e) => {
                            if (hlJedlo === e.target.value) {
                              setHlJedlo("");
                            } else {
                              setHlJedlo(e.target.value);
                            }
                          }}
                        />
                      }
                      label={item.name}
                    />
                    <Typography variant="p">{item.price}€</Typography>
                  </div>
                ) : null
              )}
            </RadioGroup>
          </FormControl>
        </div>
        <Divider />
        <Divider />

        <div className="inputs-container">
          <div className="one-input-field">
            <InputLabel htmlFor="heslo">Heslo:</InputLabel>
            <OutlinedInput
              id="heslo"
              size="small"
              type="password"
              className="custom-outlined-outlinedInput"
              onChange={(e) => setPassword(e.target.value)}
              value={password}
              placeholder="Heslo"
            />
          </div>
        </div>
        <Button
          className="dark-button mt-15"
          variant="outlined"
          size="medium"
          onClick={handleOrder}
        >
          Objednať a zaplatiť
        </Button>
      </div>
    </Container>
  );
};
