import React, { useState, useContext, useEffect } from "react";

import { hashPassword, keyPairs } from "../components_old/KeyPairs";

import {
  InputLabel,
  OutlinedInput,
  Typography,
  Button,
  Divider,
  Container,
} from "@mui/material";

import "./css/general.css";
import "./css/Profil.css";
import { AlertContext, BACKEND_URL } from "../App";

export const Profil = ({ userId }) => {
  const [firstName, setFirstName] = useState("Meno");
  const [lastName, setLastName] = useState("Priezvisko");
  const [email, setEmail] = useState("E-mail");
  const [displayedPublicKey, setDisplayedPublicKey] = useState("Verejny kluc");
  const [publicKey, setPublicKey] = useState(null);
  const [password, setPassword] = useState("");

  const { setAlert } = useContext(AlertContext);

  useEffect(() => {
    const storedUser = JSON.parse(localStorage.getItem("user"));

    const getUser = async (id) => {
      const data = await fetch(`${BACKEND_URL}/api/users/${id}`, {
        method: "GET",
        headers: { "Content-Type": "application/json" },
      });
      const fetchedUser = await data.json();

      if (fetchedUser.error === "Používateľ neexistuje.") {
        localStorage.removeItem("user");
      } else {
        setFirstName(fetchedUser.firstName);
        setLastName(fetchedUser.lastName);
        setEmail(fetchedUser.email);
        setDisplayedPublicKey(fetchedUser.publicKey ? "Zadaný" : "Nezadaný");
      }
    };

    getUser(storedUser.id);
  }, [publicKey == null]);

  const handleFileChange = async (file) => {
    try {
      const fileReader = new FileReader();
      fileReader.onloadend = (e) => {
        const content = fileReader.result;
        setPublicKey(content);
      };
      fileReader.readAsText(file);
    } catch (e) {
      setPublicKey(null);
    }
  };

  const submitPublicKey = async (pubKey, from) => {
    if (password === "" || password.trim().length === 0) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nezadali ste heslo",
      });
      return null;
    }

    let o;

    if (from === "form") o = await validateImportedKey(pubKey);

    if (o === null && from === "form") {
      const inp = document.getElementById("importPublicKeyInput");
      inp.value = "";
      setPassword("");
      setPublicKey(null);
      setAlert({
        state: true,
        severity: "error",
        message: "Vložili ste verejný kľúč v nesprávnom formáte!",
      });
      return;
    }

    let hashedPassword = null;

    try {
      hashedPassword = await hashPassword(password);
    } catch (e) {
      setAlert({
        state: true,
        severity: "error",
        message: "Nepodarilo sa zahešovať heslo",
      });
      return;
    }

    const body = {
      userId,
      publicKey: pubKey,
      password: hashedPassword,
    };

    const data = await fetch(`${BACKEND_URL}/api/user/publicKey`, {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: JSON.stringify(body),
    });

    const response = await data.json();

    setPassword("");
    setPublicKey(null);

    const inp = document.getElementById("importPublicKeyInput");
    inp.value = "";

    if (response.error) {
      setAlert({
        state: true,
        severity: "error",
        message: response.error,
      });
      return null;
    }

    if (response.status) {
      setAlert({
        state: true,
        severity: "info",
        message: response.status,
      });

      return;
    }
  };

  const generateKeyPairs = async () => {
    const { publicKey: pk, privateKey } = await keyPairs();

    const out = await submitPublicKey(pk, "func");

    if (out !== null) {
      setPublicKey(pk);
      downloadPrivateKey(privateKey);
      setAlert({
        state: true,
        severity: "success",
        message: "Pár verejného a privátneho kľúča bol úspešne vygenerovaný.",
      });
    }
  };

  return (
    <Container maxWidth="lg" className="custom-container">
      <div>
        <Typography variant="h3" gutterBottom>
          Profil
        </Typography>
        <Divider />
        <div className="inputs-container">
          <div className="one-input-field">
            <InputLabel htmlFor="meno">Meno:</InputLabel>
            <Typography
              id="meno"
              size="small"
              className="custom-outlined-typography"
            >
              {firstName}
            </Typography>
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="priezvisko">Priezvisko:</InputLabel>
            <Typography
              id="priezvisko"
              size="small"
              className="custom-outlined-typography"
            >
              {lastName}
            </Typography>
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="e-mail">E-mail:</InputLabel>
            <Typography
              id="e-mail"
              size="small"
              className="custom-outlined-typography"
            >
              {email}
            </Typography>
          </div>
          <div className="one-input-field">
            <InputLabel htmlFor="verejny-kluc">Verejný klúč:</InputLabel>
            <Typography
              id="verejny-kluc"
              size="small"
              className="custom-outlined-typography"
            >
              {displayedPublicKey}
            </Typography>
          </div>
        </div>

        <Divider />
        <div className="verejny-kluc-section">
          <Typography variant="h4" gutterBottom>
            Verejný kľúč
          </Typography>

          <div className="inputs-container">
            <div className="one-input-field">
              <InputLabel htmlFor="heslo">Heslo:</InputLabel>
              <OutlinedInput
                id="heslo"
                size="small"
                type="password"
                className="custom-outlined-outlinedInput"
                onChange={(e) => setPassword(e.target.value)}
                value={password}
                placeholder="Heslo"
              />
            </div>
          </div>
          <div className="upload-container">
            <Typography variant="h6">Nahraj verejný kľúč:</Typography>
            <div className="file-input-container">
              <OutlinedInput
                id="importPublicKeyInput"
                size="small"
                className="custom-file-input"
                type="file"
                accept=".pem"
                onChange={(e) => {
                  handleFileChange(e.target.files[0]);
                }}
              />
              <Button
                className="nahrat-button"
                variant="outlined"
                size="medium"
                onClick={() => submitPublicKey(publicKey, "form")}
              >
                Nahrať
              </Button>
            </div>
            <div className="generate-container">
              <Typography variant="h6">
                Vygeneruj si pár asymetrických kľúčov:
              </Typography>

              <Button
                className="dark-button"
                variant="outlined"
                size="medium"
                onClick={() => generateKeyPairs()}
              >
                Generovať
              </Button>
            </div>
          </div>
        </div>
      </div>
    </Container>
  );
};

function downloadPrivateKey(privateKey) {
  const file = new Blob([privateKey], { type: "text/plain" });

  const url = URL.createObjectURL(file);

  const a = document.createElement("a");
  a.href = url;
  a.download = "privateKey.pem";
  a.click();

  URL.revokeObjectURL(url);
}

async function validateImportedKey(key) {
  try {
    function getSpkiDer(spkiPem) {
      const pemHeader = "-----BEGIN PUBLIC KEY-----";
      const pemFooter = "-----END PUBLIC KEY-----";
      var pemContents = spkiPem.substring(
        pemHeader.length,
        spkiPem.length - pemFooter.length
      );
      var binaryDerString = window.atob(pemContents);
      return str2ab(binaryDerString);
    }

    function str2ab(str) {
      const buf = new ArrayBuffer(str.length);
      const bufView = new Uint8Array(buf);
      for (let i = 0, strLen = str.length; i < strLen; i++) {
        bufView[i] = str.charCodeAt(i);
      }
      return buf;
    }

    await window.crypto.subtle.importKey(
      "spki",
      getSpkiDer(key),
      {
        name: "RSA-OAEP",
        hash: "SHA-256",
      },
      true,
      ["encrypt"]
    );
  } catch (e) {
    return null;
  }
}
