import React, { useState } from 'react';
import { ErrorMessage } from './ErrorMessage';
import { BACKEND_URL } from '../App'
import { ImportingAndGeneratingKeys } from './ImportingAndGeneratingKeys';

export function CreateInvoice({ userId }) {

    const [privateKey, setPrivateKey] = useState();
    const [error, setError] = useState("")

    const handleFileChange = async (file) => {
        try {
            const fileReader = new FileReader();
            fileReader.onloadend = (e) => {
                const content = fileReader.result;
                setPrivateKey(content)
            }
            fileReader.readAsText(file)
        }
        catch (e) {
            // console.log("No file provided")
            setPrivateKey(null)
        }
    }

    const handleSubmit = async (e) => {
        e.preventDefault();

        setError("")

        if (!privateKey) {
            setError("No private key provided !")
            return
        }

        const body = {
            dinerId: 1,
            foodName: 'Suche rozky',
            price: 5.50,
            userId
        }

        const data = await fetch(`${BACKEND_URL}/api/invoices`, {
            method: "POST",
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify(body)
        })

        const response = await data.json()


        const inp = document.getElementById('importPrivateKeyInput')
        inp.value = ""

        if (response.error) {
            setError(response.error)
        }

        if (response.bytes) {
            const bytes = response.bytes

            var nonceCiphertextTag = base64ToArrayBuffer(bytes);
            nonceCiphertextTag = new Uint8Array(nonceCiphertextTag);

            const keyData = privateKey.match(/-----BEGIN [A-Z ]+-----([\s\S]+)-----END [A-Z ]+-----/);
            const out = keyData ? keyData[1].trim() : null;

            const binaryString = atob(out);
            const newBytes = new Uint8Array(binaryString.length);
            for (let i = 0; i < binaryString.length; i++) {
                newBytes[i] = binaryString.charCodeAt(i);
            }

            let parsedPrivateKey = null
            try {
                parsedPrivateKey = await window.crypto.subtle.importKey(
                    'pkcs8',
                    newBytes,
                    { name: 'RSA-OAEP', hash: 'SHA-256' },
                    true,
                    ['decrypt']
                );
            }
            catch (e) {
                setError("Unable to import key.")
                return
            }

            var decrypted = await decrypt(nonceCiphertextTag, parsedPrivateKey);

            if (decrypted == null) {
                setError("Unable to decrypt, bad private key provided.")
                return;
            }

            createAndDownloadPDFFromBytes(decrypted)
        }

    };

    return (
        <>
            <h2>Create invoice</h2>

            <ImportingAndGeneratingKeys userId={userId} />

            <br />
            <br />


            <div style={{ border: '1px solid black', padding: '10px' }}>
                <ErrorMessage
                    condition={error.length !== 0}
                    message={<> {error.length !== 0 && error}</>}
                />

                <div>
                    <label>Import pem file with your private key</label> <br />
                    <input id="importPrivateKeyInput" type='file' accept='.pem' onChange={e => handleFileChange(e.target.files[0])} />
                </div>

                <br />

                <form>
                    <button
                        onClick={handleSubmit}
                        className="btn"
                        type="submit"
                    >
                        Create invoice
                    </button>
                </form>

            </div>
        </>
    )
}

function createAndDownloadPDFFromBytes(pdfBytes) {
    const blob = new Blob([pdfBytes], { type: 'application/pdf' });
    const url = URL.createObjectURL(blob);

    const a = document.createElement('a');
    a.href = url;
    a.download = 'decryptedInvoice.pdf';
    a.click();

    URL.revokeObjectURL(url);
}

async function decrypt(nonceCiphertextTag, privateKey) {

    var nonce = nonceCiphertextTag.subarray(0, 16);
    const encryptedKey = nonceCiphertextTag.subarray(16, 272)
    var ciphertextTag = nonceCiphertextTag.subarray(272);
    var aesKeyBuffer = null;

    try {
        aesKeyBuffer = await crypto.subtle.decrypt(
            { name: 'RSA-OAEP' },
            privateKey,
            encryptedKey
        );
    } catch (error) {
        return null;
    }

    let aesKey = null;

    try {
        aesKey = await window.crypto.subtle.importKey('raw', aesKeyBuffer, 'AES-GCM', true, ['encrypt', 'decrypt']);
    }
    catch (e) {
        return null;
    }


    try {
        var decrypted = await crypto.subtle.decrypt({ name: 'AES-GCM', iv: nonce }, aesKey, ciphertextTag);
    } catch (error) {
        return null;
    }

    return decrypted
}

function base64ToArrayBuffer(base64) {
    var binary_string = window.atob(base64);
    var len = binary_string.length;
    var bytes = new Uint8Array(len);
    for (var i = 0; i < len; i++) {
        bytes[i] = binary_string.charCodeAt(i);
    }
    return bytes.buffer;
}