import React, { useState } from "react"
import { hashPassword, keyPairs } from "./KeyPairs";
import { BACKEND_URL } from "../App";
import { ErrorMessage } from "./ErrorMessage";

export function ImportingAndGeneratingKeys({ userId }) {

    const [publicKey, setPublicKey] = useState(null)
    const [errorContent, setErrorContent] = useState("")
    const [successContent, setSuccessContent] = useState("")

    const [password, setPassword] = useState("");


    const handleFileChange = async (file) => {
        try {
            const fileReader = new FileReader();
            fileReader.onloadend = (e) => {
                const content = fileReader.result;
                setPublicKey(content)
            }
            fileReader.readAsText(file)
        }
        catch (e) {
            // console.log("No file provided")
            setPublicKey(null)
        }
    }

    const submitPublicKey = async (pubKey, from) => {

        setErrorContent("")
        setSuccessContent("")

        let o;

        if(from === "form")
            o = await validateImportedKey(pubKey)

        if (o === null && from === "form") {
            setErrorContent("Invalid or unsupported public key.")
            const inp = document.getElementById('importPublicKeyInput')
            inp.value = ""
            setPassword("")
            setPublicKey(null)
            return
        }

        if (password === '' || password.trim().length === 0) {
            setErrorContent("You need to fill in password.")
            return null;
        }

        let hashedPassword = null;

        try {
            hashedPassword = await hashPassword(password)
        }
        catch (e) {
            setErrorContent("Unable to hash password.")
            return
        }

        const body = {
            userId,
            publicKey: pubKey,
            password: hashedPassword
        }

        const data = await fetch(`${BACKEND_URL}/api/user/publicKey`, {
            method: "POST",
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify(body)
        })

        const response = await data.json()

        setPassword("")
        setPublicKey(null)

        const inp = document.getElementById('importPublicKeyInput')
        inp.value = ""

        if (response.error) {
            setErrorContent(response.error)
            return null
        }

        if (response.status) {
            setSuccessContent(response.status)
            return
        }
    }

    const generateKeyPairs = async () => {
        const { publicKey: pk, privateKey } = await keyPairs()

        const out = await submitPublicKey(pk, "func")

        if (out !== null) {
            downloadPrivateKey(privateKey)
        }

    }

    const handlePassword = (e) => {
        setPassword(e.target.value);
    };

    return <div style={{ border: '1px solid black', padding: '10px' }}>

        <ErrorMessage
            condition={errorContent.length !== 0 || successContent.length !== 0}
            message={<>
                {errorContent.length !== 0 && errorContent}
                {successContent.length !== 0 && <span style={{ color: 'green' }}>{successContent}</span>}
            </>} />

        <label className="label"><b>Password: </b></label>
        <input
            onChange={handlePassword}
            className="input"
            value={password}
            type="password"
        />
        <br /><br />


        <div>
            <label>Import pem file with public key</label> <br />
            <input id="importPublicKeyInput" type='file' accept='.pem' onChange={e => { handleFileChange(e.target.files[0]) }} />
        </div>


        <br />

        <button onClick={() => submitPublicKey(publicKey, "form")}>Submit key</button>
        <h3>OR</h3>

        <button onClick={() => generateKeyPairs()}>Generate key pairs</button>

    </div>
}


function downloadPrivateKey(privateKey) {
    const file = new Blob([privateKey], { type: 'text/plain' });

    const url = URL.createObjectURL(file);

    const a = document.createElement('a');
    a.href = url;
    a.download = 'privateKey.pem';
    a.click();

    URL.revokeObjectURL(url);
}


async function validateImportedKey(key) {
    try {

        function getSpkiDer(spkiPem) {
            const pemHeader = "-----BEGIN PUBLIC KEY-----";
            const pemFooter = "-----END PUBLIC KEY-----";
            var pemContents = spkiPem.substring(pemHeader.length, spkiPem.length - pemFooter.length);
            var binaryDerString = window.atob(pemContents);
            return str2ab(binaryDerString);
        }

        function str2ab(str) {
            const buf = new ArrayBuffer(str.length);
            const bufView = new Uint8Array(buf);
            for (let i = 0, strLen = str.length; i < strLen; i++) {
                bufView[i] = str.charCodeAt(i);
            }
            return buf;
        }


        await window.crypto.subtle.importKey(
            "spki",
            getSpkiDer(key),
            {
                name: "RSA-OAEP",
                hash: "SHA-256",
            },
            true,
            ["encrypt"]
        );

    }
    catch (e) {
        return null;
    }
}


