import React, { useState } from 'react';
import { hashPassword } from './KeyPairs';
import { ErrorMessage } from './ErrorMessage';
import { BACKEND_URL } from '../App';

export function Register({ user, setUser }) {
	const [firstName, setFirstName] = useState("");
	const [lastName, setLastName] = useState("");
	const [email, setEmail] = useState("");
	const [password, setPassword] = useState("");

	const [submitted, setSubmitted] = useState(false);
	const [passwordError, setPasswordError] = useState(false);
	const [alreadyExists, setAlreadyExists] = useState("")
	const [emailError, setEmailError] = useState(false)

	const [errorContent, setErrorContent] = useState("")

	const handleFirstName = (e) => {
		setFirstName(e.target.value);
		setSubmitted(false);
	};

	const handleLastName = (e) => {
		setLastName(e.target.value);
		setSubmitted(false);
	};

	const handleEmail = (e) => {
		setEmail(e.target.value);
		setSubmitted(false);
	};

	const handlePassword = (e) => {
		setPassword(e.target.value);
		setSubmitted(false);
	};

	const handleSubmit = async (e) => {
		e.preventDefault();

		setPasswordError(false)
		setAlreadyExists(false)
		setSubmitted(false)
		setEmailError(false)
		setErrorContent("")

		if (firstName === '' || lastName === '' || email === '' || password === '') {
			setErrorContent("You need to fill in this form.")
			return
		}

		const nameRegex = /^[a-zA-ZáäčďéíľĺňóôŕšťúýžÁÄČĎÉÍĽĹŇÓÔŔŠŤÚÝŽ -]+$/

		if(!nameRegex.test(firstName) || !nameRegex.test(lastName)) {
			setErrorContent("Name contains inappropriate characters.")
			return
		}

		const emailRegex = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}$/i;

		if (!emailRegex.test(email)) {
			setEmailError(true)
			return
		}

		const regex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$/;

		if (!regex.test(password)) {
			setPasswordError(true);
			return
		}

		let hashedPassword = null;

		try {
			hashedPassword = await hashPassword(password)
		}
		catch (e) {
			setErrorContent("Unable to hash password.")
			return
		}

		const data = await fetch(`${BACKEND_URL}/api/users`, {
			method: "POST",
			headers: { "Content-Type": "application/json" },
			body: JSON.stringify({ firstName, lastName, email, password: hashedPassword })
		})

		const response = await data.json()

		if (response.error) {
			setErrorContent(response.error)
			return
		}

		if (response.user) {
			localStorage.setItem("user", JSON.stringify(response.user))
			setUser(response.user)
		}

		setSubmitted(true);
		setPasswordError(false);
		setEmailError(false)

		setFirstName("");
		setLastName("");
		setEmail("");
		setPassword("");
	};

	const successMessage = () => {
		return !alreadyExists && (
			<div
				className="success"
				style={{
					display: submitted ? '' : 'none',
					color: 'green'
				}}>
				User {firstName} {lastName} successfully registered!
			</div>
		);
	};

	const emailErrorMessage = () => {
		return (
			<div
				className="error"
				style={{
					display: emailError ? '' : 'none',
				}}>
				Please enter a valid email address.
			</div>
		);
	};

	const passwordErrorMessage = () => {
		return <div className="error" style={{ display: passwordError ? '' : 'none' }}>
			Password should contain at least one lower case letter,
			one upper case letter, number, symbol and should be at least 8 characters long!
		</div>
	}

	return (
		<div className="form">
			<div>
				<h1>Registration</h1>
			</div>

			{!user ? <div>

				<ErrorMessage
					condition={(!alreadyExists && submitted) || emailError || passwordError || errorContent.length !== 0}
					message={<>
						{successMessage()}
						{passwordErrorMessage()}
						{emailErrorMessage()}
						{alreadyExists !== "" && alreadyExists}
						{errorContent && errorContent}
					</>} />

				<form>
					<label className="label">First name: </label>
					<input
						onChange={handleFirstName}
						className="input"
						value={firstName}
						type="text"
					/>

					<br /><br />

					<label className="label">Last name: </label>
					<input
						onChange={handleLastName}
						className="input"
						value={lastName}
						type="text"
					/>
					<br /><br />

					<label className="label">Email: </label>
					<input
						onChange={handleEmail}
						className="input"
						value={email}
						type="email"
					/>
					<br /><br />

					<label className="label">Password: </label>
					<input
						onChange={handlePassword}
						className="input"
						value={password}
						type="password"
					/>
					<br /><br />

					<button
						onClick={handleSubmit}
						className="btn"
						type="submit"
					>
						Submit
					</button>
				</form>
			</div>
				:
				<span>You are logged in.</span>
			}

		</div>
	);
}
