const express = require('express');
const route = express.Router();
const bodyParser = require("body-parser");
const { User } = require("../models");
const { createHash } = require('crypto');
const { createUser } = require('../controllers/createUser');
const readline = require('readline');
const fs = require('fs')
const crypto = require('crypto');
const generateSalt = require('../commands/generateSalt');


route.use(bodyParser.json());

// route.get('/users', async (req, res) => {
//     try {
//         // Query the database to get all users
//         const users = await User.findAll();

//         // Send the list of users as a JSON response
//         res.json(users);
//     } catch (error) {
//         res.status(500).json({ error: 'Internal Server Error' });
//     }
// });

route.get('/users/:id', async (req, res) => {
    try {
        // Extract the user ID from the request parameters
        const userId = req.params.id;

        // Query the database to find the user by ID
        const user = await User.findByPk(userId);

        // Check if the user exists
        if (!user) {
            return res.status(404).json({ error: 'Používateľ neexistuje.' });
        }

        // Send the user information as a JSON response
        return res.json({
            id: user.id,
            email: user.email,
            firstName: user.firstName,
            lastName: user.lastName,
            publicKey: user.publicKey != null
        });
    } catch (error) {
        return res.status(500).json({ error: 'Internal Server Error' });
    }
});

const loginAttempts = {};

// Time delay after 3 logins
route.use('/users/login', (req, res, next) => {
    const { email } = req.body;

    if (loginAttempts[email] >= 3) {
        res.status(429).json({ status: "Too many login attempts, cooldown." })
        setTimeout(() => {
            loginAttempts[email] = 0;
        }, 5000); // 5 seconds
    } else {
        next();
    }
});

route.post('/users/login', async (req, res) => {
    const { email, password } = req.body;

    if (typeof email !== 'string' || typeof password !== 'string' || !email || !password || email.trim().length === 0 || password.trim().length === 0) {
        return res.status(400).json({ error: "Zlé prihlasovacie údaje." }); // Send a 400 Bad Request response
    }

    let decryptedPassword
    try {
        decryptedPassword = decryptHashedPassword(password)
    }
    catch (e) {
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }

    const user = await User.findOne({ where: { email }, attributes: ['id', 'email', 'password', 'salt', 'firstName', 'lastName'] })

    if (user == null) {
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }

    const passhash = createHash("sha256")
        .update(decryptedPassword)
        .update(createHash("sha256").update(user.salt, "utf8").digest("hex"))
        .digest("hex");

    if (!user || (user && user.password !== passhash)) {
        loginAttempts[email] = (loginAttempts[email] || 0) + 1;
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }

    return res.json({ user: { id: user.id, email: user.email, firstName: user.firstName, lastName: user.lastName } })
})

route.post('/user/publicKey', async (req, res) => {

    const { userId, publicKey, password } = req.body

    const error = "Neboli poskytnuté požadované dáta."


    if (typeof userId !== 'string' || typeof publicKey !== 'string' || typeof password !== 'string'
        || userId == null || publicKey == null || password == null
        || publicKey.trim().length === 0 || userId.trim().length === 0 || password.trim().length === 0) {
        return res.json({ error })
    }


    let decryptedPassword
    try {
        decryptedPassword = decryptHashedPassword(password)
    }
    catch (e) {
        return res.json({ error: "Zadané heslo je v zlom formáte." })
    }


    let user = null

    try {
        user = await User.findOne({ where: { id: userId } })
    }
    catch (e) {
        return res.json({ error })
    }

    if (user == null) {
        return res.json({ error })
    }


    const passhash = createHash("sha256")
        .update(decryptedPassword)
        .update(createHash("sha256").update(user.salt, "utf8").digest("hex"))
        .digest("hex");

    if (!user || (user && user.password !== passhash)) {
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }



    try {
        await user.update({
            publicKey
        })
    }
    catch (e) {
        return res.json({ error: "Verejný kľúč sa nepodarilo upraviť." })
    }


    return res.json({ status: "Verejný kľúč bol úspešne upravený." })
})

route.post('/users', async (req, res) => {

    const { firstName, lastName, email, password } = req.body;

    // Check if the required fields are present
    if (!firstName || !lastName || !email || !password
        || typeof firstName !== 'string' || typeof lastName !== 'string' || typeof email !== 'string' || typeof password !== 'string'
        || (firstName.trim().length === 0 || lastName.trim().length === 0 || email.trim().length === 0 || password.trim().length === 0)) {
        return res.status(400).json({ error: "Zlé prihlasovacie údaje." }); // Send a 400 Bad Request response
    }

    const nameRegex = /^[a-zA-ZáäčďéíľĺňóôŕšťúýžÁÄČĎÉÍĽĹŇÓÔŔŠŤÚÝŽ -]+$/

    if (!nameRegex.test(firstName) || !nameRegex.test(lastName)) {
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }

    let userExists = null;

    try {
        userExists = await User.findOne({ where: { email } })
    }
    catch (e) {
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }


    if (userExists) {
        return res.json({ error: "Používateľ s danou emailovou adresou už existuje." })
    }

    let decryptedPassword
    try {
        decryptedPassword = decryptHashedPassword(password)
    }
    catch (e) {
        return res.json({ error: "Zlé prihlasovacie údaje." })
    }

    // check if password is from dictionary
    if (isPasswordInDictionary(decryptedPassword)) {
        return res.json({ error: "Vaše heslo je veľmi slabé." })
    }

    const regex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[@$!%*?&])[A-Za-z\d@$!%*?&]{8,}$/;

    if (!regex.test(decryptedPassword)) {
        return res.json({ error: "Heslo nespĺňa požadovaný formát." });
    }

    const salt = generateSalt()

    const passhash = createHash("sha256")
        .update(decryptedPassword.toString('utf-8'))
        .update(createHash("sha256").update(salt, "utf8").digest("hex"))
        .digest("hex");


    let user = null

    try {
        user = await createUser({
            firstName: firstName.trim(),
            lastName: lastName.trim(),
            email: email.trim(),
            password: passhash,
            salt
        })
    }
    catch (e) {
        return res.json({ error: "Nepodarilo sa vytvoriť používateľa." })
    }


    if (user) {
        return res.json({ user: { id: user.id, email: user.email, firstName: user.firstName, lastName: user.lastName } })
    }
    else {
        return res.status(500).json({ error: "Internal Server Error" });
    }
})

// route.put('/users/:id', async (req, res) => {
//     try {
//         // Extract the user ID from the request parameters
//         const userId = req.params.id;

//         // Check if the user exists
//         const user = await User.findByPk(userId);
//         if (!user) {
//             return res.status(404).json({ error: 'User not found' });
//         }

//         // Extract the updated user data from the request body
//         const { firstName, lastName, age } = req.body;

//         // Update the user's data in the database
//         await user.update({
//             firstName,
//             lastName,
//             age,
//         });

//         // Send the updated user as a JSON response
//         res.json(user);
//     } catch (error) {
//         res.status(500).json({ error: 'Internal Server Error' });
//     }
// });

// route.delete('/users/:id', async (req, res) => {
//     try {
//         // Extract the user ID from the request parameters
//         const userId = req.params.id;

//         // Check if the user exists
//         const user = await User.findByPk(userId);
//         if (!user) {
//             return res.status(404).json({ error: 'User not found' });
//         }

//         // Delete the user from the database
//         await user.destroy();

//         // Send a success response
//         res.json({ message: 'User deleted successfully' });
//     } catch (error) {
//         res.status(500).json({ error: 'Internal Server Error' });
//     }
// });

module.exports = route;



function isPasswordInDictionary(password) {
    const fileStream = fs.createReadStream('/usr/src/app/10-million-password-list-top-100000.txt');
    const rl = readline.createInterface({
        input: fileStream,
        crlfDelay: Infinity, // Read lines until a '\n' character is encountered
    });

    let found = false;

    rl.on('line', (line) => {
        if (line === password) {
            found = true;
            rl.close(); // Stop reading the file once a match is found
        }
    });

    rl.on('close', () => {
        if (found) {
            console.log('String found in the file.');
        } else {
            console.log('String not found in the file.');
        }
    });

    return found
}


function decryptHashedPassword(password) {

    const binaryString = atob(password);
    const newBytes = new Uint8Array(binaryString.length);
    for (let i = 0; i < binaryString.length; i++) {
        newBytes[i] = binaryString.charCodeAt(i);
    }

    const privateKey = crypto.createPrivateKey(process.env.PRIVATE_KEY)

    // Decrypt the data
    const decryptedPassword = crypto.privateDecrypt(
        {
            key: privateKey,
            padding: crypto.constants.RSA_PKCS1_OAEP_PADDING,
            oaepHash: 'sha256', // Specify the OAEP hash algorithm
        },
        newBytes
    );

    return decryptedPassword
}